Package: MorphoRegions
Type: Package
Title: Analysis of Regionalization Patterns in Serially Homologous
        Structures
Version: 0.1.0
Authors@R: c(
  person("Amandine", "Gillet", role = c("cre", "aut"),
         email = "gillet.aman@gmail.com", 
         comment = c(ORCID = "0000-0001-9763-6061")),
  person("Katrina", "Jones", role = c("aut"), 
         email = "katrina.jones@manchester.ac.uk",
	       comment = c(ORCID="0000-0003-1088-0497")),
  person("Noah", "Greifer", role = c("aut"), 
         email = "ngreifer@iq.harvard.edu",
	       comment = c(ORCID="0000-0003-3067-7154")),
  person("Stephanie", "Pierce", role = c("aut"),
         email = "spierce@oeb.harvard.edu", 
         comment = c(ORCID = "0000-0003-0717-1841"))
  )
Description: Computes the optimal number of regions (or subdivisions) and 
    their position in serial structures without a priori assumptions and to visualize 
    the results. After reducing data dimensionality with the built-in function for
    data ordination, regions are fitted as segmented linear regressions
    along the serial structure. Every region boundary
    position and increasing number of regions are iteratively fitted and
    the best model (number of regions and boundary positions) is selected
    with an information criterion. This package expands on the previous
    'regions' package (Jones et al., Science 2018) with improved
    computation and more fitting and plotting options.
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 4.0.0)
Imports: stats, parallel, utils, grDevices, RColorBrewer (>= 1.1-3),
        cluster (>= 2.1.4), scales (>= 1.3.0), ggplot2 (>= 3.5.1), chk
        (>= 0.9.0), pbapply (>= 1.7-2)
Suggests: viridisLite, patchwork (>= 1.1.0), knitr, rmarkdown
VignetteBuilder: knitr
LazyData: true
RoxygenNote: 7.3.2
URL: https://aagillet.github.io/MorphoRegions/,
        https://github.com/aagillet/MorphoRegions
BugReports: https://github.com/aagillet/MorphoRegions/issues
NeedsCompilation: no
Packaged: 2024-08-16 09:17:48 UTC; Amandine Gillet
Author: Amandine Gillet [cre, aut] (<https://orcid.org/0000-0001-9763-6061>),
  Katrina Jones [aut] (<https://orcid.org/0000-0003-1088-0497>),
  Noah Greifer [aut] (<https://orcid.org/0000-0003-3067-7154>),
  Stephanie Pierce [aut] (<https://orcid.org/0000-0003-0717-1841>)
Maintainer: Amandine Gillet <gillet.aman@gmail.com>
Repository: CRAN
Date/Publication: 2024-08-21 12:40:06 UTC
Built: R 4.6.0; ; 2026-04-26 08:56:21 UTC; unix

Package: MorphoRegions
Type: Package
Title: Analysis of Regionalization Patterns in Serially Homologous
        Structures
Version: 0.2.0
Date: 2026-07-17
Authors@R: c(
  person("Amandine", "Gillet", role = c("cre", "aut"),
         email = "gillet.aman@gmail.com", 
         comment = c(ORCID = "0000-0001-9763-6061")),
  person("Katrina", "Jones", role = c("aut"), 
         email = "katrina.jones@bristol.ac.uk",
	       comment = c(ORCID="0000-0003-1088-0497")),
  person("Noah", "Greifer", role = c("aut"), 
         email = "ngreifer@iq.harvard.edu",
	       comment = c(ORCID="0000-0003-3067-7154")),
  person("Stephanie", "Pierce", role = c("aut"),
         email = "spierce@oeb.harvard.edu", 
         comment = c(ORCID = "0000-0003-0717-1841"))
  )
Description: Computes the optimal number of regions (or subdivisions) and 
    their position in serial structures without a priori assumptions and to visualize 
    the results. After reducing data dimensionality with the built-in function for
    data ordination, regions are fitted as segmented linear regressions
    along the serial structure. Every region boundary
    position and increasing number of regions are iteratively fitted and
    the best model (number of regions and boundary positions) is selected
    with an information criterion. This package expands on the previous
    'regions' package (Jones et al. (2018) <doi:10.1126/science.aar3126>) with improved
    computation and more fitting and plotting options.
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 4.0.0)
Imports: stats, parallel, utils, grDevices, RColorBrewer (>= 1.1-3),
        cluster (>= 2.1.4), scales (>= 1.3.0), ggplot2 (>= 4.0.0), arg
        (>= 0.1.1), pbapply (>= 1.7-2)
Suggests: viridisLite, patchwork (>= 1.1.0), knitr, rmarkdown
VignetteBuilder: knitr
LazyData: true
URL: https://aagillet.github.io/MorphoRegions/,
        https://github.com/aagillet/MorphoRegions
BugReports: https://github.com/aagillet/MorphoRegions/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-17 11:11:10 UTC; wc25184
Author: Amandine Gillet [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-9763-6061>),
  Katrina Jones [aut] (ORCID: <https://orcid.org/0000-0003-1088-0497>),
  Noah Greifer [aut] (ORCID: <https://orcid.org/0000-0003-3067-7154>),
  Stephanie Pierce [aut] (ORCID: <https://orcid.org/0000-0003-0717-1841>)
Maintainer: Amandine Gillet <gillet.aman@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-17 12:40:13 UTC
Built: R 4.5.2; ; 2026-07-17 16:49:45 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(modelperf,regions_pco)
S3method(modelperf,regions_results_single)
S3method(modelperf,regions_sim)
S3method(plot,regions_pco)
S3method(plot,regions_pco_load)
S3method(plot,regions_pco_select)
S3method(plot,regions_results_single)
S3method(plot,regions_sim)
S3method(plotsegreg,regions_pco)
S3method(plotsegreg,regions_results_single)
S3method(plotsegreg,regions_sim)
S3method(print,regions_BPvar)
S3method(print,regions_modelselect)
S3method(print,regions_modelsupport)
S3method(print,regions_pco)
S3method(print,regions_pco_load)
S3method(print,regions_pco_select)
S3method(print,regions_perf)
S3method(print,regions_results)
S3method(print,regions_results_single)
S3method(print,regions_sim)
S3method(print,summary.regions_pco_select)
S3method(print,summary.regions_results)
S3method(summary,regions_pco_select)
S3method(summary,regions_results)
export(PCOload)
export(PCOselect)
export(addregions)
export(calcBPvar)
export(calcmodel)
export(calcregions)
export(modelperf)
export(modelselect)
export(modelsupport)
export(ncombos)
export(plotsegreg)
export(plotvertmap)
export(process_PC)
export(process_gmPC)
export(process_measurements)
export(simregions)
export(subsample)
export(svdPCO)
import(ggplot2)
import(stats)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# *MorphoRegions*: Analysis of Regionalization Patterns in Serially Homologous Structures

<!-- badges: start -->

<!-- badges: end -->

*MorphoRegions* is an R package built to computationally identify
regions (morphological, functional, etc.) in serially homologous
structures such as, but not limited to, the vertebrate backbone. Regions
are modeled as segmented linear regressions with each segment
corresponding to a region and region boundaries (or breakpoints)
corresponding to changes along the serially homologous structure. The
optimal number of regions and their breakpoint positions are identified
using maximum-likelihood methods without *a priori* assumptions.

Since version 0.2.0, *MorphoRegions* can now be applied to both
**traditional morphometric** and **geometric morphometric** data,
providing a unified workflow for identifying regionalization patterns in
serially homologous structures.

This package was first presented in [Gillet et
al. (2024)](https://doi.org/10.1038/s41467-024-51963-w) and is an
updated version of the [*regions* R
package](https://github.com/katrinajones/regions) from [Jones et
al. (2018)](https://www.science.org/doi/abs/10.1126/science.aar3126)
with improved computational methods and expanded fitting and plotting
options.

## Installing *MorphoRegions*

You can install the released version of *MorphoRegions* from
[CRAN](https://CRAN.R-project.org/package=MorphoRegions) with:

``` r
install.packages("MorphoRegions")
```

Or the development version on
[GitHub](https://github.com/AaGillet/MorphoRegions) with:

``` r
# install.packages("pak")
pak::pak("AaGillet/MorphoRegions")
```

## Example

The following example illustrates the basic steps to prepare the data,
fit regionalization models, select the best model, and plot the results.
See `vignette("MorphoRegions")` or the [*MorphoRegions*
website](https://aagillet.github.io/MorphoRegions/) for a detailed guide
of the package and its functionalities.

``` r
library(MorphoRegions)
```

#### Preparing the data

##### Traditional morphometrics

Data should be provided as a data frame where each row is an element of
the serially homologous structure (e.g., a vertebra). One column should
contain positional information of each element (e.g., vertebral number)
and other columns should contain variables that will be used to
calculate regions (e.g., morphological measurements). The `dolphin`
dataset contains vertebral measurements of a dolphin with the positional
information (vertebral number) in the first column.

``` r
data("dolphin")
```

|     | Vertebra |   Lc |   Wc |   Hc |  Hnp |  Wnp |  Inp |   Ha |   Wa |   Lm |   Wm |  Hm | Hch | Wch |  Ltp |  Wtp |  Itp |
|:----|---------:|-----:|-----:|-----:|-----:|-----:|-----:|-----:|-----:|-----:|-----:|----:|----:|----:|-----:|-----:|-----:|
| 8   |        8 | 1.33 | 3.37 | 2.02 | 2.85 | 1.17 | 2.01 | 1.72 | 1.48 | 0.00 | 0.00 | 0.0 |   0 |   0 | 1.71 | 1.67 | 1.57 |
| 9   |        9 | 1.46 | 3.67 | 2.10 | 3.20 | 1.63 | 2.01 | 1.44 | 1.65 | 0.00 | 0.00 | 0.0 |   0 |   0 | 1.51 | 1.61 | 1.57 |
| 10  |       10 | 1.57 | 3.62 | 2.26 | 3.13 | 1.71 | 2.01 | 1.42 | 2.18 | 0.00 | 0.00 | 0.0 |   0 |   0 | 1.06 | 1.90 | 1.57 |
| 11  |       11 | 1.71 | 3.75 | 2.24 | 3.07 | 1.71 | 2.01 | 1.38 | 1.25 | 0.56 | 0.38 | 1.7 |   0 |   0 | 1.03 | 1.91 | 1.66 |
| 12  |       12 | 1.74 | 3.72 | 2.28 | 2.66 | 1.96 | 1.99 | 1.30 | 1.50 | 1.45 | 1.09 | 2.0 |   0 |   0 | 0.60 | 1.71 | 1.57 |
| 13  |       13 | 1.82 | 3.92 | 2.28 | 2.61 | 1.74 | 1.88 | 1.29 | 1.74 | 1.86 | 1.12 | 2.0 |   0 |   0 | 0.37 | 1.44 | 1.57 |

Prior to analysis, data must be processed into an object usable by
*MorphoRegions* using `process_measurements()`. The `pos` argument is
used to specify the name or index of the column containing positional
information and the `fillNA` argument allows to fill missing values in
the dataset (up to two successive elements).

``` r
dolphin_data <- process_measurements(dolphin, pos = 1)
class(dolphin_data)
#> [1] "regions_data"
```

Data are then ordinated using a Principal Coordinates Analysis (PCO) to
reduce dimensionality and allow the combination of a variety of data
types. The number of PCOs to retain for analyses can be selected using
`PCOselect()` (see the vignette for different methods of PCO axes
selection).

``` r
dolphin_pco <- svdPCO(dolphin_data, metric = "gower")

# Select PCOs with variance > 0.05 :
PCOs <- PCOselect(dolphin_pco, method = "variance",
                  cutoff = .05)
PCOs
#> A `regions_pco_select` object
#> - PC scores selected: 1, 2
#> - Method: variance (cutoff: 0.05)
```

##### Geometric morphometrics and ordinated data

Alternatively to computing PCO scores in *MorphoRegions*, externally
computed PC scores can be provided. Use `process_PC()` for traditional
morphometric data or `process_gmPC()` for 2D or 3D geometric
morphometric data.

The `seal_gmm` dataset contains coordinates of Procrustes aligned 3D
landmarks taken on presacral vertebrae, PC scores of each vertebra, and
eigenvalues of each PC axis.

``` r
data("seal_gmm")
seal_pca <- process_gmPC(data = seal_gmm$coord_gpa,
                         pcscores = seal_gmm$scores,
                         eigenvals = seal_gmm$eigenvals)

# Select PCs with variance > 0.05 :
PCs <- PCOselect(seal_pca, method = "variance",
                  cutoff = .05)
PCs
#> A `regions_pco_select` object
#> - PC scores selected: 1, 2, 3
#> - Method: variance (cutoff: 0.05)
```

See `vignette("MorphoRegions")` or the [*MorphoRegions*
website](https://aagillet.github.io/MorphoRegions/) for complete
workflow, including the use of `process_PC()` for externally computed PC
scores on traditional morphometric datasets.

#### Fitting regressions and selecting the best model

`calcregions()` allows fitting all possible combinations of segmented
linear regressions from 1 region (no breakpoint) to the number of
regions specified in the `noregions` argument. In this example, up to 5
regions (4 breakpoints) will be fitted along the backbone, however,
there is no limit for this value and it is possible to fit as many
regions as you would like. For this example, regions will be fitted with
a minimum of 3 vertebrae per region (`minvert = 3`) and using a
continuous fit (`cont = TRUE`) (see `vignette("MorphoRegions")` or the
[*MorphoRegions* website](https://aagillet.github.io/MorphoRegions/) for
details about fitting options).

``` r
regionresults <- calcregions(dolphin_pco, scores = PCOs, noregions = 5,
                             minvert = 3, cont = TRUE, 
                             exhaus = TRUE)
regionresults
#> A `regions_results` object
#>  - number of PCOs used: 2 
#>  - number of regions: 1, 2, 3, 4, 5 
#>  - model type: continuous 
#>  - min vertebrae per region: 3 
#>  - total models saved: 28810 
#> Use `summary()` to examine summaries of the fitting process.
```

For each given number of regions, the best fit is selected by minimizing
the residual sum of squares (`sumRSS`):

``` r
models <- modelselect(regionresults)
models
#>  Regions BP 1 BP 2 BP 3 BP 4 sumRSS RSS.1 RSS.2
#>        1    .    .    .    .  1.898 1.456 0.441
#>        2   26    .    .    .  0.413 0.105 0.308
#>        3   23   29    .    .  0.147 0.092 0.055
#>        4   23   30   40    .  0.073 0.034 0.040
#>        5   23   27   34   40  0.046 0.026 0.020
```

The best overall model (best number of regions) is then select by
ordering models from the best fit (top row) to the worst fit (last row)
using either the AICc or BIC criterion:

``` r
supp <- modelsupport(models)
supp
#> - Model support (AICc)
#>  Regions BP 1 BP 2 BP 3 BP 4 sumRSS     AICc deltaAIC model_lik Ak_weight
#>        5   23   27   34   40  0.046 -556.036    0.000         1         1
#>        4   23   30   40    .  0.073 -528.096   27.940         0         0
#>        3   23   29    .    .  0.147 -480.952   75.084         0         0
#>        2   26    .    .    .  0.413 -405.787  150.250         0         0
#>        1    .    .    .    .  1.898 -290.769  265.267         0         0
#> Region score: 5 
#> 
#> - Model support (BIC)
#>  Regions BP 1 BP 2 BP 3 BP 4 sumRSS      BIC deltaBIC model_lik BIC_weight
#>        5   23   27   34   40  0.046 -526.559    0.000         1          1
#>        4   23   30   40    .  0.073 -502.645   23.914         0          0
#>        3   23   29    .    .  0.147 -460.321   66.238         0          0
#>        2   26    .    .    .  0.413 -390.668  135.891         0          0
#>        1    .    .    .    .  1.898 -281.774  244.784         0          0
#> Region score: 5
```

Here, for both criteria, the best model is the 5 regions models with
breakpoints at vertebrae 23, 27, 34, and 40. *The breakpoint value
corresponds to the last vertebra included in the region, so the first
region here is made of vertebrae 8 to 23 included and the second region
is made of vertebrae 24 to 27.* The function also returns the **region
score**, a continuous value reflecting the level of regionalization
while accounting for uncertainty in the best number of regions (see
`vignette("MorphoRegions")` or the [*MorphoRegions*
website](https://aagillet.github.io/MorphoRegions/) for more details).

#### Plotting results

Results of the best model (or any other model) can be visualized either
as a scatter plot or as a vertebral map.

The **scatter plot** shows the PCO score (here for PCO 1 and 2) of each
vertebra along the backbone (gray dots) and the segmented linear
regressions (cyan line) of the model to plot. Breakpoints are showed by
dotted orange lines.

``` r
plotsegreg(dolphin_pco, scores = 1:2, modelsupport = supp,
           criterion = "bic", model = 1)
```

<img src="man/figures/README-scatterplot-1.png" alt="" width="55%" style="display: block; margin: auto;" />

In the **vertebral map** plot, each vertebra is represented by a
rectangle color-coded according to the region to which it belongs.
Vertebrae not included in the analysis (here vertebrae 1 to 7) are
represented by gray rectangles and can be removed using `dropNA = TRUE`.

``` r
plotvertmap(dolphin_pco, name = "Dolphin", modelsupport = supp, 
            criterion = "bic", model = 1)

plotvertmap(dolphin_pco, name = "Dolphin", modelsupport = supp, 
            criterion = "bic", model = 1, dropNA = TRUE)
```

<img src="man/figures/README-vertebralmap-1.png" alt="" width="80%" style="display: block; margin: auto;" /><img src="man/figures/README-vertebralmap-2.png" alt="" width="80%" style="display: block; margin: auto;" />

The variability around breakpoint positions can be calculated using
`calcBPvar()` and then displayed on the vertebral map. The weighted
average position of each breakpoint is shown by the black dot and the
weighted variance is illustrated by the horizontal black bar.

``` r
bpvar <- calcBPvar(regionresults, noregions = 5,
                   pct = 0.1, criterion = "bic")

plotvertmap(dolphin_pco, name = "Dolphin", 
            dropNA = TRUE, bpvar = bpvar)
```

<img src="man/figures/README-vertebralmapBPvar-1.png" alt="" width="80%" style="display: block; margin: auto;" />

## Citation

To cite *MorphoRegions*, please use:

``` r
citation("MorphoRegions")
```

#' Calculate variability of breakpoints
#'
#' `calcBPvar()` computes an estimate of the variability of the breakpoints for a given number of regions. This involves computing the weighted mean and standard deviation of each breakpoint using Akaike weights.
#'
#' @param regions_results a `regions_results` object; the output of a call to [calcregions()] or [addregions()].
#' @param noregions the number of regions for which the weighted mean and standard deviation are to be computed.
#' @param pct the proportion of best model to keep from the original total number of possible models
#' @param criterion string; the criterion used to compute the weights. Allowable options include `"aic"` and `"bic"`. Abbreviations allowed.
#'
#' @returns A `regions_BPvar` object, which has two components:
#' * `WeightedBP` is a matrix containing the weighted mean and standard deviation of each breakpoint
#' * `BestModels` is a data frame containing the models used to compute the weighted breakpoint statistics and the weights each one is given.
#'
#' @seealso [calcregions()] for fitting segmented regression models to all combinations of breakpoints.
#'
#' @example man/examples/example-calcBPvar.R
#'

#' @export
calcBPvar <- function(regions_results, noregions, pct = .05, criterion = "aic") {
  arg::arg_is(regions_results, "regions_results")

  arg::arg_supplied(noregions)
  arg::arg_count(noregions)
  arg::arg_between(noregions, range(regions_results$stats$Nregions))

  arg::arg_number(pct)
  arg::arg_between(pct, c(0, 1), inclusive = c(FALSE, TRUE))

  criterion <- arg::match_arg(criterion, c("aic", "bic"))

  nmodel <- regions_results$stats$Nmodel_possible[regions_results$stats$Nregions == noregions]

  nvert <- length(attr(regions_results, "pos"))
  cont <- attr(regions_results, "cont")

  # Define number of models to keep to correspond to given pct of models wants to keep
  ntop <- ceiling(nmodel * pct)

  # Get number of PCs on which analysis was run
  noPC <- sum(startsWith(colnames(regions_results$results), "RSS."))

  # Extract models corresponding to the given number of regions
  dat <- regions_results$results[regions_results$results$regions == noregions, , drop = FALSE]

  # Order by increasing sumRSS (from best to worst model)
  dat <- dat[order(dat$sumRSS), , drop = FALSE]

  if (nrow(dat) >= ntop) {
    dat <- dat[seq_len(ntop), , drop = FALSE]
  }
  else {
    pct_calc <- round(nrow(dat) / nmodel * 100, 2L)
    arg::wrn("number of models provided lower than percentage requested. Weighted means and SD calculated on {pct_calc}% of total number of models")
  }

  # Calculate probability (with AICc/BIC) and weight of each model:
  IC <- vapply(dat$sumRSS, function(r) {
    ic <- .AICcalc(r, noPC = noPC, nvert = nvert, noregions = noregions, cont = cont)
    ic[switch(criterion, "aic" = "AICc", "bic" = "BIC")]
  }, numeric(1L))

  deltaIC <- IC - min(IC)	#Calculate AIC difference
  model_lik <- exp(-deltaIC / 2)	#Likelihood of the model

  IC_weight <- model_lik / sum(model_lik)	#Akaike Weights

  # Calculate weighted mean and weighted SD of each BP position using
  # Akaike weights: (formulae from Symonds & Moussalli, Behav Ecol Sociobiol 2011)
  bps <- as.matrix(dat[paste0("breakpoint", seq_len(noregions - 1L))])

  # Weighted mean is the sum of bps values multiplied by the corresponding Akaike weight
  wMean <- colSums(bps * IC_weight)
  wSD <- vapply(seq_len(ncol(bps)), function(i) {
    sqrt(sum(IC_weight * (bps[, i] - wMean[i])^2))
  }, numeric(1L))

  WeightedBP <- rbind(wMean, wSD)

  wt_dat <- data.frame(IC_weight, CumWeight = cumsum(IC_weight))
  names(wt_dat)[1L] <- switch(criterion, "aic" = "AICweight", "bic" = "BICweight")

  out <- list(WeightedBP = WeightedBP, BestModels = cbind(dat, wt_dat))

  attr(out, "pct") <- nrow(dat) / nmodel

  class(out) <- "regions_BPvar"

  out
  # WeightedBP = matrix with row1 = weighted means and row2 = weighted SD
  # BestModels = top pct % models ordered from best to worst with corresponding AIC weight and cumulative weight
}

#' @exportS3Method print regions_BPvar
print.regions_BPvar <- function(x, digits = 3, ...) {
  x0 <- x
  colnames(x$WeightedBP) <- sub("breakpoint", "BP ", colnames(x$WeightedBP), fixed = TRUE)
  x$WeightedBP <- round(x$WeightedBP, digits)

  print(x$WeightedBP)
  cat("- Computed using top ", round(100 * attr(x, "pct"), 2), "% of models\n", sep = "")

  invisible(x0)
}

#' Subsample a dataset
#'
#' `subsample()` creates a smaller version of the original dataset by sampling its rows. Because PCOs should be computed on the full dataset and most other functions take in `regions_pco` objects, `subsample()` requires a `regions_pco` object as its input.
#'
#' @param pco a `regions_pco` object; the output of a call to [svdPCO()].
#' @param sample `numeric`; either the number or proportion of vertebrae to remain the sampled dataset. If `NULL`, the original dataset is returned.
#' @param type string; the type of subsampling to do, either `"seq"` for sampling in sequence or `"random"` for random sampling. Default is `"seq"`. Abbreviations allowed.
#'
#' @return
#' A `regions_pco` object, a subset of the original supplied to `pco`. The original dataset is stored as an attribute, which itself contains the subsampling indices.
#'
#' @seealso [svdPCO()], [process_measurements()], [plotvertmap()] to visualize the vertebral map after subsampling.
#'
#' @example man/examples/example-subsample.R
#'

#' @export
subsample <- function(pco, sample = NULL, type = "seq") {

  arg::arg_supplied(pco)
  arg::arg_is(pco, "regions_pco")

  pos <- .get_pos(pco)

  if (!identical(pos, .get_pos(pco, subset = FALSE))) {
    arg::err("{.fun subsample} cannot be used on a {.cls regions_pco} object after using {.fun subsample} or {.code subset(., drop = FALSE)} on it")
  }

  eligible_vertebrae <- .get_eligible_vertebrae(pco)

  arg::arg_number(sample)
  arg::arg_gt(sample, 0)
  arg::arg_lte(sample, length(eligible_vertebrae))

  type <- arg::match_arg(type, c("seq", "random"))

  if (sample <= 1) {
    sample <- round(sample * length(eligible_vertebrae))
  }

  sampled_pos_ind <- switch(type,
                            "seq" = round(seq(1, length(eligible_vertebrae), length.out = sample)),
                            "random" = sample(seq_along(eligible_vertebrae), sample))

  pco$scores <- pco$scores[pos %in% eligible_vertebrae[sampled_pos_ind], , drop = FALSE]
  pco$eigen.val <- pco$eigen.val[pos %in% eligible_vertebrae[sampled_pos_ind]]

  attr(attr(pco, "data"), "eligible_vertebrae") <- eligible_vertebrae[sampled_pos_ind]

  pco
}

#' Calculate PCO (principal co-ordinates analysis) based on SVD
#'
#' Calculates distance matrix from raw data, then conducts a PCO ordination using a
#' single value decomposition (SVD). This differs from other PCO functions which use [stats::cmdscale()] and rely on a
#' spectral decomposition.
#'
#' @param x a `regions_data` object; the output of a call to [process_measurements()].
#' @param metric string; the distance matrix calculation metric. Allowable options include those support by [cluster::daisy()], which are `"euclidean"`, `"manhattan"`, or `"gower"`. Default is `"gower"`. Abbreviations allowed.
#' @param scale `logical`; whether to scale the variables prior to including them in the PCO estimation. Default is `TRUE`, which is especially advisable when using the bootstrap to select the number of PCOs to use in downstream analyses. Passed to the `stand` argument of `cluster::daisy()`. Ignored if `metric = "gower"`.
#'
#' @return
#' A `regions_pco` object, which contains eigenvectors in the `scores` component and eigenvalues in the `eigen.val` component. The original dataset is stored in the `data` attribute.
#'
#' @seealso
#' * [plot.regions_pco()] for plotting PCO axes
#' * [cluster::daisy()], which is used to compute the distance matrix used in the calculation
#' * [stats::cmdscale()] for a spectral decomposition-based implementation
#'
#' @example man/examples/example-svdPCO.R
#'

#' @export
svdPCO <- function(x, metric = "gower", scale = TRUE) {
  arg::arg_supplied(x)
  arg::arg_is(x, "regions_data")

  metric <- arg::match_arg(metric, eval(formals(cluster::daisy)[["metric"]]))

  arg::arg_flag(scale)

  dat <- .get_data_without_pos(x)

  if (metric != "gower" && !all(vapply(dat, is.numeric, logical(1L)))) {
    arg::wrn('{.code metric = "{metric}"} cannot be used when non-numeric measurement variables are present. Setting {.arg metric} to {.val gower}')
    metric <- "gower"
  }

  #Set distance metric
  dist <- cluster::daisy(dat, metric = metric, stand = scale)

  out <- .svdPCO_internal(dist)

  attr(out, "data") <- x
  attr(out, "metric") <- metric
  attr(out, "scale") <- scale
  attr(out, "specimen") <- factor(rep(seq_along(x), unlist(lapply(x, nrow))),
                                  labels = paste("Specimen", seq_along(x)),
                                  levels = seq_along(x))

  class(out) <- "regions_pco"

  out
}

.svdPCO_internal <- function(dist, val.only = FALSE) {

  ### PCO analysis based on SVD not eigen as in cmdscale()
  d2 <- as.matrix(dist^2)

  # Double centering code
  n <- nrow(d2)
  k <- ncol(d2)
  r_m <- matrix(rowMeans(d2, na.rm = TRUE), n, k, byrow = TRUE)
  c_m <- matrix(colMeans(d2, na.rm = TRUE), n, k, byrow = FALSE)
  m_m <- mean(d2, na.rm = TRUE)
  step3 <- -0.5 * (d2 - r_m - c_m + m_m)

  if (val.only) {
    #Primarily for use in PCOcutoff(); faster than full SVD
    eigen.val <- sqrt(abs(eigen(tcrossprod(step3), symmetric = TRUE,
                                only.values = TRUE)$values))
    return(list(eigen.val = eigen.val))
  }

  # Single value decomp
  step4 <- svd(step3)
  eigen.val <- step4$d
  step5 <- step4$v

  # Scale by root eigenval
  for (i in seq_len(ncol(step5))) {
    step5[, i] <- step5[, i] * sqrt(step4$d[i])
  }

  eigen.vect <- step5[, -ncol(step5), drop = FALSE]  #remove the last PCO which has no variance

  colnames(eigen.vect) <- paste0("PCO.", seq_len(ncol(eigen.vect)))

  list(scores = eigen.vect,
       eigen.val = eigen.val)
}

#' @exportS3Method print regions_pco
print.regions_pco <- function(x, digits = 3, ...) {
  arg::arg_whole_number(digits)

  cat("- Scores:\n\n")

  print(as.data.frame(x$scores, row.names = rownames(attr(x, "data")))[seq_len(min(nrow(x$scores), 6)), ],
        digits = digits, ...)

  if (nrow(x$scores) > 6) {
    cat(sprintf("(First 6 of %s rows displayed.)\n", nrow(x$scores)))
  }

  cat("\n- Eigenvalues:\n\n")

  print(x$eigen.val, digits = digits, ...)

  invisible(x)
}

# Simulate 40 vertebra, 4 regions (3 breakpoints), 3 PCOs,
# true model R2 of .9, continuous
set.seed(11)
sim <- simregions(nvert = 30, nregions = 4, nvar = 3, r2 = .95,
                  minvert = 3, cont = TRUE)

sim

# Plot the true data-generating lines and breakpoints
plot(sim, scores = 1:3)

# Run segmented regression models on simulated data,
# up to 6 regions
simresults <- calcregions(sim, scores = 1:3, noregions = 6,
                          minvert = 3, cont = TRUE,
                          verbose = FALSE)

summary(simresults)

# Select best model for each number of regions
(simmodels <- modelselect(simresults))

# Evaluate support for each model and rank models
(simsupp <- modelsupport(simmodels))
# AICc supports 3-4 regions

# Evaluate model performance of best model
modelperf(sim, modelsupport = simsupp,
          criterion = "aic", model = 1)
# Second best model (3 regions) does quite well, too
modelperf(sim, modelsupport = simsupp,
          criterion = "aic", model = 2)

#Plot best model fit
plotsegreg(sim, scores = 1:3,
           modelsupport = simsupp,
           criterion = "aic", model = 1)

# Calculate variability of estimate breakpoints for
# 3-region model; high uncertainty for breakpoints
# 1 and 2. Note weighted value for breakpoint 2
# differs from that of best model
bpvar <- calcBPvar(simresults, noregions = 4,
                   pct = .05, criterion = "aic")
bpvar

# Plot estimated vertebral map with variability
plotvertmap(sim, modelsupport = simsupp, model = 1,
            criterion = "aic", text = TRUE)

# True map; pretty close
plotvertmap(sim, bps = c(3, 7, 24),
            text = TRUE)

data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Plot vertebrae before subsetting
plotvertmap(alligator_PCO, text = TRUE)

# Subset data after estimating PCOs
alligator_PCO_subset <- subset(alligator_PCO,
                               Vertebra < 20)

# Plot vertebrae after subsetting; vertebrae outside
# subset are dropped
plotvertmap(alligator_PCO_subset, text = TRUE)

# Subset data after estimating PCOs with `drop = FALSE`
alligator_PCO_subset <- subset(alligator_PCO,
                               Vertebra < 20,
                               drop = FALSE)

# Plot vertebrae after subsetting; vertebrae outside
# subset remain and are treated as missing
plotvertmap(alligator_PCO_subset, text = TRUE)

data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Evaluate model performance (R2) given supplied
# breakpoints for a continuous model
modelperf(alligator_PCO, scores = 1:3,
          bps = c(7, 15, 20), cont = TRUE)

plotsegreg(alligator_PCO, scores = 1:3,
           bps = c(7, 15, 20), cont = TRUE)
## See also `?calcmodel` for use with a single model

# Fit segmented regression models for 1 to 7 regions
# using PCOs 1 to 4 and a continuous model with a
# non-exhaustive search
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 7,
                             minvert = 3,
                             cont = TRUE,
                             exhaus = FALSE,
                             verbose = FALSE)

regionresults

# For each number of regions, identify best
# model based on minimizing RSS
bestresults <- modelselect(regionresults)

# Evaluate support for each model and rank
supp <- modelsupport(bestresults)

# Evaluate model performance (R2) for best model
# as chosen by BIC
modelperf(alligator_PCO, scores = 1:4,
          modelsupport = supp,
          criterion = "bic", model = 1)

# Plot that model for the first PCO score
plotsegreg(alligator_PCO, scores = 1:4,
           modelsupport = supp,
           criterion = "bic", model = 1)

## See `?simregions` for use with simulated data

data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Plot vertebral map with specified breakpoints
plotvertmap(alligator_PCO,
            type = "percent",
            name = "Alligator",
            bps = c(8, 15, 19),
            text = TRUE)

# Fit segmented regression models for 1 to 7 regions
# using PCOs 1 to 4 and a continuous model with a
# exhaustive search
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 7,
                             minvert = 3,
                             cont = TRUE,
                             exhaus = TRUE,
                             verbose = FALSE)

# For each number of regions, identify best
# model based on minimizing RSS
bestresults <- modelselect(regionresults)

# Evaluate support for each model and rank models
supp <- modelsupport(bestresults)

# Plot vertebral map with breakpoints corresponding to
# best segmented regression model as determined by
# AICc
plotvertmap(alligator_PCO,
            type = "percent",
            name = "Alligator",
            modelsupport = supp,
            model = 1,
            criterion = "aic",
            text = TRUE)

# Plot vertebral map with breakpoints corresponding to
# best segmented regression model as determined by
# AICc, using centrum length to size vertebrae
plotvertmap(alligator_PCO,
            name = "Alligator",
            modelsupport = supp,
            model = 1,
            criterion = "aic",
            centraL = "CL",
            text = TRUE)

# Compute Akaike-weighted location and SD of optimal
# breakpoints using top 10% of models with 4 regions
bpvar <- calcBPvar(regionresults, noregions = 5,
                   pct = .1, criterion = "aic")

#Using weighted BPs and SDs from calcBPvar()
plotvertmap(alligator_PCO, name = "Dolphin",
            bpvar = bpvar,
            text = TRUE)

data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data,
                        metric = "gower")

alligator_PCO

# Plot PCOs against vertebra index
plot(alligator_PCO, pco_y = 1:2)

# Plot PCOs against each other
plot(alligator_PCO, pco_y = 1, pco_x = 2)

.AICcalc <- function(RSS, noPC, nvert, noregions, cont) {
  n <- noPC * nvert 	# No of variables used
  var <- RSS / n 	# Variance calculated ML way
  k <- { # Calculating the number of parameters (k) being fitted in the model
    if (cont) {
      # For a continuous fit, a slope is estimated for each region and each PCO,
      # only the intercept of the fist segment is estimated for each PCO since
      # other intercepts are forced by the previous slope and breakpoint
      # position, + a noregions-1 number of breakpoint is estimated
      (noregions * noPC) + noPC + noregions - 1
    }
    else {
      # For discontinuous fit, a slope and an intercept (2 params) are evaluated
      # for region and for each PCO, + a noregions-1 number of breakpoint is estimated
      (2 * noregions * noPC) + noregions - 1
    }
  }

  if (n < k + 2) {
    arg::err("the ratio of variables to parameters is too small. Reduce the number of regions or increase the number of variables")
  }

  AIC <- n * log(var) + (2 * k)
  corr <- (2 * k * (k + 1)) / (n - k - 1)	# Correct for number of parameters and small sample
  AICc <- AIC + corr 	# Calculate AICc
  BIC <- n * log(var) + log(n) * k

  c(AICc = AICc, BIC = BIC)
}

#' Select the best models
#'
#' `modelselect()` narrows down the search for the best model by identifying the best model for each number of regions as determined by its residual sums of squares (RSS).
#'
#' @param results a `regions_results` object; the output of a call to `calcregions()` or `addregions()`.
#' @param scores `numeric`; a vector corresponding to the indices of the PCOs the \eqn{R^2} of which will be used to determine the best model for each number of regions. If `NULL`, the default, all PCOs used included in the fitting will be used.
#'
#' @returns A `regions_modelselect` object, which contains information about the best models for each number of regions extracted from `results`.
#'
#' @seealso [modelsupport()] for computing statistics that describe the support of each model using information criteria; [modelperf()] for computing fit statistics for selected models.
#'
#' @example man/examples/example-modelselect.R
#'

#' @export
modelselect <- function(results, scores = NULL) {

  arg::arg_supplied(results)
  arg::arg_is(results, "regions_results")

  regiondata <- results$results

  noregions <- sort(unique(results$stats$Nregions))

  arg::when_not_null(
    scores,
    arg::arg_whole_numeric,
    arg::arg_between(c(1, sum(startsWith(names(regiondata), "RSS."))))
  )

  if (!is.null(scores)) {
    keep.pcos <- paste0("RSS.", sort(scores))
    regiondata$sumRSS <- rowSums(regiondata[keep.pcos])

    regiondata[startsWith(names(regiondata), "RSS.") & !names(regiondata) %in% keep.pcos] <- NULL
  }

  models <- do.call("rbind", lapply(noregions, function(i) {
    allmodels <- regiondata[regiondata$regions == i, , drop = FALSE]	#select only models with correct region no
    allmodels[which.min(allmodels$sumRSS), , drop = FALSE]
  }))

  attr(models, "cont") <- attr(results, "cont")
  attr(models, "nvert") <- length(attr(results, "pos"))

  class(models) <- c("regions_modelselect", class(models))

  models
}

#' @exportS3Method print regions_modelselect
print.regions_modelselect <- function(x, digits = 3L, ...) {
  arg::arg_whole_number(digits)

  x0 <- x
  class(x) <- setdiff(class(x), "regions_modelselect")

  for (i in which(startsWith(names(x), "breakpoint"))) {
    x[[i]] <- ifelse(is.na(x[[i]]), ".",
                     format(x[[i]], justify = "right"))
    names(x) <- sub("breakpoint", "BP ", names(x), fixed = TRUE)
  }

  for (i in which(grepl("RSS", names(x), fixed = TRUE))) {
    x[[i]] <- round(x[[i]], digits)
  }

  names(x)[1] <- "Regions"

  print(x, row.names = FALSE)

  invisible(x0)
}

data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Select which PCOs to use
## Manually (first 4 PCOs)
(PCOs <- PCOselect(alligator_PCO, "manual", scores = 4))

## Using variance cutoff: PCOs that explain 5% or more
## of total PCO variance
(PCOs <- PCOselect(alligator_PCO, "variance", cutoff = .05))

## Using bootstrapping with 50 reps (more reps should
## be used in practice; default is fine)
(PCOs <- PCOselect(alligator_PCO, "boot", nreps = 50))

plot(PCOs) #plot true eigenvalues against bootstrapped

## Using PCOs that optimize region score:
regionresults <- calcregions(alligator_PCO, scores = 1:21, noregions = 7,
                             minvert = 3, cont = TRUE, exhaus = TRUE,
                             verbose = FALSE)

(PCOs <- PCOselect(alligator_PCO, "max",
                   results = regionresults,
                   criterion = "bic"))

plot(PCOs)

summary(PCOs)

#' Process vertebral measurements
#'
#' `process_measurements()` initializes the analysis workflow by processing a dataset of vertebra measurements into an object usable by \pkg{MorphoRegions}. Such processing includes identifying the vertebra indices and the measurements and filling in missing values.
#'
#' @param data a data frame containing a column of vertebra indices and measurements for each vertebra, or a list thereof for multiple specimens.
#' @param pos the name or index of the variable in `data` containing the vertebra indices. Default is to use the first column.
#' @param measurements the names or indices of the variables in `data` containing the relevant vertebra measurements. If unspecified, will use all variables other than that specified in `pos`.
#' @param fillNA `logical`; whether to fill in missing values using a simple linear imputation. Default is `TRUE`. See Details.
#'
#' @return
#' A `regions_data` object, which is a list of data frames (one for each specimen) with attributes containing metadata.
#'
#' @details
#' Any rows with missing values for all measurements will be removed. When missing values in non-removed rows are present and `fillNA` is set to `TRUE`, `process_measurements()` fills them in if the sequence of missing values is no greater than 2 in length. For numeric variables, it uses a linear interpolation, and for categorical variables, it fills in the missing values with the surrounding non-missing values if they are identical and leaves them missing otherwise. Otherwise, missing values are left as they are.
#'
#' When a list of data frames is supplied to `data`, only the variables named in `measurements` that are common across datasets will be stored as measurement variables.
#'
#' @seealso
#' [svdPCO()] for computing principal coordinate axes from processed vertebra data.
#'
#' @example man/examples/example-process_measurements.R

#' @export
process_measurements <- function(data, pos = 1L, measurements, fillNA = TRUE) {
  arg::arg_supplied(data)

  if (is.matrix(data) || is.data.frame(data)) {
    data <- list(as.data.frame(data))
  }
  else if (is.list(data) && all(vapply(data, function(i) is.data.frame(i) || is.matrix(i), logical(1L)))) {
    for (i in seq_along(data)) {
      data[[i]] <- as.data.frame(data[[i]])
    }
  }
  else {
    arg::err("{.arg data} must be a matrix, data frame, or list thereof")
  }

  arg::arg_flag(fillNA)

  pos_names <- character(length(data))
  pos_var_list <- vector("list", length(data))

  for (i in seq_along(data)) {
    arg::arg_index(pos, data[[i]],
                   .msg = "{.arg pos} must be a single value indicating the column in {.arg data} containing the vertebra positions")

    if (is.numeric(pos)) {
      pos_names[i] <- names(data[[i]])[as.integer(pos)]
    }
    else {
      pos_names[i] <- pos
    }

    pos_var_list[[i]] <- data[[i]][[pos_names[i]]]
  }

  if (length(unique(pos_names)) > 1L) {
    arg::err("the variable identified by {.arg pos} must have the same name in all supplied datasets")
  }

  pos <- pos_names[1]

  for (i in seq_along(pos_var_list)) {
    arg::arg_whole_numeric(pos_var_list[[i]],
                           .msg = "{.arg pos} must refer to a variable of whole numbers identifying vertebra positions")
  }

  measurements_names_list <- vector("list", length(data))

  for (i in seq_along(data)) {
    arg::when_supplied(
      measurements,
      arg::arg_or(
        arg::arg_indices(data[[i]], .arg_data = "data"),
        arg::arg_null
      )
    )

    if (missing(measurements)) {
      measurements_names_list[[i]] <- setdiff(names(data[[i]]), pos)
    }
    else if (length(measurements) == 0) {
      measurements_names_list[[i]] <- character()
    }
    else if (is.numeric(measurements)) {
      measurements_names_list[[i]] <- names(data[[i]])[as.integer(measurements)]
    }
    else {
      measurements_names_list[[i]] <- measurements
    }

    if (pos %in% measurements_names_list[[i]]) {
      arg::err("{.arg pos} and {.arg measurements} cannot overlap")
    }
  }

  # Get only measurements that are common across datasets
  measurements <- Reduce(union, measurements_names_list)

  for (i in seq_along(data)) {

    #Subset to specified variables
    data[[i]] <- data[[i]][names(data[[i]]) %in% c(pos, measurements)]

    #Reorder columns to be consistent
    if (i > 1L) {
      data[[i]] <- data[[i]][names(data[[1L]])]
    }

    pos_ind <- match(pos, names(data[[i]]))

    #Rows that are all NA
    all_NA_rows <- which(apply(data[[i]], 1L, function(x) all(is.na(x[-pos_ind]))))

    if (length(all_NA_rows) > 0) {
      data[[i]] <- data[[i]][-all_NA_rows, ]
    }

    if (fillNA) {
      #Fill in missing values
      data[[i]] <- .missingval(data[[i]], pos_ind)

      if (anyNA(data[[i]])) {
        if (length(data) == 1L) {
          arg::wrn("missing values remain in the dataset because there were sequences of missing values greater than 2 in length")
        }
        else {
          arg::wrn("missing values remain in dataset {i} because there were sequences of missing values greater than 2 in length")
        }
      }
    }
  }

  attr(data, "pos_ind") <- match(pos, names(data[[1L]]))
  attr(data, "eligible_vertebrae") <- sort(unique(unlist(lapply(data, `[[`, pos))))

  class(data) <- "regions_data"

  data
}


#' Process vertebral measurements and PC scores of traditional morphometric data
#'
#' `process_PC()` initializes the analysis workflow by processing a dataset of vertebral measurements and already computed PC scores into an object usable by \pkg{MorphoRegions}. Such processing includes identifying the vertebra indices, measurements, and PC scores.
#'
#' @inheritParams process_measurements
#' @param data a data frame containing a column of vertebra indices and measurements for each vertebra, or a named list thereof for multiple specimens with the names of the list corresponding to the unique identifiers of each specimen.
#' @param pcscores a matrix or data frame containing PC scores of each vertebra. Note that for multiple specimens, an ordination method should have been performed on the concatenated data across all specimens so that `pcscores` is a single data frame or matrix containing scores for all specimens.
#' @param eigenvals a numeric vector containing the eigenvalues of each PC axis.
#' @param posPC a vector containing the positional information of vertebrae in `pcscores`, or a named list thereof for multiple specimens with the names of the list corresponding to the unique identifiers of each specimen. If not provided, will assume the row order of `pcscores` matches the vertebra order in `pos`.
#'
#' @return
#' A `regions_pco` object, which contains user-provided eigenvectors in the `scores` component and eigenvalues in the `eigen.val` component. The original dataset, including positional information, is stored in the `data` attribute.
#'
#' @details
#' Unlike `process_measurements()`, `process_PC()` does not fill in missing values and these should have been removed or replaced by numeric values before running `process_PC()`.
#'
#' @seealso
#' * [process_gmPC()] for processing PC scores from 2D or 3D geometric morphometric datasets
#' * [svdPCO()] for computing principal coordinate axes from processed vertebra data.
#' * [plot.regions_pco()] for plotting PCO axes

#' @example man/examples/example-process_PC.R

#' @export
process_PC <- function(data, pos = 1L, pcscores, eigenvals, posPC) { #, specimen) {
  arg::arg_supplied(data)
  arg::arg_supplied(pcscores)
  arg::arg_supplied(eigenvals)

  # Check if multiple specimens:
  multiSpec <- is.list(data) && !is.data.frame(data)

  # Check format of posPC and extract specimen names if multi-specimens:
  if (multiSpec && !missing(posPC)) {
    arg::arg_list(posPC)
    arg::arg_named(posPC)
    arg::arg_named(data)

    vertposPC <- setNames(unlist(posPC), NULL)

    arg::arg_whole_numeric(vertposPC,
                           .msg = "{.arg posPC} must only contain whole numbers identifying vertebra positions")

    spec_names_PC <- names(posPC)
    spec_names_data <- names(data)

    if (!setequal(spec_names_PC, spec_names_data)) {
      arg::err("the specimen names in {.arg pcscores} must be the same as those in {.arg data}")
    }
  }

  # Check format data and convert to list if needed:
  if (is.matrix(data) || is.data.frame(data)) {
    data <- list(as.data.frame(data))
  }
  else if (is.list(data) && all(vapply(data, function(i) is.data.frame(i) || is.matrix(i), logical(1L)))) {
    for (i in seq_along(data)) {
      data[[i]] <- as.data.frame(data[[i]])
    }
  }
  else {
    arg::err("{.arg data} must be a matrix, data frame, or list thereof")
  }

  pos_names <- character(length(data))
  pos_var_list <- vector("list", length(data))

  # Get positional info for each specimen:
  for (i in seq_along(data)) {
    arg::arg_index(pos, data[[i]],
                   .msg = "{.arg pos} must be a single value indicating the column in {.arg data} containing the vertebra positions")

    if (is.numeric(pos)) {
      pos_names[i] <- names(data[[i]])[as.integer(pos)]
    }
    else {
      pos_names[i] <- pos
    }

    pos_var_list[[i]] <- data[[i]][[pos_names[i]]]
  }

  if (length(unique(pos_names)) > 1L) {
    arg::err("the variable identified by {.arg pos} must have the same name in all supplied datasets")
  }

  pos <- pos_names[1L]

  for (i in seq_along(pos_var_list)) {
    arg::arg_whole_numeric(pos_var_list[[i]],
                           .msg = "{.arg pos} must refer to a variable of whole numbers identifying vertebra positions")
  }

  # Check pcscores
  if (is.data.frame(pcscores) && all(vapply(pcscores, is.numeric, logical(1L)))) {
    pcscores <- as.matrix(pcscores)
  }
  else if (!is.matrix(pcscores) || !is.numeric(pcscores)) {
    arg::err("{.arg pcscores} must be a numeric matrix or a data frame containing only numeric columns")
  }

  # Check eigenvals
  arg::arg_vector(eigenvals)
  arg::arg_numeric(eigenvals)
  arg::arg_no_NA(eigenvals)

  # Check format posPC and extract positional info from PCs:
  allvertData <- setNames(unlist(lapply(data, `[[`, pos)), NULL)

  nvData <- length(allvertData)

  if (missing(posPC)) {
    arg::wrn("{.arg posPC} not supplied: assuming order of vertebrae in {.arg pcscores} matches the order of {.arg data}")

    if (nrow(pcscores) != nvData) {
      arg::err("the number of rows in {.arg pcscores} must match the number of vertebrae in {.arg data}")
    }

    vertposPC <- allvertData
    spec_names_PC <- names(data)
  }
  else if (!multiSpec) {
    arg::arg_vector(posPC)
    arg::arg_whole_numeric(posPC)

    if (length(posPC) != nvData) {
      arg::err("the number of elements in {.arg posPC} must match the number of vertebrae in {.arg data}")
    }

    vertposPC <- posPC
  }

  # Check number of PC pcscores and eigenvals match:
  if (length(eigenvals) != ncol(pcscores)) {
    arg::err(c("{.arg eigenvals} must have one value per column in {.arg pcscores}.",
               "i" = "Length of {.arg eigenvals}: {length(eigenvals)}",
               "i" = "Number of columns in {.arg pcscores}: {ncol(pcscores)}"))
  }

  # Process data:
  # Note other transformations made to data in process_measurements have been removed here to prevent heavy modifications to data since it won't be used for downstream analyses
  for (i in seq_along(data)[-1L]) {
    #Reorder columns to be consistent
    data[[i]] <- data[[i]][names(data[[1L]])]
  }

  # Format 'data' part of output:
  attr(data, "pos_ind") <- match(pos, names(data[[1L]]))
  attr(data, "eligible_vertebrae") <- sort(unique(unlist(lapply(data, `[[`, pos))))
  class(data) <- "regions_data"

  # Process pcscores:
  # Order vertebrae in pcscores according to order in data:
  if (!setequal(vertposPC, allvertData)) {
    arg::err("the vertebrae in {.arg pcscores} must match those included in {.arg data}")
  }

  if (!missing(posPC)) {                                                # Reorder only if posPC is supplied
    if (multiSpec) {
      vert_ord_data <- paste(rep(spec_names_data, unlist(lapply(data, nrow))),
                             allvertData, sep = "_")                     # Create vector of vertebra order in data
      vert_ord_pc <- paste(rep(spec_names_PC, lengths(posPC)),
                           vertposPC, sep = "_")                         # Create vector of vertebra order in pcscores
      pcscores <- pcscores[match(vert_ord_data, vert_ord_pc), , drop = FALSE]         # Re-order pcscores to match order in data
    }
    else {
      pcscores <- pcscores[match(allvertData, vertposPC), , drop = FALSE]         # Re-order pcscores to match order in data
    }
  }

  # Format PCA output:
  out <- list(scores = pcscores,
              eigen.val = eigenvals)

  attr(out, "data") <- data
  attr(out, "metric") <- "custom"
  attr(out, "scale") <- "custom"
  attr(out, "specimen") <- factor(rep(seq_along(data), unlist(lapply(data, nrow))),
                                  labels = paste("Specimen", seq_along(data)),
                                  levels = seq_along(data))
  class(out) <- "regions_pco"

  out
}

#' Process vertebral measurements and PC scores of 2D or 3D geometric morphometric data
#'
#' `process_gmPC()` initializes the analysis workflow by processing vertebral measurements and already computed PC scores from 2D or 3D geometric morphometric datasets into an object usable by \pkg{MorphoRegions}. Such processing includes identifying the vertebra indices, measurements, and PC scores.
#'
#' @inheritParams process_PC
#' @param data a 3D array (\eqn{p \times k \times n}) of Procrustes aligned landmarks where \eqn{p} is the number of of landmark points per vertebra, \eqn{k} is the number of landmark dimensions (2D or 3D), and \eqn{n} is the number of vertebrae sampled for the specimen.
#' @param pos either character string `"names"` in which case the function will extract vertebral position info form the names of the array (using `dimnames(data)[[3]]`), or a vector containing numeric values of vertebral position. Default is `"names"`.
#' @param pcscores a matrix or data frame containing PC scores of each vertebra.
#' @param specimens if `data` contains multiple specimens, a vector of specimen names of length equal to the number of vertebrae in `data`.
#'
#' @return
#' A `regions_pco` object, which contains user-provided eigenvectors in the `scores` component and eigenvalues in the `eigen.val` component. The original dataset, including landmark coordinates and positional information, is stored in the `data` attribute.
#'
#' @details
#' Unlike `process_measurements()`, `process_gmPC()` does not fill in missing values and these should have been removed or replaced by numeric values before running `process_gmPC()`.
#'
#' @seealso
#' * [process_PC()] for processing PC scores from traditional morphometric datasets
#' * [svdPCO()] for computing principal coordinate axes from processed vertebra data.
#' * [plot.regions_pco()] for plotting PCO axes

#' @example man/examples/example-process_gmPC.R

#' @export
process_gmPC <- function(data, pos = "names", pcscores, eigenvals, specimens = NULL) {

  # Check format of landmark data:
  if (!is.array(data) || !is.numeric(data) || length(dim(data)) != 3L) {
    arg::err("{.arg data} must be a 3-dimensional numeric array")
  }

  # Check format of vertebral position info:
  arg::arg_or(
    pos,
    arg::arg_element("names"),
    arg::arg_numeric
  )

  if (identical(pos, "names")) {
    raw_names <- dimnames(data)[[3L]]

    # Check no NULL values in names of array
    if (length(raw_names) == 0L) {
      arg::err("when {.arg pos} is {.val names}, {.code dimnames(data)[[3]]} cannot be empty")
    }

    # If vertebra position info taken from names of array, convert them to numeric
    pos <- suppressWarnings(as.numeric(raw_names))

    # Explicitly test for coercion-induced NAs
    if (anyNA(pos)) {
      arg::err(c("Cannot derive numeric vertebral positions from data.",
                 "i" = "The following names could not be converted to numbers: {.val {raw_names[is.na(pos)]}}"))
    }
  }
  else if (length(pos) == dim(data)[3L]) {
    arg::arg_no_NA(pos)
    # Warn that function cannot verify that order of pos and data match each other
    arg::wrn("{.arg pos} supplied as numeric vector; assuming its order matches the order of {.arg data}")
    pos <- as.vector(pos)
  }
  else {
    # If user-supplied vertebral position, check if has same length as number of vertebrae in array
    arg::err("when supplied as a numeric vector, {.arg pos} must have length equal to that of {.code dim(data)[3]}")
  }

  # Check if specimen provided:
  if (!missing(specimens)) {
    if (length(specimens) != dim(data)[3L]) {
      arg::err("when supplied, {.arg specimens} must have length equal to {.code dim(data)[3]}")
    }

    specimens <- as.factor(specimens)
  }
  else if (anyDuplicated(pos) == 0L) {
    specimens <- factor(rep.int("Specimen1", length(pos)), nmax = 1L)
  }
  else {
    arg::err("{.arg specimens} must be provided when the same vertebral position is sampled more than once in {.arg data} or {.arg pos}")
  }

  # Check pcscores
  if (is.data.frame(pcscores) && all(vapply(pcscores, is.numeric, logical(1L)))) {
    pcscores <- as.matrix(pcscores)
  }
  else if (!is.matrix(pcscores) || !is.numeric(pcscores)) {
    arg::err("{.arg pcscores} must be a numeric matrix or a data frame containing only numeric columns")
  }

  # Check eigenvals
  arg::arg_vector(eigenvals)
  arg::arg_numeric(eigenvals)
  arg::arg_no_NA(eigenvals)

  # Check number of PC pcscores and eigenvals match:
  if (length(eigenvals) != ncol(pcscores)) {
    arg::err(
      c("{.arg eigenvals} must have one value per column in {.arg pcscores}.",
        "i" = "Length of {.arg eigenvals}: {.val {length(eigenvals)}}",
        "i" = "Columns in {.arg pcscores}: {.val {ncol(pcscores)}}")
    )
  }

  # Check number of vertebrae in pos and pcscores match:
  if (length(pos) != nrow(pcscores)) {
    arg::err("the number of vertebrae in {.arg pcscores} must match the number of vertebrae in {.arg pos}")
  }

  # Check rownames pcscores:
  if (length(rownames(pcscores)) == 0L) {
    # If no rownames, add pos as rownames
    arg::wrn("{.arg pcscores} has no row names; assuming row order matches {.arg pos}")
    rownames(pcscores) <- pos
  }
  else if (!setequal(rownames(pcscores), as.character(pos))) {
    # If rownames different from pos, warning but do not replace original row names
    arg::wrn("the row names of {.arg pcscores} do not match {.arg pos}; assuming row order matches {.arg pos}")
  }

  # # Order pos, data, and pcscores from most anterior to most posterior vertebra:
  # ord <- order(pos)
  # pos <- pos[ord]
  # data <- data[,,ord, drop=F]
  # pcscores <- pcscores[ord,, drop=F]


  # Format 'data' part of output:
  attr(data, "Xvar") <- pos
  attr(data, "eligible_vertebrae") <- sort(unique(pos))
  class(data) <- "regions_dataGM"

  # Format PCA output:
  out <- list(scores = pcscores,
              eigen.val = eigenvals)

  attr(out, "data") <- data
  attr(out, "metric") <- "custom"
  attr(out, "scale") <- "custom"
  attr(out, "specimen") <- specimens

  class(out) <- "regions_pco"

  out
}

# Takes in a dataframe and fills in missing values
# `max_NA_seq_len` controls max number of consecutive missing values that can
# be imputed
.missingval <- function(data, pos_ind, max_NA_seq_len = 2L) {

  if (!anyNA(data[-pos_ind])) {
    return(data)
  }

  arg::arg_count(max_NA_seq_len)

  # Add missing vertebrae
  missing_vertebrae <- setdiff(seq(min(data[[pos_ind]]),
                                   max(data[[pos_ind]])),
                               data[[pos_ind]])

  if (length(missing_vertebrae) > 0) {
    missing_rows <- as.data.frame(matrix(NA, nrow = length(missing_vertebrae),
                                         ncol = ncol(data)))
    names(missing_rows) <- names(data)
    missing_rows[[pos_ind]] <- missing_vertebrae

    data <- rbind(data, missing_rows)
  }

  # Order by vertebra
  o <- order(data[[pos_ind]])

  pos <- data[[pos_ind]]

  # Find strings of 1-2 NAs
  variables_with_missing <- which(vapply(data[!(pos %in% missing_vertebrae), -pos_ind],
                                         anyNA, logical(1L)))

  for (i in variables_with_missing) {
    # Extract variable ordered by vertebra
    dat <- data[o, -pos_ind][[i]]

    miss.par <- which(is.na(dat)) #find which ones are missing
    seqs <- split(miss.par, cumsum(c(1, diff(miss.par) != 1))) #split them into sequences

    l.seq <- which(lengths(seqs) <= max_NA_seq_len) #which strings are two or less

    if (length(l.seq) == 0) {
      #if no short strings skip
      next
    }

    seqs <- seqs[l.seq]

    for (fill in seqs) { #Fill each string
      before <- min(fill) - 1
      after <- max(fill) + 1

      if (before < 1) before <- after #if at the beginning, use the end points
      if (after > length(dat)) after <- before #if at the end, use beginning points

      if (!is.numeric(dat) && dat[before] != dat[after]) {
        next
      }

      if (is.numeric(dat)) {
        val <- seq(dat[before], dat[after], length.out = length(fill) + 2) #calculate missing as mean of adjacent
        dat[fill] <- val[-c(1, length(val))] #fill in the missing
      }
      else {
        val <- rep(dat[before], length(fill))
        dat[fill] <- val #fill in the missing
      }
    }

    data[o, -pos_ind][[i]] <- dat
  }

  # Remove extra fully missing rows
  subset(data, !pos %in% missing_vertebrae)
}

data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Fit segmented regression models for 1 to 5 regions
# using PCOs 1 to 4 and a continuous model with a
# non-exhaustive search
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 5,
                             minvert = 3,
                             cont = TRUE,
                             exhaus = FALSE)

regionresults

# View model fitting summary
summary(regionresults)

# Add additional regions to existing results,
# exhaustive search this time
regionresults <- addregions(regionresults,
                            noregions = 6:7,
                            exhaus = TRUE)

regionresults

summary(regionresults)

# Fit segmented regression models for 1 to 5 regions
# using PCOs 1 to 4 and a discontinuous model with a
# exhaustive search, excluding breakpoints at vertebrae
# 10 and 15
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 5,
                             minvert = 3,
                             cont = FALSE,
                             omitbp = c(10, 15))

regionresults

summary(regionresults)

# Compute the number of breakpoint combinations for given
# specification using `ncombos()`; if any number exceeds
# the value supplied to `ncombos_file_trigger`, results
# will temporary be stored in files before being read in and
# deleted.
ncombos(alligator_PCO,
        noregions = 1:8,
        minvert = 3)

#' Fit segmented regression models for all combinations of breakpoints
#'
#' `calcregions()` enumerates all possible combinations of breakpoints to fit multivariate segmented regression models. `addregions()` adds models with additional numbers of regions to the resulting output object. `ncombos()` computes an upper bound on the number of breakpoint combinations that will be tested.
#'
#' @param pco a `regions_pco` object; the output of a call to [svdPCO()].
#' @param scores `numeric`; the indices of the PCO scores to use as the outcomes in fitting the models (e.g., `1:4` to use the first four scores). Can also be the output of a call to [PCOselect()].
#' @param noregions `numeric`; for `calcregions()`, the maximum number of regions for which models are fit (e.g, 4 to request models with 1 to 4 regions); for `addregions()`, a vector containing the numbers of regions to add (e.g., 5:6 to request models with 5 and 6 regions); for `ncombos()`, a vector containing the numbers of regions to check.
#' @param minvert `numeric`; the minimum number of vertebrae allowed in each region. Default is 3.
#' @param cont `logical`; whether to fit models that are continuous (`TRUE`) or discontinuous (`FALSE`) at the breakpoints. Default is `TRUE`.
#' @param exhaus `logical`; whether to fit all possible models (`TRUE`) or use heuristics to reduce the number of models fit (`FALSE`). Default is `TRUE`. See Details. Setting to `FALSE` can reduce the size of the resulting object.
#' @param includebp an optional vector of vertebrae that must be included in any tested set of breakpoints, e.g., if it is known that two regions are divided at that vertebra. `includebp` does not have to obey the `minvert` rules, but a warning will be thrown if it doesn't.
#' @param omitbp an optional vector of vertebrae to be omitted from the list of possible breakpoints, e.g., if it is known that two adjacent vertebrae belong to the same region.
#' @param ncombos_file_trigger `numeric`; when the number of eligible combinations of breakpoints exceeds this number, the problem will be split into smaller problems, with the results of each stored in its own temporary file in the directory supplied to `temp_file_dir` before being re-read into memory. The primary purpose of this is to preserve memory when `exhaus = FALSE` by delegating storage of the results to disk instead of RAM.
#' @param temp_file_dir string; the directory where the temporary files will be saved (and then deleted) when the number of breakpoint combinations exceeds `ncombos_file_trigger`. Default is the directory produced by [tempdir()], but it is much safer to provide your own directory, which must already exist on your machine. See Details.
#' @param cl a cluster object created by [parallel::makeCluster()], an integer to indicate number of child-processes (integer values are ignored on Windows) for parallel evaluations, or `"future"` to use a future backend. `NULL` (the default) refers to sequential evaluation (no parallelization). See [pbapply::pbapply()] for details.
#' @param verbose `logical`; whether to print information about the fitting process, including a progress bar. Default is `TRUE` when running interactively and `FALSE` otherwise.
#' @param regions_results,object a `regions_results` object; the output of a call to `calcregions()` or `addregions()`.
#' @param \dots ignored.
#'
#' @returns A `regions_results` object with the following components:
#' * `results` - the results of the fitting process for each combination of breakpoints
#' * `stats` - statistics summarizing the fitting process. Use `summary()` to view this information in a clean format.
#'
#' `ncombos()` returns a numeric vector with the number of breakpoint combinations for each number of regions (which are stored as the names).
#'
#' @details `calcregions()` enumerates all possible combinations of breakpoints that satisfy the constraint imposed by `minvert` (i.e., that breakpoints need to be at least `minvert` vertebrae apart) and fits the segmented regression models implied by each combination. These are multivariate regression models with the PCO scores specified by `scores` as the outcomes. When `cont = TRUE`, these regression models are continuous; i.e., the regression lines for each region connect at the breakpoints. Otherwise, the models are discontinuous so that each region has its own intercept and slope. The models are fit using [.lm.fit()], which efficiently implements ordinary least squares regression.
#'
#' When `exhaus = FALSE`, heuristics are used to reduce the number of models to fit, which can be useful for keeping the size of the resulting object down by avoiding fitting models corresponding to breakpoint combinations that yield a poor fit to the data. Only breakpoint combinations that correspond to the breakpoints of the best fitting model with a smaller number of regions +/- 3 vertebrae are used, and only models that have an RSS smaller than half a standard deviation more the smallest RSS are kept.
#'
#' `addregions()` should be used on an existing `regions_results` object to add models with more regions. Internally, it works just the same as `calcregions()`.
#'
#' `ncomobs()` computes an upper bound on the number of possible breakpoint combinations. When `exhaus = FALSE` or `includebp` is specified, the actual number of combinations will be smaller than that produced by `ncombos()`.
#'
#' When the number of possible combinations of breakpoints for a given number of regions (as computed by `ncombos()`) is larger than `ncombos_file_trigger`, the problem will be split into smaller problems, with the results of each stored in temporary files that are deleted when the function completes. These temporary files will be stored in the directory supplied to `temp_file_dir`. By default, this is the temporary directory produced by [tempdir()]. However, this directory can be deleted by R at any time without warning, which will cause the function to crash, so it is a good idea to supply your own directory that will be preserved. You can use `ncombos()` to check to see if the number of breakpoint combinations exceeds `ncombos_file_trigger`.
#'
#' @seealso [calcmodel()] to fit a segmented regression model for a single set of breakpoints; [modelselect()] to select the best model for each number of regions based on RSS; [modelsupport()] to compute statistics the describe the support of the best models; [calcBPvar()] to compute the variability in the optimal breakpoints.
#'
#' @example man/examples/example-calcregions.R

#' @export
calcregions <- function(pco, scores, noregions, minvert = 3, cont = TRUE,
                        exhaus = TRUE, includebp = NULL, omitbp = NULL,
                        ncombos_file_trigger = 1e7, temp_file_dir = tempdir(TRUE),
                        cl = NULL, verbose = interactive()) {
  # Argument checks
  arg::arg_is(pco, c("regions_pco", "regions_sim"))

  if (inherits(pco, "regions_pco")) {
    arg::arg_supplied(scores)
    arg::arg_whole_numeric(scores)
    arg::arg_between(scores, c(1L, ncol(pco[["scores"]])))

    Xvar <- .get_pos(pco)
    Yvar <- pco[["scores"]][, scores, drop = FALSE]

    eligible_vertebrae <- .get_eligible_vertebrae(pco)
  }
  else {
    arg::when_supplied(
      scores,
      arg::arg_whole_numeric,
      arg::arg_between(c(1, ncol(pco[["Yvar"]])))
    )

    if (missing(scores)) {
      scores <- seq_len(ncol(pco[["Yvar"]]))
    }

    Xvar <- pco[["Xvar"]]
    Yvar <- pco[["Yvar"]][, scores, drop = FALSE]

    eligible_vertebrae <- sort(unique(Xvar))
  }

  arg::arg_supplied(noregions)
  arg::arg_count(noregions)
  arg::arg_gte(noregions, 1)

  arg::arg_count(minvert)
  arg::arg_gte(minvert, 2)
  arg::arg_flag(cont)
  arg::arg_flag(exhaus)
  arg::arg_flag(verbose)

  arg::when_not_null(
    includebp,
    arg::arg_whole_numeric
  )

  if (!is.null(includebp)) {
    if (!all(includebp %in% eligible_vertebrae)) {
      arg::err("all breakpoints specified in {.arg includebp} must correspond to measured vertebrae")
    }

    if (length(includebp) > 1) {
      includebp <- sort(includebp)

      if (sum(eligible_vertebrae < min(includebp)) < 1 ||
          sum(eligible_vertebrae > max(includebp)) < 2) {
        arg::err("breakpoints specified in {.arg includebp} must be be at least 2 measured vertebrae from the ends")
      }

      for (i in seq_along(includebp)[-1]) {
        if (sum(eligible_vertebrae > includebp[i - 1] &
                eligible_vertebrae <= includebp[i]) < 2) {
          arg::err("breakpoints specified in {.arg includebp} must have at least 2 measured vertebrae between them")
        }
      }

      if (length(includebp) >= noregions) {
        arg::err("the length of {.arg includebp} must be less than {.arg noregions}")
      }

      if (any(diff(c(eligible_vertebrae[1L] - 1, includebp, max(eligible_vertebrae))) < minvert)) {
        arg::wrn("the breakpoints specified in {.arg includebp} do not obey the {.arg minvert} crtierion")
      }
    }
  }

  eligible_noregions <- {
    if (is.null(includebp)) seq_len(noregions)
    else seq(length(includebp) + 1, noregions)
  }

  arg::when_not_null(
    omitbp,
    arg::arg_whole_numeric
  )

  if (!is.null(omitbp)) {
    if (any(omitbp %in% includebp)) {
      arg::err("breakpoints specified in {.arg omitbp} cannot also be present in {.arg includebp}")
    }

    eligible_vertebrae <- eligible_vertebrae[!eligible_vertebrae %in% omitbp]
  }

  # Re-order data
  o <- order(Xvar)
  Xvar <- Xvar[o]
  Yvar <- Yvar[o, , drop = FALSE]

  ncombs <- vapply(eligible_noregions, function(i) {
    .ncombos(length(eligible_vertebrae), minvert, i - 1)
  }, numeric(1L))

  if (any(ncombs == 0)) {
    max.regions <- max(which(ncombs > 0))

    arg::wrn("models for {length(eligible_vertebrae)} vertebrae cannot be fit with more than {max.regions} region{?s} and {.arg minvert} of {minvert}")

    ncombs <- ncombs[eligible_noregions <= max.regions]
    eligible_noregions <- eligible_noregions[eligible_noregions <= max.regions]
  }

  if (any(eligible_noregions > 3)) {
    arg::arg_count(ncombos_file_trigger)
    arg::arg_gt(ncombos_file_trigger, 0)

    if (any(ncombs[eligible_noregions > 3] > ncombos_file_trigger)) {
      if (!"temp_file_dir" %in% names(match.call())[-1L] ||
          identical(temp_file_dir, tempdir())) {
        fn <- "calcregions"
        # arg::wrn(sprintf("`%s()` will use temporary files stored in the directory supplied to `temp_file_dir`. Because this was unspecified or left as its default, you may see the error \"cannot write to connection\" after a long run of this function. See `help(\"%s\")` for solutions and how to avoid this problem", fn, fn))
      }
      else {
        arg::arg_string(temp_file_dir)
      }
    }
  }

  opb <- pbapply::pboptions(type = if (verbose) "timer" else "none")
  on.exit(pbapply::pboptions(opb))

  if (inherits(cl, "cluster")) {
    parallel::clusterExport(cl,
                            unclass(utils::lsf.str(envir = asNamespace(utils::packageName()),
                                                   all.names = TRUE)),
                            envir = getNamespace(utils::packageName()))
  }

  # Prep output object
  out <- list(results = NULL, stats = NULL)

  attr(out, "scores") <- Yvar
  attr(out, "cont") <- cont
  attr(out, "minvert") <- minvert
  attr(out, "includebp") <- includebp
  attr(out, "omitbp") <- omitbp
  attr(out, "pos") <- Xvar
  attr(out, "eligible_vertebrae") <- eligible_vertebrae

  class(out) <- "regions_results"

  for (noregion in eligible_noregions) {
    out <- .addregions_internal(out, noregion, exhaus = exhaus,
                                ncombos_file_trigger = ncombos_file_trigger,
                                temp_file_dir = temp_file_dir,
                                cl = cl, verbose = verbose)
  }

  out
}

#' @export
#' @rdname calcregions
addregions <- function(regions_results, noregions, exhaus = TRUE,
                       ncombos_file_trigger = 1e7, temp_file_dir = tempdir(TRUE),
                       cl = NULL, verbose = interactive()) {
  # Argument checks
  arg::arg_is(regions_results, "regions_results")

  arg::arg_supplied(noregions)
  arg::arg_whole_numeric(noregions)
  arg::arg_gte(noregions, 1)
  noregions <- sort(noregions)

  arg::arg_flag(exhaus)
  arg::arg_flag(verbose)

  opb <- pbapply::pboptions(type = if (verbose) "timer" else "none")
  on.exit(pbapply::pboptions(opb))

  eligible_vertebrae <- attr(regions_results, "eligible_vertebrae")

  minvert <- attr(regions_results, "minvert")

  ncombs <- vapply(noregions, function(i) {
    .ncombos(length(eligible_vertebrae), minvert, i - 1)
  }, numeric(1L))

  if (any(ncombs == 0)) {
    max.regions <- max(noregions[ncombs > 0])

    arg::wrn("models for {length(eligible_vertebrae)} vertebrae cannot be fit with more than {max.regions} region{?s} and {.arg minvert} of {minvert}")

    ncombs <- ncombs[noregions <= max.regions]
    noregions <- noregions[noregions <= max.regions]
  }

  if (any(noregions > 3)) {
    arg::arg_count(ncombos_file_trigger)
    arg::arg_gt(ncombos_file_trigger, 0)
    if (any(ncombs[noregions > 3] > ncombos_file_trigger)) {
      if (!"temp_file_dir" %in% names(match.call())[-1L] ||
          identical(temp_file_dir, tempdir())) {
        fn <- "addregions"
        # arg::wrn(sprintf("`%s()` will use temporary files stored in the directory supplied to `temp_file_dir`. Because this was unspecified or left as its default, you may see the error \"cannot write to connection\" after a long run of this function. See `help(\"%s\")` for solutions and how to avoid this problem", fn, fn))
      }
      else {
        arg::arg_string(temp_file_dir)
      }
    }
  }

  if (inherits(cl, "cluster")) {
    parallel::clusterExport(cl,
                            unclass(utils::lsf.str(envir = asNamespace(utils::packageName()),
                                                   all.names = TRUE)),
                            envir = getNamespace(utils::packageName()))
  }

  for (n in noregions) {
    regions_results <- .addregions_internal(regions_results, noregions = n,
                                            exhaus = exhaus,
                                            ncombos_file_trigger = ncombos_file_trigger,
                                            temp_file_dir = temp_file_dir,
                                            cl = cl, verbose = verbose)
  }

  regions_results
}

#' @exportS3Method print regions_results
print.regions_results <- function(x, ...) {
  cat("A `regions_results` object\n")
  cat(" - number of PCOs used:", ncol(attr(x, "scores")), "\n")
  cat(" - number of regions:", toString(sort(unique(x$stats[["Nregions"]]))), "\n")
  cat(" - model type:", if (attr(x, "cont")) "continuous" else "discontinuous", "\n")
  cat(" - min vertebrae per region:", attr(x, "minvert"), "\n")
  if (!is.null(attr(x, "omitbp"))) {
    cat(" - omitted breakpoints:", toString(attr(x, "omitbp")), "\n")
  }
  cat(" - total models saved:", nrow(x$results), "\n")
  cat("Use `summary()` to examine summaries of the fitting process.\n")

  invisible(x)
}

#' @exportS3Method summary regions_results
#' @rdname calcregions
summary.regions_results <- function(object, ...) {
  stats <- object$stats[1:6]
  names(stats) <- c("Regions", "Possible", "Tested", "Saved",
                    "Comp. method", "Saving method")

  class(stats) <- c("summary.regions_results", class(stats))

  stats
}

#' @exportS3Method print summary.regions_results
print.summary.regions_results <- function(x, ...) {
  print.data.frame(x, row.names = FALSE)

  invisible(x)
}

#' @export
#' @rdname calcregions
ncombos <- function(pco, noregions, minvert = 3, includebp = NULL, omitbp = NULL) {
  # Argument checks
  arg::arg_is(pco, c("regions_pco", "regions_sim"))

  if (inherits(pco, "regions_pco")) {
    eligible_vertebrae <- .get_eligible_vertebrae(pco)
  }
  else {
    Xvar <- pco[["Xvar"]]

    eligible_vertebrae <- sort(unique(Xvar))
  }

  arg::arg_supplied(noregions)
  arg::arg_whole_numeric(noregions)
  arg::arg_gte(noregions, 1)
  noregions <- sort(noregions)

  arg::arg_count(minvert)
  arg::arg_gte(minvert, 2)

  arg::when_not_null(
    includebp,
    arg::arg_whole_numeric
  )

  if (!is.null(includebp)) {

    if (!all(includebp %in% eligible_vertebrae)) {
      arg::err("all breakpoints specified in {.arg includebp} must correspond to measured vertebrae")
    }

    if (length(includebp) > 1L) {
      includebp <- sort(includebp)

      if (sum(eligible_vertebrae < min(includebp)) < 1 ||
          sum(eligible_vertebrae > max(includebp)) < 2) {
        arg::err("breakpoints specified in {.arg includebp} must be be at least 2 measured vertebrae from the ends")
      }

      for (i in seq_along(includebp)[-1]) {
        if (sum(eligible_vertebrae > includebp[i - 1] &
                eligible_vertebrae <= includebp[i]) < 2) {
          arg::err("breakpoints specified in {.arg includebp} must have at least 2 measured vertebrae between them")
        }
      }

      if (length(includebp) >= noregions) {
        arg::err("the length of {.arg includebp} must be less than {.arg noregions}")
      }

      if (any(diff(c(eligible_vertebrae[1] - 1, includebp, max(eligible_vertebrae))) < minvert)) {
        arg::wrn("the breakpoints specified in {.arg includebp} do not obey the {.arg minvert} crtierion")
      }
    }
  }

  eligible_noregions <- {
    if (is.null(includebp)) noregions
    else noregions[noregions >= length(includebp)]
  }

  arg::when_not_null(
    omitbp,
    arg::arg_whole_numeric
  )

  if (!is.null(omitbp)) {
    if (any(omitbp %in% includebp)) {
      arg::err("breakpoints specified in {.arg omitbp} cannot also be present in {.arg includebp}")
    }

    eligible_vertebrae <- eligible_vertebrae[!eligible_vertebrae %in% omitbp]
  }

  ncombs <- vapply(eligible_noregions, function(i) {
    .ncombos(length(eligible_vertebrae), minvert, i - 1)
  }, numeric(1L))

  setNames(ncombs, eligible_noregions)
}

# Does the actual work of fitting the segmented regression models
.addregions_internal <- function(regions_results, noregions, exhaus = FALSE,
                                 ncombos_file_trigger = 1e6,
                                 temp_file_dir = tempdir(TRUE),
                                 cl = NULL, verbose = FALSE) {

  minvert <- attr(regions_results, "minvert")

  cont <- attr(regions_results, "cont")

  Xvar <- attr(regions_results, "pos")
  Yvar <- attr(regions_results, "scores")
  eligible_vertebrae <- attr(regions_results, "eligible_vertebrae")

  #Calculate weights to ensure each vertebra counts equally
  vert_tab <- tabulate(Xvar)
  w <- 1 / vert_tab[Xvar]

  includebp <- attr(regions_results, "includebp")

  nbp <- noregions - 1L

  noPC <- ncol(Yvar)

  colhead <- c("regions", paste0("breakpoint", seq_len(max(1, nbp))),
               "sumRSS", paste("RSS", 1:noPC, sep = "."))

  # Set verbosity of progress bar
  if (!verbose) {
    opb <- pbapply::pboptions(type = "none")
    on.exit(pbapply::pboptions(opb))
  }

  # Function to fit regression based on BPs
  fregions <- function(BPs, Xvar, Yvar, w) {
    x <- .design_matrix(Xvar, BPs, cont)

    lines <- .fast_lm(x = x, y = Yvar, w = w)
    RSS <- sum(w * lines$residuals^2)

    rsq <- {
      if (noPC > 1) .colSums(w * lines$residuals^2,
                             m = length(Xvar), n = ncol(Yvar))
      else RSS
    }

    c(RSS, rsq)
  }

  if (nbp == 0) {
    # 1 Regions (0 bp)

    if (!is.null(includebp)) {
      return(regions_results)
    }

    if (verbose) {
      cat("Fitting model with 1 region (0 breakpoints)...\n")
    }

    lines <- .fast_lm(x = cbind(1, Xvar), y = Yvar, w = w)

    RSS <- sum(w * lines$residuals^2)

    rsq <- {
      if (noPC > 1) .colSums(w * lines$residuals^2,
                             m = length(Xvar), n = ncol(Yvar))
      else RSS
    }

    res <- setNames(list2DF(c(as.list(nbp + 1L),
                              as.list(NA_real_),
                              as.list(RSS),
                              as.list(rsq))),
                    colhead)

    bpkeep <- NA_character_

    stats <- list2DF(list(Nregions = nbp + 1L,
                          Nmodel_possible = 1L,
                          Nmodel_tested = 1L,
                          Nmodel_saved = 1L,
                          Comp_method = "Exhaustive",
                          Saving_method = "All"))

    best_bps <- matrix(bpkeep, nrow = 1L, dimnames = list(NULL, "Best_BPs1"))

    if (verbose) {
      cat("Done. 1 model tested, 1 model saved.\n")
    }
  }
  else if (nbp == 1) {
    # 2 Regions (1 bp)

    # Get all possible combinations for 1 breakpoint:
    goodcomb <- .combos(eligible_vertebrae, minvert, nbp, includebp = includebp)

    if (nrow(goodcomb) == 0) {
      return(regions_results)
    }

    if (verbose) {
      cat("Fitting models with 2 regions (1 breakpoint)...\n")
    }

    # Run fitting on good combinations:
    res <- do.call("rbind", pbapply::pbapply(goodcomb, 1L, function(bps) {
      fregions(bps, Xvar = Xvar, Yvar = Yvar, w = w)
    }, simplify = FALSE, cl = cl))

    res <- setNames(cbind(nbp + 1L,
                          as.data.frame(goodcomb),
                          as.data.frame(res)),
                    colhead)

    bpkeep <- {
      NA_character_
      # if (!exhaus) paste(goodcomb[,1], collapse = "|")
      # else NA_character_
    }

    stats <- list2DF(list(Nregions = nbp + 1L,
                          Nmodel_possible = nrow(res),
                          Nmodel_tested = nrow(res),
                          Nmodel_saved = nrow(res),
                          Comp_method = "Exhaustive",
                          Saving_method = "All"))

    best_bps <- matrix(bpkeep, nrow = 1L, dimnames = list(NULL, "Best_BPs1"))

    if (verbose) {
      cat(sprintf("Done. %s models tested, %s models saved.\n",
                  nrow(res), nrow(res)))
    }
  }
  else {
    # 3 and more Regions (2+ bp)

    # Define BPs to include if non-exhaustive search:
    if (!exhaus) {

      stats <- regions_results$stats
      max_nregion_so_far <- max(stats$Nregions[stats$Nregions < noregions])
      best_bps <- unlist(stats[stats$Nregions == max_nregion_so_far,
                               startsWith(names(stats), "Best_BPs")])
      bpkeep <- .drop_na(best_bps)

      if (length(bpkeep) == 0 && max_nregion_so_far > 2) {
        # Find best_bps
        res0 <- regions_results$res[regions_results$res$regions == max_nregion_so_far, , drop = FALSE]
        bestBPs <- drop(as.matrix(res0)[which.min(res0$sumRSS), startsWith(names(res0), "breakpoint")])

        # Add 3 BPs left and right to best BPs
        bestBPs <- lapply(bestBPs, function(b) {
          bpp <- match(b, eligible_vertebrae) + seq(-3, 3)

          # Convert position of bp to keep to actual value of bp
          eligible_vertebrae[bpp[bpp %in% seq_along(eligible_vertebrae)]]
        })

        bpkeep <- vapply(seq_along(bestBPs), function(i) {
          paste(
            sort(.drop_na(unique(c(res0[, i + 1L], bestBPs[[i]]), nmax = length(eligible_vertebrae)))),
            collapse = "|"
          )
        }, character(1L))

        regions_results$stats[stats$Nregions == max_nregion_so_far,
                              startsWith(names(stats), "Best_BPs")] <- bpkeep
      }
    }

    # Number of combinations
    ncombi <- .ncombos(length(eligible_vertebrae), minvert, nbp)

    # Max length for an R object to not exceed 2Gb in size (Memory safety net)
    # lim <- 5e7

    # Find nrow max for given number of cols (bps) to avoid exceeding 2Gb objects
    # max_rows <- ceiling(lim/nbp)

    if (ncombi <= ncombos_file_trigger) {

      goodcomb <- .combos(eligible_vertebrae, minvert, nbp, includebp = includebp)

      if (nrow(goodcomb) == 0L) {
        return(regions_results)
      }

      nmodel_possible <- nrow(goodcomb)

      if (!exhaus && length(bpkeep) > 0L) {
        # Keep only probable combinations (non-exhaustive search)
        for (m in strsplit(bpkeep, "|", fixed = TRUE)) {
          mn <- as.numeric(m)
          goodcomb <- goodcomb[.any_mat_in(goodcomb, mn), , drop = FALSE]
        }
      }

      if (nrow(goodcomb) == 0) {
        return(regions_results)
      }

      if (verbose) {
        cat(sprintf("Fitting models with %s regions (%s breakpoints)...\n",
                    nbp + 1, nbp))
      }

      # Run fitting on good combinations:
      res <- do.call("rbind", pbapply::pbapply(goodcomb, 1L, function(bps) {
        fregions(bps, Xvar = Xvar, Yvar = Yvar, w = w)
      }, simplify = FALSE, cl = cl))

      res <- setNames(cbind(nbp + 1L,
                            as.data.frame(goodcomb),
                            as.data.frame(res)),
                      colhead)

      nmodel_tested <- nrow(res)

      if (!exhaus) {
        # If non-exhaustive search, keep only models with sumRSS <= min(sumRSS)+(0.5*sd(sumRSS))
        cutoff <- min(res$sumRSS) + sd(res$sumRSS) / 2
        if (!is.na(cutoff)) {
          # Subsample res only if more than 1 option
          res <- res[res$sumRSS <= cutoff, , drop = FALSE]
        }
      }
    }
    else {

      #Check directory
      if (!dir.exists(temp_file_dir)) {
        dir.create(dir.exists)
      }

      # Max number of rows per iteration; keeps sizes small and
      # improves progress bar
      max_rows <- ceiling(ncombos_file_trigger / 100)

      # Required number of splits of combos
      aa <- ceiling(ncombi / max_rows)

      # File names to store intermediate res objects
      files <- character(aa)
      # files <- vapply(1:aa, function(i) tempfile("regions_", fileext = ".Rds"),
      #                 character(1L))
      on.exit(unlink(files))

      # Initialize statistics
      nmodel_possible <- nmodel_tested <- 0
      last_combo <- NULL

      n_total <- 0
      if (!exhaus) {
        min_RSS <- Inf
        Xbar_total <- M2_total <- 0
      }

      for (i in 1:aa) {

        # Only returns next max_rows rows of combos, using last_combo as
        # starting place
        goodcomb <- .combos(eligible_vertebrae, minvert, nbp, ncombos = max_rows,
                            last_combo = last_combo,
                            includebp = includebp)

        if (i == 1) {
          if (nrow(goodcomb) == 0) {
            return(regions_results)
          }

          if (verbose) {
            cat(sprintf("Fitting models with %s regions (%s breakpoints)...\n",
                        nbp + 1, nbp))
          }

          # Initialize progress bar (pb). Won't activate if `verbose = FALSE`
          pb <- pbapply::startpb(0, ncombi)
        }

        last_combo <- goodcomb[nrow(goodcomb), ]
        nmodel_possible <- nmodel_possible + nrow(goodcomb)

        if (!exhaus) {
          for (m in strsplit(bpkeep, "|", fixed = TRUE)) {
            mn <- as.numeric(m)
            goodcomb <- goodcomb[.any_mat_in(goodcomb, mn), , drop = FALSE]
          }
        }

        if (nrow(goodcomb) == 0) {
          pbapply::setpb(pb, min(i * max_rows, ncombi))
          next
        }

        res <- do.call("rbind", .lapply_selector(seq_len(nrow(goodcomb)), function(g) {
          fregions(goodcomb[g, ], Xvar = Xvar, Yvar = Yvar, w = w)
        }, cl = cl))

        res <- setNames(cbind(nbp + 1L,
                              as.data.frame(goodcomb),
                              as.data.frame(res)),
                        colhead)

        nmodel_tested <- nmodel_tested + nrow(res)

        # Save intermediate object to disk
        files[i] <- tempfile("regions_", tmpdir = temp_file_dir, fileext = ".Rds")
        saveRDS(res, files[i])

        n_old <- n_total #old n
        n_i <- nrow(res) #n of new subset
        n_total <- n_old + n_i #total n so far

        if (!exhaus) {
          #Accumulate values for cutoff
          Xbar_old <- Xbar_total #old mean
          Xbar_i <- sum(res$sumRSS) / n_i #mean of new subset
          Xbar_total <- (n_old * Xbar_old + n_i * Xbar_i) / (n_old + n_i) #new mean

          delta <- Xbar_i - Xbar_old

          M2_old <- M2_total #old M2
          M2_i <- sum((res$sumRSS - Xbar_i)^2) #M2 of new subset

          M2_total <- M2_old + M2_i + delta^2 * n_old * n_i / (n_old + n_i) #new M2

          min_RSS <- min(min_RSS, res$sumRSS) #new min
        }

        pbapply::setpb(pb, nmodel_possible)
      }

      pbapply::setpb(pb, ncombi)
      pbapply::closepb(pb)

      if (!exhaus) {
        sd_RSS <- sqrt(M2_total / (nmodel_tested - 1))
        cutoff <- min_RSS + sd_RSS / 2
      }

      if (verbose) {
        cat("Reading files...\n")
      }

      files <- files[files != character(1L)]

      res <- do.call("rbind", .lapply_selector(files, function(file) {
        tryCatch({res_tmp <- readRDS(file)},
                 error = function(err) {
                   e <- conditionMessage(err)

                   if (!grepl("cannot open connection", e)) {
                     arg::err("{e}")
                   }

                   if (!dir.exists(temp_file_dir)) {
                     arg::err("the directory containing the stored files has been deleted. See {.help calcregions} for more information. You will likely have to re-run the function")
                   }

                   arg::err("at least one of the stored files cannot be found, suggesting that something has happened to the directory in which the files were stored. See {.help calcregions} for more information. You will likely have to re-run the function")
                 })

        if (exhaus) {
          return(res_tmp)
        }

        # If non-exhaustive search, keep only models with sumRSS <= min(sumRSS)+(0.5*sd(sumRSS))
        res_tmp[res_tmp$sumRSS <= cutoff, , drop = FALSE]

      }, cl = cl))

      unlink(files)
    }

    nmodel_saved <- nrow(res)

    # Create string with info on best BPs to use for next region:	# BPs kept are
    # all  > cutoff (min+0.5*sd) and BPs of best models +/- 3 vertebrae
    if (!exhaus) {
      bestBPs <- drop(res[which.min(res$sumRSS), startsWith(names(res), "breakpoint")])

      # Add 3 BPs left and right to best BPs
      bestBPs <- lapply(bestBPs, function(x) {
        #Convert bp value to position, add range
        bpp <- match(x, eligible_vertebrae) + seq(-3, 3)
        # Convert position of bp to keep to actual value of bp
        eligible_vertebrae[bpp[bpp > 0]]
      })

      bpkeep <- vapply(seq_along(bestBPs), function(i) {
        paste(
          sort(.drop_na(unique(c(res[, i + 1L], bestBPs[[i]]), nmax = length(eligible_vertebrae)))),
          collapse = "|"
        )
      }, character(1L))

      stats <- list2DF(list(Nregions = noregions,
                            Nmodel_possible = nmodel_possible,
                            Nmodel_tested = nmodel_tested,
                            Nmodel_saved = nmodel_saved,
                            Comp_method = "Non-exhaus",
                            Saving_method = "SD/2"))
    }
    else {
      bpkeep <- rep.int(NA_character_, nbp)

      stats <- list2DF(list(Nregions = noregions,
                            Nmodel_possible = nmodel_possible,
                            Nmodel_tested = nmodel_tested,
                            Nmodel_saved = nmodel_saved,
                            Comp_method = "Exhaustive",
                            Saving_method = "All"))
    }

    best_bps <- matrix(bpkeep, nrow = 1L, dimnames = list(NULL, paste0("Best_BPs", 1:nbp)))

    if (verbose) {
      cat(sprintf("Done. %s models tested, %s models saved.\n",
                  nmodel_tested, nmodel_saved))
    }
  }

  rownames(res) <- NULL

  stats <- cbind(stats, as.data.frame(best_bps))

  #Remove analysis with same number of regions
  if (any(regions_results$results$regions == noregions)) {
    regions_results$results <- regions_results$results[regions_results$results$regions != noregions, , drop = FALSE]
  }

  if (any(regions_results$stats$Nregions == noregions)) {
    regions_results$stats <- regions_results$stats[regions_results$stats$Nregions != noregions, , drop = FALSE]
  }

  #Combine res and stats with exist components of input object
  regions_results$results <- .rbind_larger(regions_results$results, res)
  regions_results$stats <- .rbind_larger(regions_results$stats, stats)

  regions_results$results <- regions_results$results[order(regions_results$results$regions), , drop = FALSE]
  regions_results$stats <- regions_results$stats[order(regions_results$stats$Nregions), , drop = FALSE]

  rownames(regions_results$results) <- NULL
  rownames(regions_results$stats) <- NULL

  regions_results
}

# Returns number of combos possible; vectorized across nvert
.ncombos <- function(nvert, minvert, nbp) {
  if (nbp == 0) {
    return(rep.int(1, length(nvert)))
  }

  # Number of possibilities for each BP
  p <- nvert - minvert * (nbp + 1) + 1

  out <- rep.int(0, length(nvert))

  if (all(p <= 0)) {
    return(out)
  }

  # Number of combinations - this is magic, kind of, but it works
  # prod(p - 1 + 1:nbp)/prod(1:nbp)
  out[p > 0] <- choose(p[p > 0] - 1 + nbp, nbp)

  out
}

# Returns all combos satisfying rules by dispatching to .combosR_slow()
# or .combosR_quick() depending on rules
.combos <- function(vert, minvert, nbp, ncombos = Inf, last_combo = NULL,
                    includebp = NULL) {

  maxcombos <- .ncombos(length(vert), minvert, nbp)

  if (!is.null(last_combo) ||
      ncombos < maxcombos ||
      maxcombos == 0 ||
      (!is.null(includebp) && any(diff(includebp) < minvert))) {
    return(.combosR_slow(vert, minvert, nbp, ncombos, last_combo,
                         includebp))
  }

  out <- .combosR_quick(vert, minvert, nbp)

  if (!is.null(includebp)) {
    out <- out[.all_mat_in(out, includebp), , drop = FALSE]
  }

  out
}

# Returns matrix of all combos, uses counter and allows other options
.combosR_slow <- function(vert, minvert, nbp, ncombos = Inf, last_combo = NULL,
                          includebp = NULL) {

  if (!is.null(includebp) && length(includebp) == nbp) {
    combos <- matrix(includebp, nrow = 1, ncol = nbp)

    colnames(combos) <- paste0("breakpoint", seq_len(nbp))
    rownames(combos) <- NULL

    return(combos)
  }

  n <- length(vert)

  y <- seq_len(n)

  if (!is.null(includebp)) {
    includebp <- match(includebp, vert)
    minvert_override <- min(includebp, diff(includebp), minvert)
  }
  else {
    minvert_override <- minvert
  }

  mins <- minvert_override * seq_len(nbp)
  maxes <- n - rev(mins)

  maxcombos <- .ncombos(n, minvert_override, nbp)

  ncombos <- min(ncombos, maxcombos)
  combos <- matrix(NA_integer_, nrow = ncombos, ncol = max(nbp, 1))

  colnames(combos) <- paste0("breakpoint", seq_len(max(nbp, 1)))
  rownames(combos) <- NULL

  if (ncombos == 0 || nbp == 0) {
    return(combos)
  }

  reset0 <- rep.int(FALSE, nbp)

  if (is.null(last_combo)) {
    counters <- mins
  }
  else {
    counters <- match(last_combo, vert)

    #Increment combo before inserting into combos matrix
    reset <- reset0
    counters[nbp] <- counters[nbp] + 1

    i <- nbp
    while (i > 1L && counters[i] > maxes[i]) {
      reset[i] <- TRUE
      counters[i - 1L] <- counters[i - 1L] + 1
      i <- i - 1L
    }
    if (any(reset)) {
      for (i in which(reset)) {
        counters[i] <- counters[i - 1L] + minvert_override
      }
    }
  }

  k <- 1L

  while (k <= nrow(combos) && counters[1L] <= maxes[1L]) {
    reset <- reset0

    if (is.null(includebp) || all(includebp %in% counters)) {
      ok <- TRUE

      if (minvert_override != minvert) {
        # If any violations of minvert aren't in includebp, move on
        for (i in which(diff(y[counters]) < minvert)) {
          if (!all(counters[c(i, i + 1L)] %in% includebp)) {
            ok <- FALSE
            break
          }
        }
      }

      if (ok) {
        combos[k, ] <- counters
        k <- k + 1L
      }
    }

    counters[nbp] <- counters[nbp] + 1L
    i <- nbp
    while (i > 1L && counters[i] > maxes[i]) {
      reset[i] <- TRUE
      counters[i - 1L] <- counters[i - 1L] + 1L
      i <- i - 1L
    }

    if (any(reset)) {
      for (i in which(reset)) {
        counters[i] <- counters[i - 1L] + minvert_override
      }
    }
  }

  combos <- combos[seq_len(k - 1L), , drop = FALSE]
  combos[] <- vert[combos]

  combos
}

#Bare bones, uses recursive solution from https://stackoverflow.com/a/76975508/6348551
#(fastest)
.combosR_quick <- function(vert, minvert, nbp) {

  y <- minvert:(length(vert) - minvert)
  vn <- y[length(y)]

  f0 <- function(v, k, m) {
    if (k == 1L) {
      return(matrix(v))
    }

    d <- Recall(v, k - 1L, m)

    u <- (m * (k - 1L)):(vn - m)

    lst <- lapply(u, function(i) {
      p <- (i + m):vn
      dd <- d[d[, k - 1L] == i, , drop = FALSE]
      cbind(dd[rep(seq_len(nrow(dd)), each = length(p)), , drop = FALSE], p)
    })

    unname(do.call("rbind", lst))
  }

  combos <- f0(y, nbp, minvert)

  colnames(combos) <- paste0("breakpoint", seq_len(nbp))

  combos <- combos[do.call("order", as.data.frame(combos)), , drop = FALSE]

  combos[] <- vert[combos]

  combos
}

.design_matrix <- function(Xvar, BPs, cont) {
  if (length(BPs) == 0) {
    return(cbind(1, Xvar))
  }

  bpmat <- matrix(BPs,
                  ncol = length(BPs),
                  nrow = length(Xvar),
                  byrow = TRUE)

  if (cont) {
    # Continuous fit
    cbind(1, Xvar, .pmax2(Xvar - bpmat, 0))
  }
  else {
    # Discontinuous fit
    m <- cbind(TRUE, Xvar > bpmat) & cbind(Xvar <= bpmat, TRUE)

    cbind(1 * m, Xvar * m)
  }
}

#' Assess model performance
#'
#' `modelperf()` computes model performance statistics in the form of \eqn{R^2} measures for a given combination of breakpoints.
#'
#' @param x a `regions_pco` object, the output of a call to [svdPCO()], or a `regions_results_single` object, the output of a call to [calcmodel()].
#' @param scores `numeric`; the indices of the PCO scores for which fit statistics are to be computed.
#' @param modelsupport a `regions_modelsupport` object, the output of a call to [modelsupport()]. When `x` is a `regions_pco` object, either `modelsupport`, `criterion`, and `model` must be supplied or `bps` and `cont` must be supplied. See Details.
#' @param criterion string; the criterion to use to select the best model for which breakpoints are to be displayed when `modelsupport` is specified. Ignored otherwise. Allowable options include `"aic"` to use the AICc and `"bic"` to use the BIC. Abbreviations allowed. Default is `"aic"`. When `x` is a `regions_pco` object, either `modelsupport`, `criterion`, and `model` must be supplied or `bps` and `cont` must be supplied. See Details.
#' @param model `numeric`; for which model among the best as determined by `criterion` should fit statistics be computed. 1 is the best model, 2 the second best, etc. Default is 1. When `x` is a `regions_pco` object, either `modelsupport`, `criterion`, and `model` must be supplied or `bps` and `cont` must be supplied. See Details.
#' @param bps `numeric`; a vector of breakpoints for which model fit should be computed. When `x` is a `regions_pco` object, either `modelsupport`, `criterion`, and `model` must be supplied or `bps` and `cont` must be supplied. See Details.
#' @param cont `logical`; whether to fit a model that is continuous (`TRUE`) or discontinuous (`FALSE`) at the breakpoints supplied to `bps`. Default is `TRUE`. When `x` is a `regions_pco` object, either `modelsupport`, `criterion`, and `model` must be supplied or `bps` and `cont` must be supplied. See Details.
#' @param \dots ignored.
#'
#' @return A `regions_perf` object containing the breakpoints of the specified model, the univariate \eqn{R^2} and adjusted \eqn{R^2} statistics for each PCO score, and the multivariate \eqn{R^2} and adjusted \eqn{R^2} statistics.
#'
#' @details `modelperf()` operates on a single model identified by breakpoints and whether the model is continuous or not. When `x` is a `regions_pco` object, the model is selected either as the best model in the supplied `modelsupport` object (where "best" is determined by the arguments to `criterion` and `model`) or as specified by the user using the arguments to `bps` and `cont`. When `x` is a `regions_results_single` object, the breakpoints and model form are determined based on the supplied object.
#'
#' @seealso [modelsupport()] for assessing model support using information criteria; [calcmodel()] for fitting a single segmented regression model; [plotsegreg()] for plotting the results of a single segmented regression model.
#'
#' @example man/examples/example-modelperf.R
#'

#' @rdname modelperf
#' @export
modelperf <- function(x, ...) {
  UseMethod("modelperf")
}

#' @exportS3Method modelperf regions_pco
#' @rdname modelperf
modelperf.regions_pco <- function(x, scores,
                                  modelsupport = NULL, criterion = "aic", model = 1,
                                  bps = NULL, cont = TRUE, ...) {
  arg::arg_supplied(scores)
  arg::arg_whole_numeric(scores)
  arg::arg_between(scores, c(1L, ncol(x[["scores"]])))

  Xvar <- .get_pos(x)
  Yvar <- x[["scores"]][, scores, drop = FALSE]

  if (!is.null(bps)) {
    if (is.atomic(bps) && all(is.na(bps))) {
      bps <- NA_real_
      cont <- TRUE
    }
    else {
      arg::arg_numeric(bps)
      arg::arg_between(bps, range(Xvar))
      arg::arg_flag(cont)
    }

    BPs <- sort(.drop_na(bps))

    names(BPs) <- paste0("breakpoint", seq_along(BPs))
  }
  else if (!is.null(modelsupport)) {
    arg::arg_is(modelsupport, "regions_modelsupport")

    criterion <- arg::match_arg(criterion, c("aic", "bic"))
    model_support_crit <- modelsupport[[switch(criterion, aic = "Model_support", bic = "Model_support_BIC")]]

    arg::when_not_null(
      model,
      arg::arg_whole_number,
      arg::arg_between(c(1, nrow(model_support_crit)))
    )

    if (is.null(model)) {
      model <- 1L
    }

    cont <- attr(modelsupport, "cont")

    keep <- which(startsWith(names(model_support_crit), "breakpoint"))
    BPs <- .drop_na(unlist(model_support_crit[model, keep]))
  }
  else {
    arg::err("{.arg bps} or {.arg modelsupport} must be provided")
  }

  .modelperf_internal(Xvar, Yvar, BPs, cont)
}

#' @exportS3Method modelperf regions_sim
#' @rdname modelperf
modelperf.regions_sim <- function(x, scores = NULL,
                                  modelsupport = NULL, criterion = "aic", model = 1,
                                  bps = NULL, cont = TRUE, ...) {
  Xvar <- x$Xvar
  Yvar <- x$Yvar

  arg::when_not_null(
    scores,
    arg::arg_whole_numeric,
    arg::arg_between(c(1, ncol(Yvar)))
  )

  if (!is.null(scores)) {
    Yvar <- Yvar[, scores, drop = FALSE]
  }

  if (!is.null(bps)) {
    if (is.atomic(bps) && all(is.na(bps))) {
      bps <- NA_real_
      cont <- TRUE
    }
    else {
      arg::arg_numeric(bps)
      arg::arg_between(bps, range(Xvar))
      arg::arg_flag(cont)
    }

    BPs <- sort(.drop_na(bps))

    names(BPs) <- paste0("breakpoint", seq_along(BPs))
  }
  else if (!is.null(modelsupport)) {
    arg::arg_is(modelsupport, "regions_modelsupport")

    criterion <- arg::match_arg(criterion, c("aic", "bic"))
    model_support_crit <- modelsupport[[switch(criterion, aic = "Model_support", bic = "Model_support_BIC")]]

    arg::when_not_null(
      model,
      arg::arg_whole_number,
      arg::arg_between(c(1, nrow(model_support_crit)))
    )

    if (is.null(model)) {
      model <- 1L
    }

    cont <- attr(modelsupport, "cont")

    keep <- which(startsWith(names(model_support_crit), "breakpoint"))
    BPs <- .drop_na(unlist(model_support_crit[model, keep]))
  }
  else {
    arg::err("{.arg bps} or {.arg modelsupport} must be provided")
  }

  .modelperf_internal(Xvar, Yvar, BPs, cont)
}

#' @exportS3Method modelperf regions_results_single
#' @rdname modelperf
modelperf.regions_results_single <- function(x, scores = NULL, ...) {
  Xvar <- attr(x, "pos")
  Yvar <- attr(x, "scores")
  BPs <- unlist(x$results[startsWith(names(x$results), "breakpoint")])
  cont <- attr(x, "cont")

  arg::when_not_null(
    scores,
    arg::arg_whole_numeric,
    arg::arg_between(c(1, ncol(Yvar)))
  )

  if (!is.null(scores)) {
    Yvar <- Yvar[, scores, drop = FALSE]
  }

  .modelperf_internal(Xvar, Yvar, BPs, cont)
}

#' @exportS3Method print regions_perf
print.regions_perf <- function(x, digits = 3, ...) {
  x0 <- x
  cat("Breakpoints:", if (length(x[["BPs"]]) == 0) "(none)"
      else toString(x[["BPs"]]), "\n")
  cat("\n- Univariate:\n")
  colnames(x$univariate) <- c("R\u00B2", "Adj. R\u00B2")
  print(round(x$univariate, digits))
  cat("\n- Multivariate:\n")
  names(x$multivariate) <- c("R\u00B2", "Adj. R\u00B2")
  print(round(x$multivariate, digits))

  invisible(x0)
}

.modelperf_internal <- function(Xvar, Yvar, BPs, cont) {

  #Calculate weights to ensure each vertebra counts equally
  vert_tab <- tabulate(Xvar)
  w <- 1 / vert_tab[Xvar]

  totrsq <- do.call("rbind", lapply(seq_len(ncol(Yvar)), function(i) {
    x <- .design_matrix(Xvar, BPs, cont)
    fit <- .fast_lm(x = x, y = Yvar[, i], w = w)

    n <- length(Xvar)
    fitted <- Yvar[, i] - fit$residuals
    mss <- sum(w * (fitted - sum(w * fitted) / sum(w))^2)
    rss <- sum(w * fit$residuals^2)
    rdf <- n - fit$rank
    r.squared <- mss / (mss + rss)

    c(r2 =     r.squared,
      r2.adj = 1 - (1 - r.squared) * ((n - 1) / rdf),
      SSres =  rss,
      df =     fit$rank + rdf - 1,
      SStot =  mss + rss,
      dfe =    rdf)
  }))

  rownames(totrsq) <- paste0("PCO.", seq_len(ncol(Yvar)))

  # Calculate R2 for all PCOs multivariately:
  tot.rsq <- colSums(totrsq[, -(1:2), drop = FALSE])
  ord.tot.rsq <- unname(1 - tot.rsq["SSres"] / tot.rsq["SStot"])
  adj.tot.rsq <- unname(1 - (tot.rsq["SSres"] / tot.rsq["dfe"]) / (tot.rsq["SStot"] / tot.rsq["df"]))

  out <- list(BPs = BPs,
              univariate = totrsq[, 1:2],
              multivariate = c(r2 = ord.tot.rsq, r2.adj = adj.tot.rsq))
  attr(out, "cont") <- cont

  class(out) <- "regions_perf"

  out
}

#' Plot PC axes
#'
#' `plot()` visualizes the relationship between a PCO axis and the vertebra or between pairs of PCO axes.
#'
#' @param x a `regions_pco` object; the output of a call to [svdPCO()].
#' @param pc_y,pc_x number; PC score indices for the y- and x-axes, respectively. `pc_x` can be `NULL`.
#' @param ... arguments passed to [ggplot2::geom_text()] when `pc_x` is not `NULL`. If `scores` is supplied as an argument, it will replace `pc_y` if unspecified.
#'
#' @return A `ggplot` object.
#'
#' @details
#' When `pc_x` is `NULL` (the default), `plot()` will display a scatterplot of the PCO axis identified by `pc_y` and vertebra position using [ggplot2::geom_point()]. This plot is similar to that generated by [plotsegreg()]. Otherwise, `plot()` uses [ggplot2::geom_text()] to identify vertebrae positions in the space corresponding to the requested PCOs.
#'
#' @seealso
#'
#' [svdPCO()] for generating the PCO scores. [plot.regions_sim()] for plotting PCO scores against vertebra position for simulated PCOs. [plotsegreg()] for plotting PCO scores against vertebra position after selecting breakpoints for a segmented regression.
#'
#' @example man/examples/example-svdPCO.R

#' @exportS3Method plot regions_pco
plot.regions_pco <- function(x, pc_y = 1, pc_x = NULL, ...) {

  if (identical(pc_y, formals()$pc_y)) {
    dotnames <- ...names()
    if ("scores" %in% dotnames) {
      scores <- ...elt(which(dotnames == "scores")[1L])
      arg::arg_whole_numeric(scores)
      if (!is.null(pc_x) && length(pc_y) > 1L) {
        arg::err("{.arg scores} can only have length 1 when {.arg pc_x} is specified")
      }
      arg::arg_between(scores, c(1, ncol(x[["scores"]])))
      pc_y <- scores
    }
  }

  pos <- .get_pos(x)
  specimen <- attr(x, "specimen")

  if (is.null(pc_x)) {
    arg::arg_whole_numeric(pc_y)
    arg::arg_between(pc_y, c(1, ncol(x[["scores"]])))

    pc_y <- sort(pc_y)

    Yvar <- x[["scores"]][, pc_y, drop = FALSE]

    # Plot PCO against vertebrae using plotsegreg() machinery
    return(.plotreg_internal(Xvar = pos, Yvar = Yvar, lines = FALSE,
                             scores = pc_y, specimen = specimen))
  }

  arg::arg_whole_numeric(pc_y)
  if (length(pc_y) > 1) {
    arg::err("{.arg pc_y} can only have length 1 when {.arg pc_x} is specified")
  }
  arg::arg_between(pc_y, c(1, ncol(x[["scores"]])))

  Yvar <- x[["scores"]][, pc_y]

  arg::arg_whole_number(pc_x)
  arg::arg_between(pc_x, c(1, ncol(x[["scores"]])))

  Xvar <- x[["scores"]][, pc_x]

  p <- ggplot(mapping = aes(x = Xvar,
                            y = Yvar,
                            label = pos,
                            color = if (nlevels(specimen) > 1) specimen)) +
    geom_text(...) +
    geom_point(alpha = 0) +
    theme_bw() +
    guides(color = guide_legend(override.aes = aes(label = "", alpha = 1))) +
    labs(x = paste("PC", pc_x),
         y = paste("PC", pc_y),
         color = NULL)

  p
}

data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Calculate a single segmented regression model
# using first 2 PCOs and a discontinuous model
regionsmodel <- calcmodel(alligator_PCO,
                          scores = 1:3,
                          bps = c(8, 16, 21),
                          cont = FALSE)

regionsmodel

#Evaluate performance (R2) on that model
modelperf(regionsmodel)

#Plot model results:
plotsegreg(regionsmodel, scores = 1:3)

# Load dataset; vertebral index in first column
# ('Vertebra' column)
data("dolphin")

# Compute PC scores with prcomp:
PCA <- prcomp(dolphin[-1], scale = TRUE)

# Extract PC scores and eigenvalues:
PCA_scores <- as.data.frame(PCA$x)
PCA_eigenval <- PCA$sdev^2


# Process PC scores and morphological data:
pco_dolphin <- process_PC(data = dolphin,
                          pos = "Vertebra",
                          pcscores = PCA_scores,
                          eigenvals = PCA_eigenval,
                          posPC = dolphin[[1]])

#' Evaluate model support
#'
#' `modelsupport()` computes measures of the relative support of each of the best models identified by [modelselect()] to facilitate selecting the optimal number and position of regions. These measures are in the form of information criteria (AICc and BIC).
#'
#' @param models a `regions_modelselect` object; the output of a call to [modelselect()].
#'
#' @returns A `regions_modelsupport` object, which contains the best model for each number of regions as determined by the AICc and BIC. The computed statistics are `AICc`/`BIC`--the value of the information criterion (IC) for each model, `deltaAIC`/`deltaBIC`--the difference between the IC for the corresponding model and that of the model with the lowest IC value, `model_lik`--the likelihood ratio of the model against the model with the lowest IC value, and `Ak_weight`/`BIC_weight`--the Akaike weights for each model used to compute the region score. The region score is a weighted average of the numbers of regions, weighted by the Akaike weights to represent the variability around the optimal number of regions.
#'
#' @seealso [modelselect()], [calcregions()], [calcBPvar()], [modelperf()], [plotsegreg()]
#'
#' @example man/examples/example-modelselect.R
#'

#' @export
modelsupport <- function(models) {

  arg::arg_supplied(models)
  arg::arg_is(models, "regions_modelselect")

  cont <- attr(models, "cont")
  nvert <- attr(models, "nvert")

  nPC <- sum(startsWith(names(models), "RSS."))

  models <- models[!startsWith(names(models), "RSS.")]

  AICc <- numeric(nrow(models))
  BIC <- numeric(nrow(models))

  for (i in seq_len(nrow(models))) {
    probs <- .AICcalc(models$sumRSS[i], nPC, nvert,
                      models$regions[i], cont) #Calculate AIC score

    AICc[i] <- probs["AICc"]
    BIC[i] <- probs["BIC"]
  }

  # AICc:
  deltaAIC <- AICc - min(AICc)	#Calculate AIC difference
  model_lik <- exp(-deltaAIC / 2)	#Likelihood of the model

  Ak_weight <- model_lik / sum(model_lik)	#Akaike Weights

  AIC_models <- cbind(models, AICc, deltaAIC, model_lik, Ak_weight)
  AIC_models <- AIC_models[order(AIC_models$AICc), , drop = FALSE] #Sort so best at top

  Regions_score <- sum(AIC_models$regions * AIC_models$Ak_weight)

  # BIC:
  deltaBIC <- BIC - min(BIC)	#Calculate AIC difference
  model_lik <- exp(-deltaBIC / 2)	#Likelihood of the model

  BIC_weight <- model_lik / sum(model_lik)	#BIC Weights

  BIC_models <- cbind(models, BIC, deltaBIC, model_lik, BIC_weight)
  BIC_models <- BIC_models[order(BIC_models$BIC), , drop = FALSE] #Sort so best at top

  Regions_scoreB <- sum(BIC_models$regions * BIC_models$BIC_weight)

  out <- list(Model_support = AIC_models,
              Region_score = Regions_score,
              Model_support_BIC = BIC_models,
              Region_score_BIC = Regions_scoreB)

  attr(out, "cont") <- cont

  class(out) <- "regions_modelsupport"

  out
}

#' @exportS3Method print regions_modelsupport
print.regions_modelsupport <- function(x, digits = 3, ...) {
  x0 <- x
  for (m in c("Model_support", "Model_support_BIC")) {
    for (i in which(startsWith(names(x[[m]]), "breakpoint"))) {
      x[[m]][[i]] <- ifelse(is.na(x[[m]][[i]]), ".",
                            format(x[[m]][[i]], justify = "right"))
      names(x[[m]]) <- sub("breakpoint", "BP ", names(x[[m]]), fixed = TRUE)
    }

    for (i in setdiff(names(x[[m]]),
                      c("regions", names(x[[m]])[startsWith(names(x[[m]]), "BP ")]))) {
      x[[m]][[i]] <- round(x[[m]][[i]], digits)
    }

    names(x[[m]])[1] <- "Regions"
  }

  cat("- Model support (AICc)\n")
  print.data.frame(x[["Model_support"]], row.names = FALSE)
  cat("Region score:", round(x[["Region_score"]], 2), "\n\n")

  cat("- Model support (BIC)\n")
  print.data.frame(x[["Model_support_BIC"]], row.names = FALSE)
  cat("Region score:", round(x[["Region_score_BIC"]], 2), "\n")

  invisible(x0)
}

#' Plot a vertebra map
#'
#' `plotvertmap()` plots a map of the supplied vertebrae, optionally adding colors, marks, and text to identify existing and estimated features of the vertebrae.
#'
#' @param x a `regions_data`, `regions_pco`, or `regions_sim` object; the output of a call to [process_measurements()], [svdPCO()], or [simregions()], respectively.
#' @param type string; the labeling of the x-axis of the plot. Either `"count"` to identify the vertebra index (or absolute position when `centraL` is supplied) or `"percent"` to identify the percent vertebra count (or percent total length when `centraL` is supplied). Abbreviations allowed. Default is `"count"`.
#' @param bps an optional vector containing the region breakpoints. One of `bps`, `modelsupport`, or `bpvar` should be specified to display regions breakpoints. See Details.
#' @param modelsupport an optional `regions_modelsupport` object; the output of a call to [modelsupport()]. One of `bps`, `modelsupport`, or `bpvar` should be specified to display regions breakpoints. See Details.
#' @param criterion string; the criterion to use to select the best model for which breakpoints are to be displayed when `modelsupport` is specified. Ignored otherwise. Allowable options include `"aic"` to use the AICc and `"bic"` to use the BIC. Abbreviations allowed. Default is `"aic"`.
#' @param model `numeric`; from which model among the best as determined by `criterion` should breakpoints be selected when `modelsupport` is supplied. Ignored otherwise. 1 is the best model, 2 the second best, etc. Default is 1.
#' @param bpvar an optional `regions_BPvar` object; the output of a call to [calcBPvar()]. One of `bps`, `modelsupport`, or `bpvar` should be specified to display regions breakpoints. See Details.
#' @param bp.sd an optional vector of the standard deviations of the breakpoints (e.g., as calculated by [calcBPvar()]). When `bpvar` is supplied, the weighted standard deviations are used.
#' @param sd.col when `bp.sd` is specified, the color of the mark on plot indicating the standard deviations. Default is black.
#' @param dropNA `logical`; when some vertebrae are missing, e.g., due to subsampling or starting the analysis at a vertebra beyond the first, whether to remove the missing vertebrae from the plot (`TRUE`) or retain them and label them as missing (i.e., lacking a region) (`FALSE`). Default is `FALSE` to retain them.
#' @param text `logical`; whether to print the vertebra index on each vertebra. Default is `FALSE`.
#' @param name an optional string containing a label used on the left side of the plot.
#' @param centraL an optional numeric vector containing centrum length for each vertebra, which is used to change the size of the plotted vertebrae, or a string containing the name of the variable in the original dataset containing centrum length. Should be of length equal to the number of included vertebrae (i.e., the length of the original dataset). Any vertebrae with centrum length of 0 will be omitted.
#' @param reg.lim a vector of breakpoints indicating other region limits, e.g., anatomic regions.
#' @param lim.col when `reg.lim` is specified, the color of the lines separating the regions. Default is black.
#' @param block.cols when breakpoints are specified (i.e., using `bps`, `modelsupport`, or `bpvar`) and `block.lim` is not specified, a vector of color names or hex codes, one for each region. If not specified, [RColorBrewer::brewer.pal()] with `name = "paired"` will be used to generate colors. When `block.lim` is specified, a named list of vectors of color names or hex codes. See Details.
#' @param block.lim a vector of breakpoints indicating the limits of traditional regions, which will be colored using `block.cols`. See Details.
#'
#' @returns A `ggplot` object that can be manipulated using `ggplot2` syntax.
#'
#' @details
#'
#' `plotvertmap()` uses [ggplot2::geom_rect()] to create the plot. The plots are best viewed with a short height and a long width.
#'
#' ### Specifying breakpoints:
#'
#' There are three ways to specify regions in `plotvertmap()`. First is to supply the vector of breakpoints directly to `bps`. Second is to supply a `regions_modelsupport` object to `modelsupport`. When supplied, the `criterion` and `model` arguments can be used to select which of the sets of breakpoints in the object is to be used. `model` selects which breakpoint model is to be used (1 for first best, 2 for second best, etc.), and `criterion` determines which criterion (AICc or BIC) is used to rank the models. Third is to supply ` regions_BPvar` object to `bpvar`. The weighted average breakpoints will be used after rounding (e.g., a weighted average breakpoint of 3.3 will place vertebrae 1, 2, and 3 in a region, and a weighted average breakpoint of 3.9 will place vertebrae 1, 2, 3, and 4 in a region).
#'
#' ### Using `block.cols`:
#'
#' When `block.lim` is specified, `block.cols` must be specified as a list of vectors of colors, with an entry for each "block". Blocks are predefined regions separate from those specified using the above arguments, e.g., traditional regions. For each region, the most common block is found and assigned to that region. A color of that block as supplied in `block.cols` is used to color that region. So, each block needs as many colors as there are regions assigned to it. For example, if regions 1 and 2 are both assigned to block 1 (i.e., because block 1 is the most common block in those regions), the entry in `block.cols` for that block must have (at least) 2 colors. If an incorrect number of colors per block is supplied, an error will be thrown identifying which blocks are lacking colors. See Examples.
#'
#' @example man/examples/example-plotvertmap.R

#' @export
plotvertmap <- function(x, type = "count",
                        bps = NULL,
                        modelsupport = NULL, criterion = "aic", model = 1,
                        bpvar = NULL, bp.sd = NULL, sd.col = "black",
                        dropNA = FALSE, text = FALSE, name = NULL,
                        centraL = NULL,
                        reg.lim = NULL, lim.col = "black",
                        block.cols = NULL, block.lim = NULL) {

  arg::arg_is(x, c("regions_pco", "regions_data", "regions_sim"))

  if (inherits(x, c("regions_pco", "regions_data"))) {
    Xvar <- .get_pos(x, subset = FALSE)
    eligible_vertebrae <- .get_eligible_vertebrae(x)
    unique_vert <- .get_eligible_vertebrae(x, subset = FALSE)
  }
  else {
    Xvar <- x$Xvar
    eligible_vertebrae <- sort(unique(Xvar))
    unique_vert <- eligible_vertebrae
  }

  arg::arg_flag(dropNA)
  arg::arg_flag(text)

  type <- arg::match_arg(type, c("count", "percent"))

  arg::when_not_null(
    name,
    arg::arg_string
  )

  border.col <- "white"

  if (!is.null(bps)) {
    if (is.atomic(bps) && all(is.na(bps))) {
      bps <- NA_real_
    }
    else {
      arg::arg_numeric(bps)
      arg::arg_between(bps, range(unique_vert))
    }

    BPs <- sort(.drop_na(bps))

    names(BPs) <- paste0("breakpoint", seq_along(BPs))
  }
  else if (!is.null(modelsupport)) {
    arg::arg_is(modelsupport, "regions_modelsupport")

    criterion <- arg::match_arg(criterion, c("aic", "bic"))
    model_support_crit <- modelsupport[[switch(criterion, aic = "Model_support", bic = "Model_support_BIC")]]

    arg::when_not_null(
      model,
      arg::arg_whole_number,
      arg::arg_between(c(1, nrow(model_support_crit)))
    )

    if (is.null(model)) {
      model <- 1L
    }

    keep <- which(startsWith(names(model_support_crit), "breakpoint"))
    BPs <- .drop_na(unlist(model_support_crit[model, keep]))

    names(BPs) <- paste0("breakpoint", seq_along(BPs))
  }
  else if (!is.null(bpvar)) {
    arg::arg_is(bpvar, "regions_BPvar")
    BPs <- round(drop(bpvar$WeightedBP["wMean", ]))

    names(BPs) <- paste0("breakpoint", seq_along(BPs))

    if (is.null(bp.sd)) {
      bp.sd <- drop(bpvar$WeightedBP["wSD", ])
    }
  }
  else if (inherits(x, "regions_sim")) {
    BPs <- x$BPs
  }
  else {
    BPs <- numeric()
  }

  nreg <- length(BPs) + 1L

  col.by.block <- !is.null(block.lim)
  if (col.by.block) {
    arg::arg_numeric(block.lim)
    arg::arg_no_NA(block.lim)
    arg::arg_unique(block.lim)

    if (is.null(block.cols)) {
      arg::err("{.arg block.cols} must be specified when {.arg block.lim} is specified")
    }

    arg::arg_list(block.cols)

    if (length(block.lim) != length(block.cols) - 1L) {
      arg::err(c("{.arg block.cols} must have length equal to one greater than that of {.arg block.lim} (i.e., there should be one more region specified than there are region limits).",
                 "x" = "{.arg block.cols} has length {.val {length(block.lim)}}, and {.arg block.lim} has length {.val {length(block.cols)}}."))
    }

    if (!all(vapply(block.cols, function(x) is.character(x) && all(.is_color(x)), logical(1L)))) {
      arg::err("all entries in {.arg block.cols} must be character vectors containing color names or hex codes")
    }

    if (anyDuplicated(lapply(unlist(block.cols), grDevices::col2rgb)) != 0) {
      arg::wrn("duplicated colors were found in {.arg block.cols}")
    }
  }

  max.pos <- .last(unique_vert)
  vert.all <- seq_len(max.pos)

  vertmap <- data.frame(vname = vert.all)

  if (!is.null(centraL)) {
    vertmap$centraL <- 0

    if (!inherits(x, "regions_sim")) {
      data_combined <- .get_data_without_pos(x, subset = FALSE)
    }

    if (is.numeric(centraL)) {
      if (!inherits(x, "regions_sim") && length(centraL) == nrow(data_combined)) {
        for (i in unique_vert) {
          vertmap$centraL[vert.all == i] <- mean(centraL[Xvar == i], na.rm = TRUE)
        }
      }
      else if (length(centraL) == max.pos) {
        vertmap$centraL <- centraL
      }
      else if (length(centraL) == length(unique_vert)) {
        vertmap$centraL[match(vert.all, unique_vert)] <- centraL
      }
      else if (length(centraL) == length(eligible_vertebrae)) {
        vertmap$centraL[match(vert.all, eligible_vertebrae)] <- centraL
      }
      else {
        arg::err(sprintf("{.arg centraL} must have length equal to %s",
                         .word_list(c(
                           "the total number of vertebrae (in this case, {max.pos})",
                           if (max.pos != length(unique_vert))
                             "the number of unique vertebrae present in the original dataset (in thise case, {length(unique_vert)})",
                           if (length(unique_vert) != length(eligible_vertebrae))
                             "the number of available vertebrae (in this case, {length(eligible_vertebrae)})",
                           if (!inherits(x, "regions_sim") && length(unique_vert) != nrow(data_combined))
                             "the number of observations across all specimens (in this case, {nrow(data_combined)})"
                         ), "or")))
      }

      arg::arg_no_NA(vertmap$centraL, .arg = "centraL")
      arg::arg_gte(vertmap$centraL, 0, .arg = "centraL")
    }
    else if (is.character(centraL) && length(centraL) == 1L) {
      if (inherits(x, "regions_sim")) {
        arg::err("when {.arg x} is a {.cls regions_sim} object, {.arg centraL} cannot be specified as a string")
      }

      if (!centraL %in% names(data_combined)) {
        arg::err("the value supplied to .arg centraL} is not the name of a variable in the dataset")
      }

      # Compute mean centrum length across specimens for each vertebra
      for (i in unique_vert) {
        vertmap$centraL[vertmap$vname == i] <- mean(data_combined[[centraL]][Xvar == i], na.rm = TRUE)
      }

      arg::arg_no_NA(vertmap$centraL,
                     .msg = "the variable named in {.arg centraL} must not contain {.val {NA}} values")
      arg::arg_gte(vertmap$centraL, 0,
                   .msg = "all values in the variable named in {.arg centraL} must be greater than or equal to {.val {0}}")
    }
    else {
      arg::err("{.arg centraL} must be a vector of centrum lengths or a string containing them name of the centrum length variable")
    }
  }

  regions <- paste0("Region", seq_len(nreg))

  # Place 1st region
  vertmap$reg <- regions[1L]
  # Add other regions
  for (i in seq_along(BPs)) {
    vertmap$reg[vert.all > BPs[i]] <- regions[i + 1L]
  }

  # Assign vertebrae not in data value of "Missing"
  vertmap$reg[!vert.all %in% eligible_vertebrae] <- "Missing"
  if (dropNA) {
    # Drop vertebrae not in data
    vertmap <- vertmap[vertmap$reg != "Missing", , drop = FALSE]
  }

  # Indices of remaining vertebrae
  vertmap$ind <- seq_len(nrow(vertmap))

  if (is.null(centraL)) {
    # Set beginning position of each vert in ind:
    vertmap$ind.beg <- vertmap$ind - .5
    vertmap$ind.end <- vertmap$ind + .5

    if (type == "percent") {
      # Calculate % vertebral count:
      vertmap$pct.beg <- (vertmap$ind - 1) / nrow(vertmap)
      vertmap$pct.end <- vertmap$ind / nrow(vertmap)
    }
  }
  else {
    vertmap$L.end <- vertmap$L.beg <- NA_real_
    vertmap$L.pct.end <- vertmap$L.pct.beg <- NA_real_
    L.pos <- cumsum(vertmap$centraL)
    max.L.pos <- L.pos[length(L.pos)]

    vertmap$L.end <- L.pos
    vertmap$L.beg <- c(0, L.pos[-length(L.pos)])

    if (type == "percent") {
      vertmap$L.pct.beg <- vertmap$L.beg / max.L.pos
      vertmap$L.pct.end <- vertmap$L.end / max.L.pos
    }
  }

  # Add indiv identity:
  vertmap$indiv <- name

  ## Define position of blocks if requested:
  if (col.by.block) {
    missing_vert <- vertmap$reg == "Missing"

    if (is.null(names(block.cols))) {
      names(block.cols) <- paste0("block", seq_along(block.cols))
    }

    block.names <- names(block.cols)
    vertmap$trad.reg <- NA_character_

    # Add traditional regions to table:
    for (i in seq_along(block.names)) {
      if (i == 1L) {
        vertmap$trad.reg[!missing_vert &
                           vertmap$vname <= block.lim[i]] <- block.names[i]
      }
      else if (i == length(block.names)) {
        vertmap$trad.reg[!missing_vert &
                           vertmap$vname > block.lim[i - 1L]] <- block.names[i]
      }
      else {
        vertmap$trad.reg[!missing_vert &
                           vertmap$vname <= block.lim[i] &
                           vertmap$vname > block.lim[i - 1L]] <- block.names[i]
      }
    }

    # Match traditional regions boundaries and new regions by finding most
    # common traditional region in each estimated region. E.g., if traditional
    # region A is most common traditional region in estimated region 1, all
    # vertebrae in estimated region 1 get assigned traditional region A
    regs <- setdiff(levels(as.factor(vertmap$reg[])), "Missing")
    vertmap$tradreg.corr <- NA_character_

    for (i in seq_along(regs)) {
      t <- table(vertmap$trad.reg[vertmap$reg == regs[i]])
      vertmap$tradreg.corr[vertmap$reg == regs[i]] <- names(which.max(t))
    }

    # Rename estimated regions based on their appartenance to traditional regions
    reg.corr <- unique(vertmap$tradreg.corr)
    reg.ct <- rep.int(NA_integer_, sum(!missing_vert))
    reg.nm <- rep.int(NA_character_, sum(!missing_vert))
    vertmap$Region <- NA_character_

    for (i in seq_along(reg.corr)) {
      reg_sub <- unique(vertmap$reg[!missing_vert & vertmap$tradreg.corr == reg.corr[i]])
      for (j in seq_along(reg_sub)) {
        reg.ct[vertmap$reg[!missing_vert] == reg_sub[j]] <- j
        reg.nm[vertmap$reg[!missing_vert] == reg_sub[j]] <- reg.corr[i]
      }
    }

    vertmap$Region[!missing_vert] <- paste(reg.nm, reg.ct, sep = "_")
    vertmap$Region[missing_vert] <- "Missing"
  }
  else {
    vertmap$Region <- vertmap$reg
  }

  ## Traditional regions limit position ##
  if (!is.null(reg.lim)) {
    if (is.null(lim.col)) {
      lim.col <- "black"
    }

    # Adjust limit position
    if (is.null(centraL)) {
      reg.lim <- switch(
        type,
        "count" = vapply(reg.lim, function(m) {
          vertmap$ind.end[.last(which(vertmap$vname <= m))]
        }, numeric(1L)),
        "percent" = vapply(reg.lim, function(m) {
          vertmap$pct.end[.last(which(vertmap$vname <= m))]
        }, numeric(1L))
      )
    }
    else {
      reg.lim <- switch(
        type,
        "count" = vapply(reg.lim, function(m) {
          vertmap$L.end[.last(which(vertmap$vname <= m))]
        }, numeric(1L)),
        "percent" = vapply(reg.lim, function(m) {
          vertmap$L.pct.end[.last(which(vertmap$vname <= m))]
        }, numeric(1L))
      )
    }
  }

  # Correct BP sd position according to % count and % length:
  if (!is.null(bp.sd)) {
    if (length(BPs) == 0) {
      arg::err("{.arg bp.sd} cannot be supplied when no breakpoints are present")
    }

    arg::arg_numeric(bp.sd)
    arg::arg_no_NA(bp.sd)

    if (length(bp.sd) != length(BPs)) {
      arg::err("{.arg bp.sd} must have length equal to the number of breakpoints (in this case, {length(BPs)})")
    }
    arg::arg_gte(bp.sd, 0)

    bp.inds <- vapply(BPs, function(b) {
      vertmap$ind[.last(which(vertmap$vname <= b))]
    }, numeric(1L))

    if (is.null(centraL)) {
      bp.sd <- data.frame(
        bp = vertmap$ind.end[bp.inds],
        beg = vertmap$ind.end[bp.inds] - bp.sd,
        end = vertmap$ind.end[bp.inds] + bp.sd
      )

      if (type == "percent") {
        bp.sd <- data.frame(
          bp = (bp.sd$bp - .5) / nrow(vertmap),
          beg = (bp.sd$beg - .5) / nrow(vertmap),
          end = (bp.sd$end - .5) / nrow(vertmap)
        )
      }
    }
    else {
      bp.sd <- data.frame(
        bp = vertmap$L.end[bp.inds],
        beg = vapply(seq_along(BPs), function(b) {
          ind <- bp.inds[b]
          p <- vertmap$L.end[ind]
          sd <- bp.sd[b]

          if (sd > 1) {
            sd.floor <- floor(sd)
            p <- p - sum(vertmap$centraL[(ind - sd.floor + 1):(ind)])
            sd <- sd - sd.floor
            ind <- ind - sd.floor
          }

          max(0, p - sd * (vertmap$L.end[ind] - vertmap$L.beg[ind]))
        }, numeric(1L)),
        end = vapply(seq_along(BPs), function(b) {
          ind <- bp.inds[b]
          p <- vertmap$L.end[ind]
          sd <- bp.sd[b]

          if (sd > 1) {
            sd.floor <- floor(sd)
            p <- p + sum(vertmap$centraL[(ind + 1):(ind + sd.floor)])
            sd <- sd - sd.floor
            ind <- ind + sd.floor
          }

          min(max.L.pos, p + sd * (vertmap$L.end[ind + 1] - vertmap$L.beg[ind + 1]))
        }, numeric(1L))
      )

      if (type == "percent") {
        bp.sd <- data.frame(
          bp = bp.sd$bp / max.L.pos,
          beg = bp.sd$beg / max.L.pos,
          end = bp.sd$end / max.L.pos
        )
      }
    }

    # Assign vertical position and deal with overlaps
    y <- rep.int(0, length(BPs))
    y[1L] <- 1

    for (i in seq_along(BPs)[-1L]) {
      num_ovl <- vapply(seq_len(max(y)), function(l) {
        sum(bp.sd$beg[y == l] < bp.sd$end[i] & bp.sd$end[y == l] >= bp.sd$beg[i])
      }, numeric(1L))

      y[i] <- {
        if (all(num_ovl > 0)) max(y) + 1
        else min(which(num_ovl == 0))
      }
    }

    # Rescale to keep first position at 0 and alternate above and below
    y <- floor(y / 2) * (-1)^y

    sd.jit <- .15

    # Rescale to keep centered at 1.5 and differ by sd.jit
    bp.sd$y <- sd.jit * (y - mean(y)) + 1.5
  }

  ## Add vertebra number; dodge SDs if requested
  if (text) {
    vertmap$text.y <- {
      if (is.null(bp.sd)) 1.5
      else .5 + .5 * min(bp.sd$y)
    }
  }

  ## Set color for each region
  if (col.by.block) {
    colors <- unlist(block.cols)
    names(colors) <- unlist(lapply(names(block.cols), function(i) paste(i, seq_along(block.cols[[i]]), sep = "_")))

    needed.colors <- vapply(unique(reg.nm), function(n) max(reg.ct[reg.nm == n]), integer(1L))
    if (!all(lengths(block.cols) >= needed.colors[names(block.cols)])) {
      deficient.regions <- names(block.cols)[lengths(block.cols) < needed.colors[names(block.cols)]]

      msg <- setNames(
        vapply(deficient.regions, function(dr) {
          sprintf("{.var %1$s} needs {.val {%2$s}} {cli::qty(%2$s)} color{?s}, but only {.val {%3$s}} {cli::qty(%3$s)} {?was/were} supplied.",
                  dr, needed.colors[dr], length(block.cols[[dr]]))
        }, character(1L)),
        rep.int("x", length(deficient.regions))
      )

      arg::err(c("Not enough colors were supplied for each region specified in {.arg block.cols}:", "",
                 msg, "",
                 "i" = "Please supply additional colors for {.var {deficient.regions}}."))
    }
  }
  else {
    if (!is.null(block.cols)) {
      arg::arg_character(block.cols)

      if (!all(.is_color(block.cols))) {
        arg::err("all values in {.arg block.cols} must be color names or hex codes")
      }

      if (length(block.cols) < nreg) {
        arg::err("{.arg block.cols} must have length greater than or equal to the number of regions (in this case, {nreg})")
      }

      if (anyDuplicated(lapply(block.cols, grDevices::col2rgb)) != 0) {
        arg::wrn("duplicated colors were found in {.arg block.cols}")
      }

      pal <- block.cols
    }
    else if (nreg < 3L) {
      pal <- RColorBrewer::brewer.pal(3L, "Paired")
    }
    else if (nreg %% 2 != 0) {
      pal <- RColorBrewer::brewer.pal((nreg + 1L), "Paired")
      pal <- pal[-(length(pal) / 2)]
    }
    else {
      pal <- RColorBrewer::brewer.pal(nreg, "Paired")
    }

    colors <- setNames(pal[seq_len(nreg)], regions)
  }

  colors <- c(Missing = "grey", colors)		# Add color for missing vertebrae

  if (is.null(centraL)) {
    # Initialize plot
    p <- ggplot(data = vertmap)

    if (type == "count") {
      p <- p +
        geom_rect(aes(xmin = .data$ind.beg,
                      xmax = .data$ind.end,
                      ymin = 1, ymax = 2,
                      fill = .data$Region),
                  color = border.col) +
        labs(y = name, x = "Vertebral count")

      if (text) {
        # Add vertebral number on each colored square
        p <- p + geom_text(aes(x = .data$ind,
                               y = .data$text.y,
                               label = .data$vname))
      }
    }
    else if (type == "percent") {
      p <- p +
        geom_rect(aes(xmin = .data$pct.beg,
                      xmax = .data$pct.end,
                      ymin = 1, ymax = 2,
                      fill = .data$Region),
                  color = border.col) +
        labs(y = name, x = "% Vertebral count")

      if (text) {
        p <- p + geom_text(aes(x = (.data$pct.beg + .data$pct.end) / 2,
                               y = .data$text.y,
                               label = .data$vname))
      }
    }
  }
  else {
    # Initialize plot with nonzero centrum lengths
    p <- ggplot(data = vertmap[vertmap$centraL > 0, ])

    if (type == "count") {
      p <- p +
        geom_rect(aes(xmin = .data$L.beg,
                      xmax = .data$L.end,
                      ymin = 1, ymax = 2,
                      fill = .data$Region),
                  color = border.col) +
        labs(y = name, x = "Centrum position")

      if (text) {
        p <- p + geom_text(aes(x = (.data$L.beg + .data$L.end) / 2,
                               y = .data$text.y,
                               label = .data$vname))
      }
    }
    else if (type == "percent") {
      p <- p +
        geom_rect(aes(xmin = .data$L.pct.beg,
                      xmax = .data$L.pct.end,
                      ymin = 1, ymax = 2,
                      fill = .data$Region),
                  color = border.col) +
        labs(y = name, x = "% Total centrum length")

      if (text) {
        p <- p + geom_text(aes(x = (.data$L.pct.beg + .data$L.pct.end) / 2,
                               y = .data$text.y,
                               label = .data$vname))
      }
    }
  }

  if (!is.null(reg.lim)) {
    p <- p + geom_vline(xintercept = reg.lim,
                        color = lim.col,
                        linewidth = 1)
  }

  if (!is.null(bp.sd)) {
    # Add horizontal line for BPs standard deviation
    p <- p + geom_pointrange(data = bp.sd,
                             aes(xmin = .data$beg,
                                 xmax = .data$end,
                                 y = .data$y,
                                 x = .data$bp),
                             color = sd.col,
                             linewidth = 1,
                             shape = "diamond",
                             size = .8)
  }

  p <- p +
    scale_fill_manual("Legend", values = colors) +
    scale_x_continuous(expand = c(0, 0),
                       labels = switch(type,
                                       percent = scales::percent,
                                       waiver())) +
    theme_classic() +
    theme(axis.text.y = element_blank(),
          axis.ticks.y = element_blank(),
          axis.line = element_blank(),
          axis.title.y = element_text(angle = 0, size = 10,
                                      vjust = 0.5, hjust = 1),
          legend.position = "none",
          plot.margin = unit(c(0, 10, 0, 0), "pt"))

  p
}

#' Select PCO scores
#'
#' `PCOselect()` provides several methods to select the number of principal coordinates (PCOs) analysis scores to be used in subsequent analyses.
#'
#' @param pco a `regions_pco` object; the output of a call to [svdPCO()].
#' @param method string; the method used to select the number of PCOs. Allowable options include `"manual"`, `"boot"`, `"variance"`, and `"max"`. Default is `"manual"`. Abbreviations allowed. See Details.
#' @param scores when `method = "manual"`, the number of PCO scores to use.
#' @param nreps when `method = "boot"`, the number of bootstrap replications to use.
#' @param cutoff when `method = "variance"`, the cutoff for the variance explained by each PCO score.
#' @param results when `method = "max"`, a `regions_results` object, the output of a call to [calcregions()].
#' @param criterion when `method = "max"`, which criterion should be used to select the number of scores. Allowable options include `"aic"` and `"bic"`. Abbreviations allowed.
#' @param verbose when `method = "boot"`, whether to display a progress bar. Default is `TRUE` when running interactively and `FALSE` otherwise.
#' @param x for `plot.regions_pco_select()`, a `regions_pco_select` object, the output of a call to `PCOselect()` with `method = "boot"` or `"max"`.
#' @param object a `regions_pco_select` object, the output of a call to `PCOselect()` with `method = "max"`.
#' @param \dots ignored.
#'
#' @return For `PCOselect()`, a `regions_pco_select` object, which is a numeric vector containing the indices of the chosen PCs, with attributes containing information about the PC scores chosen by the specified method. When `method = "boot"`, the bootstrap results are stored in the `"boot"` attribute. When `method = "max"`, the `regions_results` object passed to `regions` and other information about the quality of fit for each number of PCs are stored in the `"pcomax"` attribute.
#'
#' The `plot()` methods each return a `ggplot` object that can manipulated using \pkg{ggplot2} syntax. The `summary()` method returns a data.frame of results.
#'
#' @details
#' Each method provides an alternate way to select the number of scores. These are described below.
#'
#' ## `method = "manual"`:
#'
#' This simply returns the number supplied to `scores` after running some checks to ensure it is valid.
#'
#' ## `method = "boot"`
#'
#' Bootstrapping works by comparing the eigenvalue distributions of PCOs to those with randomized data in order to extract PCO axes with significant signal, which are defined as those with eigenvalues greater than those from randomized data. The returned PCO cutoff is the largest PCO axis whose eigenvalues fall below the mean eigenvalue for that axis from the randomized data. Data are randomly sampled by row. Bootstrapping is sensitive to unequal variances of columns, so `scale = TRUE` should be set in the call to [svdPCO()], which is the default; the data are scaled in the same way prior to bootstrapping. The `plot()` method displays the eigenvalues of the true PCOs and boxplots summarizing the distribution of the bootstrapped eigenvalues for each PCO. The `boot` method is unavailable for user-supplied PC scores (`regions_pco` object obtained from `process_PC` or `process_gmPC` functions).
#'
#' ## `method = "variance"`
#'
#' This method works by computing the ratio of each eigenvalue to the sum of the eigenvalues (i.e., to compute the proportion of variance explained by each PC score) and select the number of scores with ratios greater than the cutoff value supplied to `cutoff`.
#'
#' ## `method = "max"`
#'
#' This method works by selecting the smallest number of PCs that gives a region score within .001 of the maximum possible region score for the segmented models fit in the object supplied to `results`. Which criterion is maximized (AIC or BIC) is determined by the value supplied to `criterion`. The `summary()` method displays the region score (estimated number of regions) for each PC (`RSind`) and for PCs cumulatively (`RScum`) selected using the AICc or BIC as well as the cumulative proportion of variance explained by the PCs. The `plot()` method displays this information graphically, with the left y-axis displaying the region score for the PCs individually (pale blue triangles) and cumulatively (orange circles) using each of the two criteria, and the right y-axis displaying the cumulative percentage of variance explained by the PCs.
#'
#' @example man/examples/example-PCOselect.R
#'

#' @export
PCOselect <- function(pco, method = "manual", scores = NULL, cutoff = .05, nreps = 500,
                      results = NULL, criterion = "aic", verbose = interactive()) {
  arg::arg_is(pco, "regions_pco")

  method <- arg::match_arg(method, c("manual", "boot", "variance", "max"))

  boot <- pcomax <- NULL
  info <- list(method = method)
  if (method == "manual") {
    arg::arg_non_null(scores)
    arg::arg_count(scores,
                   .msg = "{.arg scores} must be the number of PC scores to select")
    arg::arg_between(scores, c(1, ncol(pco$scores)),
                     .msg = sprintf("{.arg scores} must be between 1 and the total number of principal coordinates (i.e., %s)",
                                    ncol(pco$scores)))
  }
  else if (method == "boot") {
    arg::arg_count(nreps)
    arg::arg_gte(nreps, 1)
    arg::arg_flag(verbose)

    boot <- .PCOboot(pco, nreps, verbose)
    scores <- boot$sigpco

    info$nreps <- nreps
  }
  else if (method == "variance") {
    arg::arg_number(cutoff)
    arg::arg_between(cutoff, c(0, 1), inclusive = FALSE)

    scores <- sum(pco$eigen.val / sum(pco$eigen.val) > cutoff)
    info$cutoff <- cutoff
  }
  else {
    arg::arg_non_null(results)
    arg::arg_is(results, "regions_results")

    criterion <- arg::match_arg(criterion, c("aic", "bic"))

    pcomax <- .PCOmax(results)

    scores <- pcomax[[switch(criterion,
                             aic = "pco.max.AICc",
                             bic = "pco.max.BIC")]]

    pcomax$pco <- pco
    pcomax$results <- results

    info$criterion <- criterion
  }

  out <- seq_len(scores)

  attr(out, "info") <- info
  attr(out, "boot") <- boot
  attr(out, "pcomax") <- pcomax

  class(out) <- c("regions_pco_select", class(out))

  out
}

#' @exportS3Method print regions_pco_select
print.regions_pco_select <- function(x, ...) {
  info <- attr(x, "info")
  cat("A `regions_pco_select` object\n")
  cat(sprintf("- PC scores selected: %s\n", toString(x[])))
  cat(sprintf("- Method: %s\n",
              switch(info$method,
                     "boot" = sprintf("boot (%s replications)", info$nreps),
                     "variance" = sprintf("variance (cutoff: %s)", info$cutoff),
                     "max" = sprintf("max (criterion: %s)", toupper(info$criterion)),
                     info$method)))
  invisible(x)
}

#' @exportS3Method plot regions_pco_select
#' @rdname PCOselect
plot.regions_pco_select <- function(x, ...) {
  if (!identical(attr(x, "info")$method, "boot") &&
      !identical(attr(x, "info")$method, "max")) {
    arg::err("{.fun plot} can only be used on {.cls regions_pco_select} objects when {.arg method} is {.val boot} or {.val max}")
  }

  if (identical(attr(x, "info")$method, "boot")) {
    boot <- attr(x, "boot")
    ind <- seq_along(boot$eigen.true)

    eigen.boot <- as.vector(boot$eigen.boot)
    ind.boot <- as.vector(row(boot$eigen.boot))

    p <- ggplot() +
      geom_point(aes(x = ind, y = boot$eigen.true), shape = "circle filled") +
      geom_line(aes(x = ind, y = boot$eigen.true)) +
      geom_boxplot(aes(y = eigen.boot, x = ind.boot, group = factor(ind.boot)),
                   outlier.shape = NA) +
      labs(x = "PC axis", y = "Eigenvalue", title = "Eigenvalue cutoff") +
      theme_bw()
  }
  else {
    s <- summary(x, plot = FALSE)

    p <- .plot_summary_regions_pco_select(s)
  }

  p
}

#' @exportS3Method summary regions_pco_select
#' @rdname PCOselect
summary.regions_pco_select <- function(object, ...) {
  if (!identical(attr(object, "info")$method, "max") ||
      is.null(attr(object, "pcomax"))) {
    arg::err("{.fun summary} can only be used on {.cls regions_pco_select} objects when {.arg method} is {.val max}")
  }

  results <- attr(object, "pcomax")$results
  eigenvals <- attr(object, "pcomax")$pco$eigen.val

  noregions <- max(results$stats$Nregions)

  nPCO <- sum(startsWith(colnames(results$results), "RSS."))

  # Calculate variance of each PCO:
  var.exp <- cumsum(eigenvals / sum(eigenvals))[seq_len(nPCO)]

  pco.no.test <- data.frame(PCO = seq_len(nPCO),
                            RSind.AICc = NA_real_,
                            RScum.AICc = NA_real_,
                            RSind.BIC = NA_real_,
                            RScum.BIC = NA_real_,
                            CumulVar = var.exp)

  for (a in seq_len(nPCO)) {
    #Run for individual PCs
    models.ind <- modelselect(results, scores = a)
    support.ind <- modelsupport(models.ind)

    #Run for cumulative PCs
    models.cum <- modelselect(results, scores = 1:a)
    support.cum <- modelsupport(models.cum)

    pco.no.test[["RSind.AICc"]][a] <- support.ind$Region_score
    pco.no.test[["RScum.AICc"]][a] <- support.cum$Region_score
    pco.no.test[["RSind.BIC"]][a] <- support.ind$Region_score_BIC
    pco.no.test[["RScum.BIC"]][a] <- support.cum$Region_score_BIC
  }

  attr(pco.no.test, "noregions") <- noregions
  class(pco.no.test) <- c("summary.regions_pco_select", class(pco.no.test))

  pco.no.test
}

#' @exportS3Method print summary.regions_pco_select
print.summary.regions_pco_select <- function(x, digits = 3L, ...) {
  arg::arg_whole_number(digits)

  x0 <- x
  for (i in seq_len(ncol(x))[-1]) {
    x[[i]] <- round(x[[i]], digits)
  }
  print.data.frame(x, row.names = FALSE, ...)
  invisible(x0)
}

.PCOboot <- function(pco, nreps = 500, verbose = FALSE) {

  if (verbose) {
    cat("Bootstrapping...\n")
  }
  else {
    opb <- pbapply::pboptions(type = "none")
    on.exit(pbapply::pboptions(opb))
  }

  #### Edited code to add support for GMM data: ##
  metric <- attr(pco, "metric")
  if (identical(metric, "custom")) {
    arg::err("method {.val boot} is not allowed for user-supplied PC scores")
  }

  #calculate 'true' eigenvalues as percentage variance
  eigen.true <- prop.table(pco$eigen.val)

  data <- .get_data_without_pos(pco)

  randdata <- data

  eigen.boot <- do.call("cbind", pbapply::pblapply(seq_len(nreps), function(i) {
    #Shuffle each row of the dataset
    for (j in seq_len(nrow(data))) {
      randdata[j, ] <- sample(data[j, ])
    }

    dist <- cluster::daisy(randdata, metric = metric, stand = attr(pco, "scale"))

    if (anyNA(dist)) {
      return(NULL)
    }

    #calculate bootstrapped eigenvalues
    pco <- .svdPCO_internal(dist, val.only = TRUE)

    #calculate as percentage variance
    prop.table(pco$eigen.val)
  }))

  eigen.mean <- rowMeans(eigen.boot)  #calculate mean and SD of bootstrapped values
  eigen.sd <- apply(eigen.boot, 1L, sd)
  diff <- eigen.true - eigen.mean  #figure out which PCOs have greater eigenvalues for the 'true' dataset
  diff[diff < 0] <- 0

  #split the dataset at the zeros, and calculate number of pcos in the first string
  sigpco <- length(split(diff, cumsum(diff == 0))[["0"]])

  list(eigen.true = eigen.true,
       eigen.mean = eigen.mean,
       eigen.sd = eigen.sd,
       sigpco = sigpco,
       eigen.boot = eigen.boot)
}

.PCOmax <- function(results, tol = .001) {

  regiondata <- results$results

  nvar <- sum(startsWith(colnames(regiondata), "RSS."))

  pco.no.test <- data.frame(PCO = seq_len(nvar),
                            RS_AICc = NA_real_,
                            RS_BIC = NA_real_)

  for (i in seq_len(nvar)) {
    #Run for cumulative PCs
    models.cum <- modelselect(results, scores = seq_len(i))
    support.cum <- modelsupport(models.cum)

    pco.no.test[["RS_AICc"]][i] <- support.cum$Region_score
    pco.no.test[["RS_BIC"]][i] <- support.cum$Region_score_BIC
  }

  pco.max.AICc <- which(.equiv(pco.no.test[["RS_AICc"]], max(pco.no.test[["RS_AICc"]]), tol = tol))[1L]
  pco.max.BIC <- which(.equiv(pco.no.test[["RS_BIC"]], max(pco.no.test[["RS_BIC"]]), tol = tol))[1L]

  list(pco.max.AICc = pco.max.AICc,
       pco.max.BIC = pco.max.BIC,
       pco.dist = pco.no.test)
}

.plot_summary_regions_pco_select <- function(x, ...) {
  arg::arg_is(x, "summary.regions_pco_select")

  pco.no.test.long <- reshape(x, direction = "long", idvar = "PCO", varying = list(c(2, 3), c(4, 5)),
                              timevar = "PCOtype", times = c("Single PCO", "Cumulated PCOs"),
                              v.names = c("RS.AICc", "RS.BIC"))
  pco.no.test.long <- reshape(pco.no.test.long, direction = "long", varying = 4:5,
                              timevar = "Testtype", times = c("AICc", "BIC"),
                              v.names = "value")

  noregions <- attr(x, "noregions")

  p <- ggplot(pco.no.test.long) +
    geom_point(data = pco.no.test.long,
               aes(x = .data$PCO, y = .data$value,
                   color = .data$PCOtype, shape = .data$PCOtype)) +
    scale_color_manual(values = c("#fc8d62", "#8da0cb")) +
    geom_line(data = x, aes(x = .data$PCO, y = (.data$CumulVar * (noregions - 1) + 1)),
              color = "darkgrey", linewidth = 1) +
    scale_y_continuous(name = "Region score",
                       sec.axis = sec_axis(~ (. - 1) / (noregions - 1), labels = scales::percent,
                                           # limits = c(0, 1),
                                           name = "Cumulated variance explained")) +
    scale_x_continuous(breaks = scales::breaks_extended(Q = c(0:5))) +
    facet_wrap(~Testtype) +
    theme_bw() +
    theme(panel.grid.minor = element_blank(), legend.position = "bottom") +
    labs(color = "Region score", shape = "Region score")

  p
}

#' Plot a segmented regression model
#'
#' `plotsegreg()` plots the fitted lines resulting from a segmented regression model.
#'
#' @inheritParams modelperf
#' @param scores `numeric`; the indices of the PC scores for which the fitted lines should be plotted.
#' @param model `numeric`; for which model among the best as determined by `criterion` should fitted lines be plotted. 1 is the best model, 2 the second best, etc. Default is 1. When `x` is a `regions_pco` object, either `modelsupport`, `criterion`, and `model` must be supplied or `bps` and `cont` must be supplied. See Details.
#' @param bps `numeric`; a vector of breakpoints for which model fitted lines should be plotted. When `x` is a `regions_pco` object, either `modelsupport`, `criterion`, and `model` must be supplied or `bps` and `cont` must be supplied. See Details.
#' @param \dots ignored.
#'
#' @return A `ggplot` object that can be manipulated using *ggplot2* syntax.
#'
#' @details `plotsegreg()` operates on a single model identified by breakpoints and whether the model is continuous or not. When `x` is a `regions_pco` object, the model is selected either as the best model in the supplied `modelsupport` object (where "best" is determined by the arguments to `criterion` and `model`) or as specified by the user using the arguments to `bps` and `cont`. When `x` is a `regions_results_single` object, the breakpoints and model form are determined based on the supplied object.
#'
#' `plot()` is an alias for `plotsegreg()` for `regions_results_single` objects.
#'
#' @seealso [modelsupport()] for assessing model support using information criteria; [calcmodel()] for fitting a single segmented regression model; [modelperf()] for computing fit statistics for a single segmented regression model.
#'
#' @example man/examples/example-modelperf.R
#'

#' @export
plotsegreg <- function(x, scores, ...) {
  arg::arg_supplied(scores)

  UseMethod("plotsegreg")
}

#' @exportS3Method plotsegreg regions_pco
#' @rdname plotsegreg
plotsegreg.regions_pco <- function(x, scores, modelsupport = NULL, criterion = "aic",
                                   model = 1, bps = NULL, cont = TRUE, ...) {
  arg::arg_supplied(scores)
  arg::arg_whole_numeric(scores)
  arg::arg_between(scores, c(1, ncol(x[["scores"]])))
  scores <- sort(scores)

  Xvar <- .get_pos(x)
  Yvar <- x[["scores"]][, scores, drop = FALSE]

  #Calculate weights to ensure each vertebra counts equally
  vert_tab <- tabulate(Xvar)
  w <- 1 / vert_tab[Xvar]

  if (!is.null(bps)) {
    if (!is.null(modelsupport)) {
      arg::wrn("{.arg bps} specified; ignoring {.arg modelsupport}")
    }

    arg::arg_numeric(bps)
    arg::arg_between(bps, range(Xvar))
    arg::arg_flag(cont)

    BPs <- sort(.drop_na(bps))
    nbp <- length(BPs)

    if (nbp > 0) {
      names(BPs) <- paste0("breakpoint", seq_len(nbp))
    }
  }
  else if (!is.null(modelsupport)) {
    arg::arg_is(modelsupport, "regions_modelsupport")

    criterion <- arg::match_arg(criterion, c("aic", "bic"))
    model_support_crit <- modelsupport[[switch(criterion, aic = "Model_support", bic = "Model_support_BIC")]]

    arg::when_not_null(
      model,
      arg::arg_whole_number,
      arg::arg_between(c(1, nrow(model_support_crit)))
    )

    if (is.null(model)) {
      model <- 1L
    }

    cont <- attr(modelsupport, "cont")

    keep <- which(startsWith(names(model_support_crit), "breakpoint"))
    BPs <- .drop_na(unlist(model_support_crit[model, keep]))
  }
  else {
    BPs <- NULL
    cont <- TRUE
  }

  fit <- .fast_lm(x = .design_matrix(Xvar, BPs, cont), y = Yvar, w = w)

  yhat <- Yvar - fit$residuals

  .plotreg_internal(Xvar, Yvar, yhat, BPs, lines = TRUE, scores = scores, weights = w)
}

#' @exportS3Method plotsegreg regions_sim
#' @rdname plotsegreg
plotsegreg.regions_sim <- function(x, scores, modelsupport = NULL, criterion = "aic",
                                   model = 1, bps = NULL, cont = TRUE, ...) {

  arg::arg_supplied(scores)
  arg::arg_whole_numeric(scores)
  arg::arg_between(scores, c(1, ncol(x$Yvar)))

  Xvar <- x$Xvar
  Yvar <- x$Yvar[, scores, drop = FALSE]

  if (!is.null(bps)) {
    if (is.atomic(bps) && all(is.na(bps))) {
      bps <- NA_real_
      cont <- TRUE
    }
    else {
      arg::arg_numeric(bps)
      arg::arg_between(bps, range(Xvar))
      arg::arg_flag(cont)
    }

    BPs <- sort(.drop_na(bps))

    names(BPs) <- paste0("breakpoint", seq_along(BPs))
  }
  else if (!is.null(modelsupport)) {
    arg::arg_is(modelsupport, "regions_modelsupport")

    criterion <- arg::match_arg(criterion, c("aic", "bic"))
    model_support_crit <- modelsupport[[switch(criterion, aic = "Model_support", bic = "Model_support_BIC")]]

    arg::when_not_null(
      model,
      arg::arg_whole_number,
      arg::arg_between(c(1, nrow(model_support_crit)))
    )

    if (is.null(model)) {
      model <- 1L
    }

    cont <- attr(modelsupport, "cont")

    keep <- which(startsWith(names(model_support_crit), "breakpoint"))
    BPs <- .drop_na(unlist(model_support_crit[model, keep]))
  }
  else {
    arg::err("{.arg bps} or {.arg modelsupport} must be provided")
  }

  fit <- .fast_lm(x = .design_matrix(Xvar, BPs, cont), y = Yvar)

  yhat <- Yvar - fit$residuals

  .plotreg_internal(Xvar, Yvar, yhat, BPs, lines = TRUE, scores = scores)
}

#' @exportS3Method plotsegreg regions_results_single
#' @rdname plotsegreg
plotsegreg.regions_results_single <- function(x, scores, ...) {

  arg::arg_supplied(scores)
  arg::arg_whole_numeric(scores)
  arg::arg_between(scores, c(1, ncol(attr(x, "scores"))))

  Xvar <- attr(x, "pos")
  Yvar <- attr(x, "scores")[, scores, drop = FALSE]

  #Calculate weights to ensure each vertebra counts equally
  vert_tab <- tabulate(Xvar)
  w <- 1 / vert_tab[Xvar]

  BPs <- unlist(x$results[startsWith(names(x$results), "breakpoint")])
  cont <- attr(x, "cont")

  fit <- .fast_lm(x = .design_matrix(Xvar, BPs, cont), y = Yvar, w = w)

  yhat <- Yvar - fit$residuals

  .plotreg_internal(Xvar, Yvar, yhat, BPs, lines = TRUE, scores = scores, weights = w)
}

.plotreg_internal <- function(Xvar, Yvar, yhat = NULL, BPs = NULL, lines = TRUE, scores = 1,
                              linescolor = "darkturquoise", BPcolor = "coral", specimen = NULL,
                              weights = NULL) {

  plot_data <- data.frame(
    PCO = factor(rep(paste("PC", scores), each = length(Xvar))),
    Xvar = rep(Xvar, ncol(Yvar))
  )

  use_specimen <- !is.null(specimen) && nlevels(specimen) > 1L
  if (use_specimen) {
    plot_data$specimen <- rep(specimen, ncol(Yvar))
  }

  plot_data$weights <- {
    if (is.null(weights)) 1
    else rep(weights, ncol(Yvar))
  }


  #Flatten scores/predicted values
  plot_data$Yvar <- as.vector(Yvar)
  if (!is.null(yhat)) {
    plot_data$yhat <- as.vector(yhat)
  }

  p <- ggplot(plot_data,
              aes(x = .data$Xvar))

  if (use_specimen)
    p <- p + geom_point(aes(y = .data$Yvar,
                            color = .data$specimen,
                            size = .data$weights),
                        shape = "circle")
  else
    p <- p + geom_point(aes(y = .data$Yvar,
                            size = .data$weights),
                        shape = "circle",
                        color = "darkgray")

  if (length(BPs) > 0) {
    breakpoints <- BPs + .5
    p <- p + geom_vline(xintercept = breakpoints,
                        color = BPcolor,
                        linetype = "dashed")
  }
  else {
    breakpoints <- numeric(0)
  }

  if (!is.null(yhat) && lines) {
    p <- p + geom_line(aes(y = .data$yhat,
                           group = cut(.data$Xvar, c(-Inf, breakpoints, Inf))),
                       show.legend = FALSE, color = linescolor)
  }

  p <- p + scale_size_area(max_size = 2, guide = NULL) +
    facet_wrap(vars(.data$PCO), ncol = 1,
               scales = "free_y",
               strip.position = "left") +
    theme_bw() +
    theme(strip.placement = "outside",
          strip.background = element_blank(),
          strip.text = element_text(size = 11)) +
    labs(x = "Position",
         y = NULL,
         color = NULL)

  p
}

#' Calculate PCO loadings
#'
#' `PCOload()` computes the loadings for each principal coordinates (PCOs) analysis score, which are the correlations between the features used to compute the PCOs and the PCOs.
#'
#' @param x for `PCOload()`, a `regions_pco` object; the output of a call to [svdPCO()]. For `plot()`, a `regions_pco_load` object.
#' @param scores a numeric vector containing the indices of the desired scores.
#' @param \dots ignored.
#'
#' @return
#' `PCOload()` returns a `regions_pco_load` object, which is a matrix with a column for each PCO score requested and a row for each variable in the original dataset; values indicate the correlation between each variable and each PCO score. `plot()` returns a `ggplot` object, which can be manipulated using \pkg{ggplot2} syntax, that displays the loadings visually.
#'
#' @details
#' The loadings for a constructed variable, `vert.size`, are also computed and displayed. This is computed as the mean of the features for each vertebra. This function is not suitable for geometric morphometric data.
#'
#' @seealso
#' [svdPCO()] for computing the PCOs; [plot.regions_pco()] for visualizing the correlations between PCO scores.
#'
#' @example man/examples/example-PCOload.R
#'

#' @export
PCOload <- function(x, scores) {

  arg::arg_is(x, "regions_pco")
  pco_scores <- x[["scores"]]

  # New code: Check if original data is GM:
  if (inherits(attr(x, "data"), "regions_dataGM")) {
    arg::err("loadings cannot be computed for geometric morphometric data")
  }

  # New code: Check if PC scores have been computed outside of MorphoRegions:
  if (identical(attr(x, "metric"), "custom")) {
    arg::wrn("loadings computed as correlations between original data and PC scores; check if this method is relevant for your data and ordination method")
  }

  arg::when_supplied(
    scores,
    arg::arg_whole_numeric,
    arg::arg_between(c(1, ncol(pco_scores)))
  )

  if (missing(scores)) {
    scores <- seq_len(ncol(pco_scores))
  }

  data <- .get_data_without_pos(x)
  vert.size <- rowMeans(data)
  data <- cbind(data, vert.size)

  load.pco <- lapply(scores, function(i) {
    cor(data, pco_scores[, i], use = "pairwise.complete.obs")
  })

  load.pco <- do.call("cbind", load.pco)
  colnames(load.pco) <- paste("PC", scores, sep = ".")

  class(load.pco) <- c("regions_pco_load", class(load.pco))

  load.pco
}

#' @exportS3Method print regions_pco_load
print.regions_pco_load <- function(x, digits = 3L, ...) {
  d <- as.data.frame.matrix(x)
  cat("- PC loadings:\n\n")
  print(d[-nrow(x), , drop = FALSE], digits = digits, ...)
  cat("\n - Corr w/ vertebra size:\n\n")
  print(d[nrow(x), , drop = FALSE], digits = digits, ...)

  invisible(x)
}

#' @exportS3Method plot regions_pco_load
#' @rdname PCOload
plot.regions_pco_load <- function(x, ...) {
  d <- as.data.frame.matrix(x)
  rownames(d)[rownames(d) == "vert.size"] <- "Size"

  d$feature <- factor(rownames(d), levels = rev(rownames(d)))
  d$featuren <- as.numeric(d$feature)
  d$featuren[nrow(d)] <- .7

  d_long <- reshape(d, direction = "long", varying = startsWith(names(d), "PC"),
                    idvar = "feature", v.names = "value",
                    timevar = "PC")

  ggplot(d_long) +
    geom_tile(aes(x = .data$PC, y = .data$featuren, fill = .data$value)) +
    labs(y = NULL, x = "PC", fill = "Loading") +
    scale_x_continuous(position = "top", breaks = seq_len(max(d_long$PC))) +
    scale_y_continuous(labels = rev(rownames(d)),
                       breaks = sort(unique(d$featuren)),
                       expand = c(0, 0)) +
    scale_fill_gradient2(limits = c(-1, 1), low = scales::muted("blue"),
                         high = scales::muted("red")) +
    coord_fixed() +
    theme_minimal() +
    theme(panel.grid = element_blank())
}

# Load coordinates of aligned landmarks, PC scores, and
# eigenvalues; vertebra index as names of array elements
# of aligned landmarks
data("seal_gmm")

# Process PC scores and morphological data:
seal_pca <- process_gmPC(data = seal_gmm$coord_gpa,
                         pcscores = seal_gmm$scores,
                         eigenvals = seal_gmm$eigenvals)

# Process multiple datasets; specimen names in 'specimens'
# vector, vertebrae index in 'vertebrae' vector
data("wolves_gmm")

# Process PC scores and geometric morphometric dataset:
wolves_pca <- process_gmPC(data = wolves_gmm$coord_gpa,
                           pos = wolves_gmm$vertebrae,
                           pcscores = wolves_gmm$scores,
                           eigenvals = wolves_gmm$eigenvals,
                           specimens = wolves_gmm$specimens)

#' Measurements from the vertebral column of an alligator
#'
#' Linear and angular measurements from Alligator mississipiensis MCZ 81457
#' @docType data
#' @format A matrix with 22 vertebrae and 19 measurements. Column 1, vertebra, is the positional
#' information.
#'
#' @keywords datasets
"alligator"

#' Measurements from the vertebral column of a mouse
#'
#' Linear and angular measurements from Mus musculus MCZ 59560
#' @docType data
#' @format A matrix with 23 vertebrae and 19 measurements. Column 1, vertebra, is the positional
#' information.
#'
#' @keywords datasets
"musm"

#' Measurements from the vertebral column of a dolphin
#'
#' Linear and angular measurements from Platanista gangetica SMNS 45653
#' @docType data
#' @format A matrix with 40 vertebrae and 16 measurements. Column 1, vertebra, is the positional
#' information.
#'
#' @keywords datasets
"dolphin"

#' Measurements from the vertebral column of three porpoises
#'
#' Linear and angular measurements from Phocoena phocoena NRM 815072 (`porpoise1`), NRM 835011 (`porpoise2`), and NRM 855083 (`porpoise3`).
#' @docType data
#' @usage data("porpoise")
#' @format Each is a data frame with 58, 56, and 59 vertebrae (respectively) and 16 measurements. Column 1, Vertebra, contains the positional information.
#'
#' @keywords datasets
#' @references Gillet, A., Frederich, B., Pierce, S. E., & Parmentier, E. (2022). Iterative habitat transitions are associated with morphological convergence of the backbone in delphinoids. Journal of Mammalian Evolution, 29(4), 931-946.
#' @name porpoise
"porpoise1"

#' @rdname porpoise
#' @usage NULL
#' @format NULL
"porpoise2"

#' @rdname porpoise
#' @usage NULL
#' @format NULL
"porpoise3"

#' Geometric morphometric data and PC scores of the vertebral column a seal
#'
#' Procrustes aligned 3d landmarks (`coord_gpa`) and PC scores (`scores`) of the backbone of Phoca vitulina (ZMUC 160).
#' @docType data
#' @usage data("seal_gmm")
#' @format A list containing: a 3D array of Procrustes aligned 3D landmarks (`coord_gpa`) of 25 presacral vertebrae with 40 landmarks per vertebra, a matrix of PC scores from a PCA performed on Procrustes aligned landmarks (`scores`),
#' and a numeric vector with the eigenvalues from the PCA (`eigenvals`).
#'
#' @keywords datasets
#' @references Esteban, J. M., Martín-Serra, A., Pérez-Ramos, A., Mulot, B., Jones, K. E., & Figueirido, B.  (2023). The impact of the land-to-sea transition on evolutionary integration and modularity of the pinniped backbone. Communications Biology, 6, 1141.
#' @name seal_gmm
"seal_gmm"

#' Geometric morphometric data and PC scores of the vertebral column two wolves
#'
#' Procrustes aligned 3d landmarks (`coord_gpa`) and PC scores (`scores`) from  10 subsampled vertebrae along the backbone of two specimens of Canis lupus (MU 071 and MU 073).
#' @docType data
#' @usage data("wolves_gmm")
#' @format A list containing: a 3D array of Procrustes aligned 3D landmarks (`coord_gpa`) of 10 subsampled vertebrae per specimen, and 30 landmarks per vertebra, a matrix of PC scores from a PCA performed on Procrustes aligned landmarks (`scores`),
#' a numeric vector with the eigenvalues from the PCA (`eigenvals`), a numeric vector with the position information of vertebrae (`vertebrae`), and a vector with the names of the specimens (`specimens`).
#'
#' @keywords datasets
#' @references Schwab, J., Figueirido, B., & Jones, K. E. (2026). Ecological inference from isolated vertebrae: Evaluating functional signal across the carnivoran spine. Journal of Morphology, 287(1), e70109.
#' @name wolves_gmm
"wolves_gmm"

# Similar to na.omit() without appending attributes
.drop_na <- function(x) {
  x[!is.na(x)]
}

# Similar to pbapply::pblapply() for handling cluster argument, but
# with no progress bar.
.lapply_selector <- function(..., cl = NULL) {
  if (is.null(cl)) {
    lapply(...)
  }
  else if (inherits(cl, "cluster")) {
    if (isTRUE(getOption("pboptions")$use_lb))
      parallel::parLapplyLB(cl, ...)
    else parallel::parLapply(cl, ...)
  }
  else {
    parallel::mclapply(..., mc.cores = as.integer(cl))
  }
}

# Similar to rbind() or dplyr::bind_rows(), but specifically to append a df
# to another with fewer columns
.rbind_larger <- function(x, y) {

  if (is.data.frame(x) || is.data.frame(y)) {
    if (is.null(x)) return(y)
    if (is.null(y)) return(x)

    x <- as.data.frame(x)
    y <- as.data.frame(y)

    nam <- {
      if (ncol(x) > ncol(y)) names(x)
      else names(y)
    }

    .expand <- function(d, nam) {
      nam_not_in_d <- setdiff(nam, names(d))
      to_add <- as.data.frame(matrix(nrow = nrow(d), ncol = length(nam_not_in_d),
                                     dimnames = list(rownames(d), nam_not_in_d)))
      cbind(d, to_add)[nam]
    }
  }
  else if (is.matrix(x) && is.matrix(y)) {
    if (is.null(x)) return(y)
    if (is.null(y)) return(x)

    nam <- {
      if (ncol(x) > ncol(y)) names(x)
      else names(y)
    }

    .expand <- function(d, nam) {
      nam_not_in_d <- setdiff(nam, colnames(d))
      to_add <- matrix(nrow = nrow(d), ncol = length(nam_not_in_d),
                       dimnames = list(rownames(d), nam_not_in_d))
      cbind(d, to_add)[, nam, drop = FALSE]
    }
  }
  else {
    arg::err("{.arg x} and {.arg y} must be data frames or matrices")
  }

  if (ncol(x) > ncol(y)) {
    rbind(x, .expand(y, nam))
  }
  else if (ncol(x) < ncol(y)) {
    rbind(.expand(x, nam), y)
  }
  else {
    rbind(x, y)
  }
}

#When given a vector of strings, creates a string of the form "a and b"
#or "a, b, and c"
.word_list <- function(word.list = NULL, and.or = "and") {

  word.list <- word.list[!word.list %in% c(NA_character_, "")]
  L <- length(word.list)

  if (L == 0) {
    return("")
  }

  if (L == 1) {
    return(word.list)
  }

  and.or <- arg::match_arg(and.or, c("and", "or"))

  if (L == 2) {
    out <- paste(word.list, collapse = paste0(" ", and.or, " "))
  }
  else {
    out <- paste(toString(word.list[seq_len(L - 1)]),
                 word.list[L], sep = paste0(", ", and.or, " "))

  }

  out
}

# Checks if supplied argument is a color; vectorized
.is_color <- function(x) {
  vapply(x, function(z) {
    tryCatch(is.matrix(grDevices::col2rgb(z)),
             error = function(e) FALSE)
  }, logical(1L))
}

# Extracts vertebra positions for remaining observations from a
# regions_data dataset
.get_pos <- function(x, subset = TRUE) {
  if (inherits(x, "regions_pco")) {
    x <- attr(x, "data")
  }

  if (inherits(x, "regions_data")) {   # Original code for data processed with process_measurements
    pos_ind <- attr(x, "pos_ind")
    pos <- unlist(lapply(x, `[[`, pos_ind))

    if (!subset) {
      return(pos)
    }
  }
  else if (inherits(x, "regions_dataGM")) {  # New code for data processed with process_gmPC
    pos <- attr(x, "Xvar")
  }

  pos[pos %in% attr(x, "eligible_vertebrae")]
}

# Extracts measurements from
# regions_data dataset
.get_data_without_pos <- function(x, subset = TRUE) {
  if (inherits(x, "regions_pco")) {
    x <- attr(x, "data")
  }

  pos_ind <- attr(x, "pos_ind")

  out <- do.call("rbind", x)[-pos_ind]

  if (!subset) {
    return(out)
  }

  out[.get_pos(x, FALSE) %in% .get_eligible_vertebrae(x), ]
}

.get_eligible_vertebrae <- function(x, subset = TRUE) {
  if (inherits(x, "regions_pco")) {
    x <- attr(x, "data")
  }

  if (subset) {
    return(attr(x, "eligible_vertebrae"))
  }

  pos <- .get_pos(x, subset = FALSE)

  sort(unique(pos))
}

# Fast lm(), uses .lm.fit() but accommodates weights
.fast_lm <- function(x, y, w = NULL) {
  if (is.null(w)) {
    return(.lm.fit(x, y))
  }

  w <- sqrt(w)
  out <- .lm.fit(x * w, y * w)
  out$residuals <- out$residuals / w

  out
}

# Get last element in a vector
.last <- function(x) {
  x[length(x)]
}

# Test whether each row of matrix contains any elements in vec
.any_mat_in <- function(m, vec) {
  comp <- {
    if (length(vec) > 1) function(x, y) x %in% y
    else function(x, y) x == y
  }

  .rowSums(matrix(comp(m, vec), ncol = ncol(m)),
           m = nrow(m), n = ncol(m)) > 0
}

# Test whether each row of matrix contains all elements in vec
.all_mat_in <- function(m, vec) {
  p <- ncol(m)

  out <- .rowSums(matrix(m == vec[1], ncol = p),
                  m = nrow(m), n = p) > 0

  for (i in seq_along(vec)[-1L]) {
    out[out][.rowSums(matrix(m[out, ] == vec[i], ncol = p),
                      m = sum(out), n = p) == 0] <- FALSE
  }

  out
}

# Fast, minimal version of pmax() that takes in vector and scalar
.pmax2 <- function(x, n = 0) {
  x[x < n] <- n
  x
}

# Checks if two vector of numbers are equal within tolerance
.equiv <- function(x, y, tol = sqrt(.Machine$double.eps)) {
  arg::arg_numeric(x)
  arg::arg_numeric(y)
  arg::arg_number(tol)
  arg::arg_gte(tol, 0)

  abs(x - y) < tol
}

data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Fit segmented regression models for 1 to 7 regions
# using PCOs 1 to 4 and a continuous model with a
# exhaustive search
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 7,
                             minvert = 3,
                             cont = TRUE,
                             exhaus = TRUE,
                             verbose = FALSE)

# Compute Akaike-weighted location and SD of optimal
# breakpoints using top 10% of models with 4 regions
calcBPvar(regionresults, noregions = 4,
          pct = .1, criterion = "aic")

data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Fit segmented regression models for 1 to 7 regions
# using PCOs 1 to 4 and a continuous model with a
# non-exhaustive search
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 7,
                             minvert = 3,
                             cont = TRUE,
                             exhaus = FALSE,
                             verbose = FALSE)

regionresults

# For each number of regions, identify best
# model based on minimizing RSS
bestresults <- modelselect(regionresults)
bestresults

# Evaluate support for each model and rank models
supp <- modelsupport(bestresults)
supp

# 5 regions best based on AICc; 6 regions based on BIC

#' Calculate results of a single segmented regression model
#'
#' `calcmodel()` fits a multivariate segmented regression model using the supplied PCOs and breakpoints.
#'
#' @param x a `regions_pco` object; the output of a call to `svdPCO()`.
#' @param scores `numeric`; the indices of the PCO scores to use as the outcomes in fitting the model (e.g., `1:4` to use the first four scores).
#' @param bps `numeric`; the indices of the breakpoints to use in fitting the model. To request a model with no breakpoints, set to `NA`.
#' @param cont `logical`; whether to fit a model that is continuous (`TRUE`) or discontinuous (`FALSE`) at the breakpoints. Default is `TRUE`. Ignored when `bps` is `NA`.
#'
#' @returns A `regions_results_single` object, which contains the results of the model (breakpoints and RSS of each PCO and overall) and model support statistics.
#'
#' @seealso [calcregions()] and [addregions()] for computing all possible models instead of just a single one; [plotsegreg()], for which the `plot` method is an alias, for plotting the fitted regression lines; [modelsupport()] for interpreting the model support statistics.
#'
#' @example man/examples/example-calcmodel.R

#' @export
calcmodel <- function(x, scores, bps, cont = TRUE) {
  arg::arg_is(x, c("regions_pco", "regions_sim"))

  if (inherits(x, "regions_pco")) {
    arg::arg_supplied(scores)
    arg::arg_whole_numeric(scores)
    arg::arg_between(scores, c(1L, ncol(x[["scores"]])))

    Xvar <- .get_pos(x)
    Yvar <- x[["scores"]][, scores, drop = FALSE]
    eligible_vertebrae <- .get_eligible_vertebrae(x)
  }
  else {
    if (missing(scores)) {
      scores <- seq_len(ncol(x[["Yvar"]]))
    }
    Xvar <- x[["Xvar"]]
    Yvar <- x[["Yvar"]][, scores, drop = FALSE]
    eligible_vertebrae <- sort(unique(Xvar))
  }

  arg::arg_supplied(bps)

  if (is.atomic(bps) && all(is.na(bps))) {
    bps <- NA_real_
    cont <- TRUE
  }
  else {
    arg::arg_numeric(bps)
    arg::arg_between(bps, range(eligible_vertebrae))
    arg::arg_flag(cont)
  }

  # Re-order data
  o <- order(Xvar)
  Xvar <- Xvar[o]
  Yvar <- Yvar[o, , drop = FALSE]

  BPs <- sort(.drop_na(bps))
  nbp <- length(BPs)

  names(BPs) <- paste0("breakpoint", seq_len(nbp))

  noPC <- ncol(Yvar)

  colhead <- c("regions", paste0("breakpoint", seq_len(max(1L, nbp))),
               "sumRSS", paste("RSS", 1:noPC, sep = "."))

  #Calculate weights to ensure each vertebra counts equally
  vert_tab <- tabulate(Xvar)
  w <- 1 / vert_tab[Xvar]

  #Fit the model
  x <- .design_matrix(Xvar, if (!anyNA(BPs)) BPs, cont)

  lines <- .fast_lm(x = x, y = Yvar, w = w)

  RSS <- sum(w * lines$residuals^2)

  rsq <- {
    if (noPC > 1) colSums(w * lines$residuals^2)
    else RSS
  }

  res <- as.data.frame(matrix(c(nbp + 1, BPs, RSS, rsq), nrow = 1L,
                              dimnames = list(NULL, colhead)))

  supp <- .AICcalc(RSS, noPC = noPC, nvert = length(Xvar),
                   noregions = nbp + 1, cont = cont)

  out <- list(results = res,
              support = supp)

  attr(out, "scores") <- Yvar
  attr(out, "cont") <- cont
  attr(out, "nvert") <- length(Xvar)
  attr(out, "pos") <- Xvar

  class(out) <- "regions_results_single"

  out
}

#' @exportS3Method print regions_results_single
print.regions_results_single <- function(x, digits = 3, ...) {
  x0 <- x
  colnames(x[["results"]])[1L] <- "Regions"
  colnames(x[["results"]]) <- sub("breakpoint", "BP ", colnames(x[["results"]]),
                                  fixed = TRUE)
  print(round(x[["results"]], digits), row.names = FALSE)
  cat("\n- Support:\n")
  print(x[["support"]])

  invisible(x0)
}

#' @exportS3Method plot regions_results_single
plot.regions_results_single <- function(x, scores, ...) {
  arg::arg_supplied(scores)
  plotsegreg.regions_results_single(x, scores = scores, ...)
}

data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Compute PCO loadings
loadings <- PCOload(alligator_PCO, scores = 1:4)
loadings

# Plot loadings
plot(loadings)

# Process dataset; vertebra index in "Vertebra" column
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Process multiple datasets; vertebra index in first column
data("porpoise")

porpoise_data <- process_measurements(list(porpoise1,
                                           porpoise2,
                                           porpoise3),
                                      pos = 1)

#' Simulate regions data
#'
#' `simregions()` simulates vertebrae and PCOs that satisfy certain constraints.
#'
#' @param nvert `numeric`; the number of vertebrae for which to simulate data.
#' @param nregions `numeric`; the desired number of regions in the simulated data.
#' @param nvar `numeric`; the number of PCO axes to simulate. Default is 1.
#' @param r2 `numeric`; a vector containing the \eqn{R^2} of the true segmented regression model for each simulated PCO. If a single value is supplied, all PCOs will receive that value. Otherwise, one value should be supplied for each simulated PCO.
#' @param minvert `numeric`; the minimum number of vertebrae allowed in each region. Default is 3.
#' @param cont `logical`; whether to use models that are continuous (`TRUE`) or discontinuous (`FALSE`) at the breakpoints to generate the data. Default is `TRUE`.
#' @param sl.dif `numeric`; the minimum required difference in slopes between adjacent regions, expressed as a proportion of the maximal difference between allowable slopes. Must be between 0 and 1. See Details.
#' @param x a `regions_sim` object.
#' @param scores `numeric`; for which simulated PCO scores the simulated values should be plotted.
#' @param lines `logical`; whether to display the simulated regression lines on the plot. Default is `TRUE`.
#' @param \dots ignored.
#'
#' @return
#' `simregions()` returns a `regions_sim` object, which contains the vertebra indices in the `Xvar` entry, the PCO scores in the `Yvar` entry, the simulated breakpoints in the `BPs` entry, the simulated model coefficients in the `coefs` entry, and the simulated error standard deviation in the `ersd` entry. The attribute `"design"` contains the design matrix, which when multiplied by the coefficients and added to a random normal variate with standard deviation equal to the error standard deviation yields the observed PCO scores.
#'
#' `plot()` returns a `ggplot` object that can be manipulated using `ggplot2` syntax. The plot is similar to that produced by [plot.regions_pco()] and to that produced by [plotsegreg()] except that the displayed lines (if requested) are the true rather than fitted regression lines.
#'
#' @details
#' `simregions()` generates PCO scores for each requested vertebra such that certain conditions are met. The slopes for each region are drawn from a uniform distribution with limits of -.5 and .5. If a set of slopes contains two adjacent slopes that have a difference less than `sl.dif`, it is rejected and a new one is drawn. The scaling of the PCOs is determined by the slopes and the requested \eqn{R^2}. The PCOs will not necessarily be in order from most variable to least variable as they are in a traditional PCO analysis.
#'
#' Intercepts (the intercept of the first region when `cont = TRUE` and the intercept of all regions when `cont = FALSE`) are drawn from a uniform distribution with limits of \eqn{-n/4} and \eqn{n/4}, where \eqn{n} is the number of breakpoints, one less than `nregions`. Intercepts other than the first when `cont = TRUE` are determined by the slopes.
#'
#' The `cont`, `r2`, and `sl.dif` arguments control how easy it is for fitted segmented regression models to capture the true structure. When `cont = TRUE`, it can be harder to determine exactly where regions begin and end, especially if `sl.dif` is 0. When `r2` is high, there is little variation around the true line, so the fitted lines will be more precise and region boundaries clearer. When `sl.dif` is high, slopes of adjacent regions are different from each other, so it is easier to detect region boundaries. Setting `sl.dif` to between .5 and 1 ensures that the slopes in adjacent regions have different signs.
#'
#' @seealso
#' [calcregions()] for fitting segmented regression models to the simulated data; [calcmodel()] for fitting a single segmented regression model to the simulated data; [plotsegreg()] for plotting estimated regression lines.
#'
#' @example man/examples/example-simregions.R
#'

#' @export
simregions <- function(nvert, nregions, nvar = 1, r2 = .95,
                       minvert = 3, cont = TRUE, sl.dif = 0) {

  #Checks
  arg::arg_supplied(nvert)
  arg::arg_count(nvert)
  arg::arg_gte(nvert, 4)

  arg::arg_supplied(nregions)
  arg::arg_count(nregions)

  arg::arg_count(nvar)
  arg::arg_gte(nvar, 1)

  arg::arg_numeric(r2)
  arg::arg_between(r2, c(0, 1), inclusive = c(FALSE, TRUE))

  arg::arg_count(minvert)
  arg::arg_gte(minvert, 2)

  arg::arg_flag(cont)

  arg::arg_number(sl.dif)
  arg::arg_between(r2, c(0, 1), inclusive = c(TRUE, FALSE))

  arg::arg_lte(minvert * nregions, nvert,
               .msg = "the product of {.arg minvert} and {.arg nregions} must not be greater than {.arg nvert}")

  Xvar <- seq_len(nvert)

  nbp <- nregions - 1

  if (length(r2) == 1L) {
    r2 <- rep.int(r2, nvar)
  }
  else if (length(r2) != nvar) {
    arg::err("{.arg r2} must have length equal to 1 or {.arg nvar}")
  }

  #Draw possible combination of breakpoints
  bp <- .sample_combosR(Xvar, minvert, nbp)

  #Create design matrix
  x <- .design_matrix(Xvar, bp, cont)

  #Initialize matrix of coefficients
  coefs <- matrix(nrow = ncol(x), ncol = nvar)

  rownames(coefs) <- {
    if (cont) c("int1", paste0("slope", seq_len(nregions)))
    else paste0(rep(c("int", "slope"), each = nregions),
                rep(seq_len(nregions), 2L))

  }

  colnames(coefs) <- paste0("PC.", seq_len(nvar))

  for (i in seq_len(nvar)) {
    # Rejection sampling for slopes
    repeat {
      if (sl.dif > .5) {
        test_slopes <- rep.int(0, nregions)
        test_slopes[1L] <- (-1)^rbinom(1, 1, .5) * runif(1, min = sl.dif - .5, max = .5)

        for (j in seq_along(test_slopes)[-1L]) {
          test_slopes[j] <- -sign(test_slopes[j - 1L]) * runif(1, min = sl.dif - .5, max = .5)
        }
      }
      else {
        test_slopes <- runif(nregions, min = -.5, max = .5)
      }

      if (all(abs(diff(test_slopes)) >= sl.dif)) {
        break
      }
    }

    if (cont) {
      for (j in seq_along(test_slopes)[-1L]) {
        test_slopes[j] <- test_slopes[j] - test_slopes[j - 1]
      }

      coefs["int1", i] <- runif(1, -nbp / 4, nbp / 4)
    }
    else {
      ints <- runif(nregions, -nbp / 4, nbp / 4)
      coefs[startsWith(rownames(coefs), "int"), i] <- ints - test_slopes * c(0, bp)
    }

    coefs[startsWith(rownames(coefs), "slope"), i] <- test_slopes

  }

  #Generate structural outcomes from coefs and design
  ypred <- x %*% coefs

  #Generate actual outcomes by adding error
  ersd <- numeric(nvar)
  y <- array(dim = dim(ypred), dimnames = dimnames(ypred))

  for (i in seq_len(nvar)) {
    #SD(error) required to make requested R2
    ersd[i] <- sqrt((1 - r2[i]) * var(ypred[, i]) / r2[i])

    y[, i] <- ypred[, i] + rnorm(nvert, sd = ersd[i])
  }

  out <- list(Xvar = Xvar, Yvar = y, BPs = bp,
              coefs = coefs, ersd = ersd)

  attr(out, "cont") <- cont
  attr(out, "design") <- x

  class(out) <- "regions_sim"

  out
}

#' @exportS3Method plot regions_sim
#' @rdname simregions
plot.regions_sim <- function(x, scores = 1, lines = TRUE, ...) {

  arg::arg_whole_numeric(scores)
  arg::arg_between(scores, c(1, ncol(x$Yvar)))
  arg::arg_flag(lines)

  Yvar <- x$Yvar[, scores, drop = FALSE]
  Xvar <- x$Xvar

  yhat <- attr(x, "design") %*% x$coef[, scores, drop = FALSE]

  .plotreg_internal(Xvar, Yvar, yhat, x$BPs, lines, scores = scores,
                    linescolor = "gray30")
}

#' @exportS3Method print regions_sim
print.regions_sim <- function(x, ...) {
  cat("A `regions_sim` object\n")
  cat(" - number of vertebrae:", length(x$Xvar), "\n")
  cat(" - number of regions:", length(x$BPs) + 1L, "\n")
  cat(" - breakpoints:", toString(x$BPs), "\n")
  cat(" - model type:", if (attr(x, "cont")) "continuous" else "discontinuous", "\n")
  cat(" - number of PCO scores:", ncol(x$Yvar), "\n")
  cat("Use `plot()` to display the true lines and simulated data.")

  invisible(x)
}

# Sample combos without needing to store all combos
.sample_combosR <- function(vert, minvert, nbp) {

  n <- length(vert)

  #Choose combination at random
  maxcombos <- .ncombos(n, minvert, nbp)

  k <- sample.int(maxcombos, 1L)

  .kth_combo(k, vert, minvert, nbp)
}

# Find the kth combo; assumes k <= maxcombs
.kth_combo <- function(k, vert, minvert, nbp) {

  n <- length(vert)

  mins <- minvert * seq_len(nbp)
  maxes <- n - rev(mins)

  combo <- rep.int(0, nbp)

  a <- 0

  for (i in seq_len(nbp)) {
    if (i < nbp) {
      sums <- cumsum(.ncombos(n - mins[i]:maxes[i], minvert, nbp - i))
      b <- min(which(k <= a + sums))
      combo[i] <- mins[i] + b - 1
      a <- a + c(0, sums)[b]
      mins[i + 1] <- combo[i] + minvert
    }
    else {
      sums <- seq_len(maxes[i] - mins[i] + 1)
      b <- min(which(k <= a + sums))
      combo[i] <- mins[i] + b - 1
    }
  }

  combo[] <- vert[combo]

  setNames(combo, paste0("breakpoint", seq_along(combo)))
}

data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Plot vertebrae before subsampling
plotvertmap(alligator_PCO, dropNA = FALSE,
            text = TRUE)

# Subsample data after estimating PCOs; subsample down
# to 15 vertebrae
alligator_PCO_sample <- subsample(alligator_PCO,
                                  sample = 15)

# Plot vertebrae after subsampling; gray vertebrae
# have been dropped
plotvertmap(alligator_PCO_sample, dropNA = FALSE,
            text = TRUE)

---
title: "MorphoRegions"
output:
  html_vignette:
    toc: yes
vignette: |
  %\VignetteIndexEntry{MorphoRegions} 
  %\VignetteEngine{knitr::rmarkdown_notangle} 
  %\VignetteEncoding{UTF-8}
bibliography: references.bib
link-citations: true
---

<style>
p.caption {
  font-size: 0.82em;
  text-align: center;
  margin-right: 8% ;
  margin-left: 8% ;
}
img {
  border: 0;
}
</style>



```{r, include=FALSE}
knitr::opts_chunk$set(echo = TRUE,
                      collapse=TRUE,
                      comment="#>",
                      fig.width=5.5, 
                      fig.height=4,
                      fig.align = "center")
```

```{r setup, include=F}
library(MorphoRegions)
```


## About *MorphoRegions*

The *MorphoRegions* package is built to **computationally identify regions** (morphological, functional, etc.) **in serially homologous structures** such as, but not limited to, the vertebrate backbone. Regions are modeled as segmented linear regressions with each segment corresponding to a region and region boundaries (or breakpoints) corresponding to changes along the serially homologous structure. The optimal number of regions and their breakpoint positions are identified without *a priori* assumptions. The algorithm fits increasingly complex models (increasing number of regions). For each given number of regions, all possible combinations of breakpoint positions will be iteratively fitted. Maximum-likelihood methods are then used to identify the best model, as described in @GilRev. 


```{r images, fig.show = "hold", out.width = "30%", fig.cap="**Fig.1.** *MorphoRegions* identifies regions in serially-homologous structures by fitting all possible breakpoint positions for a given number of region and then repeating the method on increasingly complex models.",echo=F}

knitr::include_graphics("images/Regions_image2.jpg")

knitr::include_graphics("images/Gif_segreg_1bp_25delay.gif")

knitr::include_graphics("images/Gif_segreg_50delay.gif")

```

<br>
This package is an updated version of the [*regions* R-package](https://github.com/katrinajones/regions) from @Jones2018 which expanded on previous work from @Head2015.
This package provides improved computational methods (reduced time and memory usage) allowing application to larger datasets. Several features have been added including the options to perform non-exhaustive searches, run parallel computing, choose between continuous and discontinuous fitting methods, define the minimum number of vertebrae in each region, select the best model using either AICc or BIC, and fit a model to multiple specimens. Plotting options have also been expanded.

<br>

## Package installation 

The package is available on CRAN and can be installed and loaded using the following code:

```{r, eval=F}
install.packages("MorphoRegions")
library(MorphoRegions)
```

```{r, include=F}
library(MorphoRegions)
```

## Preparing the data

Data should be provided as a dataframe where each row is an element of the serially-homologous structure (e.g., a vertebra). One column should contain positional information of each element (e.g., vertebral number) and other columns should contain variables that will be used to calculate regions (e.g., morphological measurements).

The package comes with **four built-in datasets**. `alligator`, `musm`, and `dolphin` contain measurements on the vertebrae of an alligator, mouse, and dolphin. `porpoise` contains vertebral measurements of three specimens of harbor porpoises and can be used to test fitting a single model for multiple specimens together   ([see below](#IntraSp)). *Alligator and mouse data are from @Jones2018; dolphin and porpoise data are from @Gillet2022.*


```{r, results="hold"}
data("dolphin")

head(dolphin)
```

In the `dolphin` dataset, the first column contains positional information (vertebral number) while other columns contain morphological measurements taken on each vertebra.

**Prior to analysis, data must be processed into an object usable by `regions` using `process_measurements()`. Data should be provided as a dataframe**. The `pos` argument is used to specify the name or index of the column containing positional information; if unspecified, the first column will be used by default. Names or indices of columns containing variables can be specified with the `measurements` argument.
The `fillNA` argument allows to fill missing values in the dataset. Missing values can be filled for up to two successive elements (e.g., up to two successive vertebrae) using a linear imputation (see @Jones2018 for more details). The `fillNA` argument is set to `TRUE` by default.


```{r}
dolphin_data <- process_measurements(dolphin, pos = 1)
class(dolphin_data)
```

## Data ordination {#PCO}

To allow the analysis of high dimensional datasets, including a variety of data types, a **Principal Coordinates Analysis (PCO)**, a distance-based data ordination method, is implemented in `svdPCO()`. The function relies on a distance matrix generated by `cluster::daisy()`, which tolerates missing values. Three types of distance metric can be used: Euclidean, Manhattan, or Gower. Euclidean should only be used where all variables are similar (e.g., linear measures on the same scale), and is most similar to a PCA. Gower is good for combining different types of continuous data (e.g., angles and linear).

```{r}
dolphin_pco <- svdPCO(dolphin_data, metric = "gower")
```

PCO scores of each vertebra can be plotted along their respective position in backbone (left) or as a morphospace (right).

```{r,fig.show = "hold", out.width = "48%", fig.cap= "**Fig.2.** *Left*: PCO scores of each vertebra along the backbone for PCO1 *(top)* and PCO2 *(bottom)*. *Right*: morphospace defined by PCO1 and 2, points plotted as numbers corresponding to the position of each vertebra along the backbone."}
#Plot PCOs along backbone for PC1 and 2:
plot(dolphin_pco, 1:2)

# Plot morphospace of PC1 and 2:
plot(dolphin_pco, pco_x = 1, pco_y = 2)
```

### Data reduction

While the regionalization analysis can be performed on all PCO axes, it's likely that lower PCO axes mostly contain noise. The number of PCO axes to retain for analysis can be selected using `PCOselect()`.

**Four different methods to select PCOs** are available and be specified using the `method` argument: 

- `"manual"`: the number of PCOs is directly defined via the `scores` argument
- `"variance"`: PCO axes with a variance (defined as the ratio between the eigenvalue of the given axis and the sum of all eigenvalues) higher to the `cutoff` value (expressed as a value between 0 and 1) are retained
- `"max"`: PCOs axes needed to obtain the maximum number of regions in the segmented models tested which must be supplied to `results`. The criterion (AICc or BIC) used to maximize the region score should be defined with `criterion` 
- `"boot"`: PCO axes are selected through bootstrapping by comparing observed eigenvalues to eigenvalues from randomized data and retaining PCO axes with eigenvalues higher than values from randomized data. When using this method, the screeplot showing observed and simulated eigenvalues can be plotted. *Note that bootstrapping is sensitive to unequal variances of variables and should only be used on data that have been scaled (setting `scale = TRUE` in the original call to `svdPCO()`).*

```{r, fig.show='all', out.width = '60%',fig.align='center'}
# Manual (keeping axes 1 to 3):
PCOs <- PCOselect(dolphin_pco, method = "manual",
                  scores = 3)
PCOs

# Variance (selecting axes with variance > 0.05):
PCOs <- PCOselect(dolphin_pco, method = "variance",
                  cutoff = .05)
PCOs
```

```{r, fig.show='all', out.width = '60%',fig.align='center', fig.cap="**Fig.3a.** Graph of region scores (left y-axis) and cumulative variance (right y-axis) obtained with varying number of PCO axes included for AICc (left graph) and BIC (right graph). Blue triangles correspond to region score obtained for each individual PCO axis and orange circle to region scores obtained with cumulative number of PCO axes. The cumulated variance explained by PCO axes is represented by the grey curve."}
# Max:
regionresults <- calcregions(dolphin_pco, scores = 1:39, noregions = 4, 
                             minvert = 3, cont = TRUE, exhaus = TRUE,
                             verbose = FALSE)
PCOs <- PCOselect(dolphin_pco, method = "max",
                  results = regionresults,
                  criterion = "bic")
PCOs
plot(PCOs)
```

```{r, fig.show='all', out.width = '60%',fig.align='center', fig.cap="**Fig.3b.** Screeplot of observed eigenvalues (line) and eigenvalues from randomized data (boxes). Here PCO axes 1 and 2 are retained for subsequent analyses."}
# Bootstrap:
PCOs <- PCOselect(dolphin_pco, method = "boot")
PCOs
plot(PCOs)
```


### Loadings

Correlations between original variables and each PCO axis are obtained through the `PCOload` function. The additional 'Size' variable corresponds to the mean value of the variables.

```{r, out.width = '70%',fig.align='center', fig.cap="**Fig.4.** Correlations between original variables and PCO axes 1 and 2."}
loadings <- PCOload(dolphin_pco, scores = PCOs)
plot(loadings)
```

<br>

## Regionalization analysis

The regionalization analysis is performed in three steps:

1. Fitting all possible models for all number of regions requested

2. Selecting the best model for each number of regions tested 

3. Selecting the overall best model


### Fitting segmented linear regressions {#SegRegFit}

`calcregions()` fits all possible combinations of breakpoints (boundaries between successive regions) for each number of regions to be tested. The analysis will fit models on PCO scores, so the output of `svdPCO()` needs to be provided. The number of PCOs to keep (supplied either as a vector or as the output of `PCOselect()`), and the maximum number of regions to test need to be provided with the `scores` and `noregions` arguments, respectively.


The function also offers **several optional arguments** (see the [Model fitting options](#FittingOptions) section for details):

- `minvert`: define the minimum number of vertebrae (elements) in each region  ([see here](#MinVert))
- `cont`: choose to fit either a continuous fit (`TRUE`) or discontinuous fit (`FALSE`) ([see here](#ContDisc))
- `includebp`: force breakpoint(s) to fall at the supplied value(s) ([see here](#ForceBP))
- `omitbp`: exclude breakpoint(s) falling at the supplied value(s) ([see here](#ForceBP))
- `exhaus`: either to fit all possible combinations (`TRUE`) or only the most probable ones (`FALSE`) to reduce computing time ([see here](#Exhaus))
- `cl`: option to use parallel computation ([see here](#Para))

```{r}
regionresults <- calcregions(dolphin_pco, scores = PCOs, noregions = 5,
                             minvert = 3, cont = TRUE, 
                             exhaus = TRUE, cl = NULL,
                             verbose = FALSE)
```

The output of the function contains information on the parameters used to run the analysis and total number of models tested while using the `summary` function allows to see how many models are possible and how many have been tested for each given number of regions.

```{r}
regionresults
summary(regionresults)
```


### Selecting best model for each number of regions

`modelselect()` retains the best model for each number of regions tested. The best models are selected as the ones with the lowest residual sum of squares (RSS).

```{r}
models <- modelselect(regionresults)
models
```

Each row of the output corresponds to the best model for a given number of regions. Columns named `BP` give the position of region boundaries (or breakpoints), each value corresponding to the position of the last element (vertebra) of a given region. The other columns provide the total residual sum of squares across all PC axes (`sumRSS`) and the residual sum of square for each PC axis individually (`RSS.`).

In the example above, in the model with three regions, the first region encompasses vertebrae 8 (first vertebra in the dataset) to 23, the second contains vertebrae 24 to 29, and the last region contains vertebrae 30 to 47 (last vertebra in the dataset).


### Selecting best overall model and region score

`modelsupport()` will find the best overall models across the models returned by `modelselect()`. The best model is **calculated using AICc or BIC** which allows the selection of the best fit model while penalizing for models that are too complex. *See @Jones2018 for AICc calculations and @GilRev for BIC calculations.* 

```{r}
supp <- modelsupport(models)
supp
```

**Models are ordered from best to worst fit**. Here, both AICc and BIC find the model with 5 regions as the best overall model with the following regions: *region1*: vertebrae 8 to 23, *region2*: vertebrae 24 to 27, *region3*: 28 to 34, *region4*: 35 to 40, *region5*: 41 to 47.

In addition to returning the best models, the function also calculates the **region score**, a continuous value accounting for the fact models with different number of regions could fit the data well. The region score is calculated as the sum of the number of region in each model weighted by their respective AICc/BIC weight. In this example, the region score for BIC is calculated as $5*1 + 4*0 + 3*0 + 2*0 + 1*0=5$.


### Adding more regions

In the example above, the model with 5 regions is by far the best one (AICc/BIC weight = 1). In such cases where the most complex model is the best model, it's likely that more complex models fit the data better. `addregions()` allows one to **fit additional models without the need to recompute models with fewer regions**.

```{r}
regionresults <- addregions(regionresults, noregions = 6:7, exhaus = TRUE,
                            verbose = FALSE)
summary(regionresults)
```

Identifying the best model now that models with 6 and 7 regions have been added:

```{r}
models <- modelselect(regionresults)
supp <- modelsupport(models)
supp
```

The best overall model now is the model with 7 regions both when using AICc and BIC. Since the best model is still the most complex model, additional model(s) with more than seven regions should be fitted (not run here for computational reasons).


### Model performance ($R^2$)

`modelperf()` returns the univariate (for each PC axis individually) and multivariate (all PC axes) $R^2$ and adjusted $R^2$ for a given model. The $R^2$ can be calculated for the best model (`model = 1`) for a given criterion (`criterion` argument), but also the second best (`model = 2`), third best (`model = 3`), etc. In this case, the output of `modelsupport()` must be supplied.

```{r}
modelperf(dolphin_pco, scores = PCOs, modelsupport = supp,
          criterion = "bic", model = 1)
```

Alternatively, it is also possible to calculate the $R^2$ for a user-defined model using the `bps` argument specifying the position of breakpoints (region boundaries) of the model to test.

```{r}
modelperf(dolphin_pco, scores = 1:3,
          bps = c(15, 24, 37), criterion = "bic")
```


### Breakpoint position variability

`calcBPvar()` calculates the variability in breakpoint position for a given number of regions (`noregions` argument) using weighted means and standard deviation based either on AICc or BIC weight (defined by `criterion` argument). The variability can be calculated on a given proportion of best models across all fitted models from this number of regions (`pct` argument). 

Here is an example of calculating the variability of breakpoint positions for models with 7 regions, using the top 10% best models evaluated with the BIC criterion.

```{r}
bpvar <- calcBPvar(regionresults, noregions = 7,
                   pct = 0.1, criterion = "bic")
bpvar
```

Note that when using output from `calcBPvar` for plotting, breakpoint positions will be rounded to the closest whole number. In the example above, breakpoints will then be 19, 24, 26, 35, 38, and 41. Since `calcregions` only returns whole numbers as breakpoint values, a weighted mean breakpoint position of 23.97 involves that most models found a breakpoint value of 24 (corresponding to a region break between vertebrae 24 and 25).

## Visualizing results

*MorphoRegions* offers two main plotting options to visualize the results: 1) scatter plots showing PC scores of each element (vertebra) along the structure (backbone) and the segmented linear regressions, 2) vertebral maps showing the regional identity of each element (vertebra).

### Segmented linear regression plots

The scatter plots of the segmented linear regressions can be plotted using `plotsegreg()`. The output of `svdPCO()` and the number of the PC axis to plot (`scores` argument) must be supplied.

The function can either be used to plot the best (or second best, third best, etc.) model (`model` argument) for a given criterion, by supplying the output of `modelsupport()`. Alternatively, a specific model can also be plotted by providing the position of the breakpoints (`bps` argument).

```{r, fig.show='hold', out.width='48%', fig.cap='**Fig.5.** Scatterplots showing PCO scores and segmented regressions on PC1 and PC2 for the best model *(left)* and on PC1 for a 4 regions model with breakpoints at vertebrae 15, 24, and 37 *(right)*.'}
# Plot regressions for best model:
plotsegreg(dolphin_pco, scores = 1:2,
           modelsupport = supp,
           criterion = "bic",
           model = 1)

# Plot regressions for a specific model:
plotsegreg(dolphin_pco, scores = 1,
           bps = c(15, 24, 37),
           cont = TRUE)
```



### Vertebral maps

`plotvertmap()` produces vertebral maps, which are plots in which each rectangle corresponds to an element (i.e., vertebra) that is color-coded according to the region to which it belongs. The function provides many different plotting options. 

The function requires `regions_data` (from `process_measurements()`), `regions_pco` (from `svdPCO()`), or `regions_sims` (from `simregions()`; see below) to be provided. The positional information of each element (i.e., vertebral number) will be extracted from one of these objects.

The regional identity of each element can be defined either by providing a `regions_modelsupport` object (from `modelsupport()`) or by directly specifying the position of breakpoints of a model.

```{r, fig.width=7, fig.height=0.7, fig.show='hold', fig.align='center', fig.cap='**Fig.6.** Vertebral maps of the best fit model using BIC criterion *(top)*, the 2^nd^ best fit model using AICc criterion *(middle)*, a user-defined model with breakpoints at vertebrae 12, 17, 20, and 30 *(bottom)*.'}
#Plot vertebral map using best BPs from modelsupport (BIC)
plotvertmap(dolphin_pco, name = "Dolphin A",
            modelsupport = supp, criterion = "bic", model = 1)

#Plot vertebral map using 2nd best BPs from modelsupport (AICc)
plotvertmap(dolphin_pco, name = "Dolphin B",
            modelsupport = supp, criterion = "aic", model = 2)

#Plot vertebral map using arbitrary breakpoints
plotvertmap(dolphin_pco, name = "Dolphin C",
            bps = c(12, 17, 20, 30))

```

Here the seven first elements (rectangles corresponding each to one vertebra) are grayed as they were not included in the analyses. It is possible to hide elements excluded from the analyses by setting the `dropNA` argument to `TRUE`.

**Variability in breakpoint positions** between regions can also be added either by supplying a `regions_BPvar` object (from `calcBPvar()`) or a user-defined vector. When a `regions_BPvar` object is supplied, the position of breakpoints corresponds to the weighted mean position computed by `calcBPvar()`.

```{r, fig.width=7, fig.height=0.7, fig.show='hold', fig.align='center', fig.cap="**Fig.7.** Vertebral maps showing breakpoint position variability calculated from *calcBPvar (top)* and with values directly supplied to the function *(bottom)*. Dots correspond to breakpoint position and horizontal lines correspond to the standard deviation of each breakpoint position."}
# BP variability from calcBPvar:
plotvertmap(dolphin_pco, name = "Dolphin",
           bpvar = bpvar, dropNA = TRUE)

# BP variability from user-defined vector, plotting SD bars in grey:
plotvertmap(dolphin_pco, name = "Dolphin",
            modelsupport = supp,
            criterion = "bic",
            bp.sd = c(2.2, 1.4, 2.5, 1.5, 2.1, 0.8),
            sd.col="#636363", dropNA = TRUE)
```



To facilitate the interpretation of the regions, the function offers some plotting options.

It is possible to **add other boundaries on the plot**, such as the boundaries between traditional anatomical regions (thoraco-lumbar, lumbo-caudal, etc). Boundary positions can be supplied as a vector to the `reg.lim` argument, with each numeric value corresponding to the position of the last element before the boundary. For example, in the dolphin dataset, the thoraco-lumbar, lumbo-caudal, and caudal-fluke transitions fall between vertebrae 17-18, 25-26, and 35-36, respectively.

It is also possible to plot the **positional info** of each element (vertebra) as a number on the corresponding rectangle.

```{r, fig.width=7, fig.height=0.7, fig.show='hold', fig.align='center', fig.cap="**Fig.8.** Vertebral maps with anatomical region boundaries represented by vertical bars *(top)* and vertebral position labelled on corresponding rectangle *(bottom)*."}
# Remove missing vertebrae, add vertebrae number 
  # and add thoraco-lumbar, lumbo-caudal, and caudal-fluke boundaries:
plotvertmap(dolphin_pco, name = "Dolphin", modelsupport = supp,
            criterion = "bic", dropNA = TRUE,
            reg.lim = c(17 ,25, 35),
            lim.col = c("red", "blue", "green"))

# Add positional info of each vertebra:
plotvertmap(dolphin_pco, name = "Dolphin", modelsupport = supp,
            criterion = "bic", dropNA = TRUE, text = TRUE)
```

By default, the function plots the elements (vertebrae) as count (vertebral count) but it is also possible to **plot elements as percentage of the total number of elements** (by setting `type = "percent"`).

In both cases, each element on the plot will have the same size. Alternatively, it is possible to **scale rectangles on the plot according to the relative size of each element** by supplying a vector containing size information to the `centraL` argument. In the example below, rectangles are scaled according to the centrum length of each vertebra (supplied in the `Lc` column of the original dataset).

```{r, fig.width=7, fig.height=0.7, fig.show='hold', fig.align='center', fig.cap="**Fig.9.** Vertebral maps plotting vertebrae as percentage of total vertebral count *(top)*, scaled to their centrum length relative to total length in absolute value *(middle)*, and scaled to their centrum length in percentage *(bottom)*."}
# Plot vertebrae as percentage of total count:
plotvertmap(dolphin_pco, name = "Dolphin",
            type = "percent",
            modelsupport = supp, criterion = "bic",
            dropNA = TRUE, reg.lim = c(17, 25, 35))

# Scale vertebrae relative to their centrum length (scale in cm):
plotvertmap(dolphin_pco, name = "Dolphin",
            type = "count", centraL = "Lc",
            modelsupport = supp, criterion = "bic",
            dropNA = TRUE, reg.lim = c(17, 25, 35))

# Scale vertebrae relative to their centrum length (scale in % total length):
plotvertmap(dolphin_pco, name = "Dolphin",
            type = "percent", centraL = "Lc",
            modelsupport = supp, criterion = "bic",
            dropNA = TRUE, reg.lim = c(17, 25, 35),
            lim.col = "grey")
```


Finally, it is possible to **change colors used for each region** with the `block.cols` argument. Colors can either be supplied as a vector, in which case regions will be *color-coded following the order of colors supplied in the vector*. 

Alternatively, it's also possible to *define a set of colors per block*, each block being defined by the `block.lim` argument. In this case, colors must be supplied as a named list, each element of the list containing a vector of color for a given block. For instance, for the dolphin data, we could define regions in the precaudal block (thoracic + lumbar) to be colors in shades of beige/brown, while regions in the caudal block (caudal + fluke) to be color in shades of blue. Since the lumbo-caudal transition falls between vertebrae 25 and 26, we'll define two blocks with a limit at vertebra 25.  


```{r, eval = requireNamespace("viridisLite", quietly = TRUE), message=FALSE, fig.width=7, fig.height=0.7, fig.show='hold', fig.align='center', fig.cap="**Fig.10.** Vertebral maps with regions colored with a user-defined palette using *viridis* *(top)* and custom palettes for different blocks *(bottom)*."}
# Manually color-coding each region using the viridis palette:
library(viridisLite)
nreg <- supp$Model_support_BIC[1,1]  # get nbr of regions
cols <- viridis(n = nreg)

plotvertmap(dolphin_pco, name = "Dolphin",
            type = "percent", centraL = "Lc",
            modelsupport = supp, criterion = "bic",
            dropNA = TRUE, 
            reg.lim = c(17, 25, 35),
            block.cols = cols)

# Color code regions by block (precaudal vs caudal):
blockcols <- list(PreCaudal = c("#BF812D","#8C510A","#543005"),
                  Caudal = c("#C7EAE5","#80CDC1","#35978F","#01665E","#236E30"))

plotvertmap(dolphin_pco, name = "Dolphin",
            type = "percent", centraL = "Lc",
            modelsupport = supp, criterion = "bic",
            dropNA = TRUE, 
            reg.lim = c(17, 25, 35),
            block.cols = blockcols, block.lim = 25)
```

*When coloring regions by blocks, regions spanning over more than one block will be allocated to the block in which the majority of its elements belong. In the example above, the 3^rd^ region has one vertebra in the precaudal block and two in the caudal block, thus this region is considered as the 1^st^ region in the caudal block.*

Vertebral maps are created using *ggplot2* meaning that they can be modified similarly to `ggplot` plots. For instance, it is possible to create a single plot with multiple vertebral maps with the *patchwork* package.

```{r, eval = requireNamespace("patchwork", quietly = TRUE), message=FALSE, fig.width=7, fig.height=1.7, fig.show='hold', fig.align='center', fig.cap="**Fig.11.** Vertebral maps of alligator, mouse, and dolphin stacked in a single figure."}
# Run regions analysis on alligator and mouse data:
dat_alligator <- process_measurements(alligator, pos = 1)
pco_alligator <- svdPCO(dat_alligator, "gower")
regions_alligator <- calcregions(pco_alligator, scores = 1:3,
                                 noregions = 5, verbose = FALSE)
supp_alligator <- modelsupport(modelselect(regions_alligator))

dat_mouse <- process_measurements(musm, pos = 1)
pco_mouse <- svdPCO(dat_mouse, "gower")
regions_mouse <- calcregions(pco_mouse, scores = 1:3,
                             noregions = 5, verbose = FALSE)
supp_mouse <- modelsupport(modelselect(regions_mouse))

# Create vertebral maps:
vertmaps <- list(alligator = plotvertmap(pco_alligator, name = "Alligator", centraL = "CL",
                                         modelsupport = supp_alligator, type = "percent",
                                         dropNA = TRUE),
                 mouse = plotvertmap(pco_mouse, name = "Mouse", centraL= "CL", 
                                     modelsupport = supp_mouse, type = "percent",
                                     dropNA = TRUE),
                 dolphin = plotvertmap(dolphin_pco, name = "Dolphin", centraL= "Lc",
                                       modelsupport = supp, type = "percent",
                                       dropNA = TRUE))

# Remove x axis for plots except bottom one:
library(ggplot2)
for (i in seq_along(vertmaps)[-length(vertmaps)]){
  vertmaps[[i]] <- vertmaps[[i]] + theme(axis.text.x = element_blank(),
                                         axis.ticks.x = element_blank(),
                                         axis.title.x = element_blank())
}

# Stack plots together:
library(patchwork)
wrap_plots(vertmaps) + plot_layout(ncol = 1, byrow = FALSE)

```

## Model fitting parameters {#FittingOptions}

As briefly described in the [section above](#SegRegFit), there are several different parameters available when fitting the segmented linear regressions. 

### Continuous _vs._ discontinuous fit {#ContDisc}

While the older *regions* package automatically fitted discontinuous segments, *MorphoRegions* allows users to choose between a continuous or discontinuous fit. In a continuous fit, the first point of the segment aligns with the last point of the preceding segment, modeling gradational variation without large jumps between segments. Because the continuous fit forces continuity between successive segments, it puts more constrains on the model fitting compared to a discontinuous fit. Depending on the dataset, one or the other fitting method might be appropriate. It's advised to visually inspect the distributions of elements along the structure using the PCO scores that will be implemented in the analysis (as in [Fig.2](#PCO)) before choosing an option.

```{r, fig.show='hold', out.width = "48%", fig.cap="**Fig.12.** Best fit on PCOs 1 and 2 of a 4 region models with breakpoints at 15, 24, and 37, using a discontinuous model (*left*) and a continuous model (*right*)."}
# Plot a discontinuous model:
plotsegreg(dolphin_pco, scores = 1:2,
           bps = c(15, 24, 37),
           cont = FALSE)

# Plot a continuous model:
plotsegreg(dolphin_pco, scores = 1:2,
           bps = c(15, 24, 37),
           cont = TRUE)
```


### Minimum number of elements per region {#MinVert}

By default, the minimum number of elements (vertebrae) in each region is set to 3. This parameter can, however, be changed with the `minvert` argument in the `calcregions` function. *Note that setting the minimum value to 2 might results in linear segments made of only two points, which will have a perfect fit (RSS=0 for these segments) and might artificially increase the goodness of fit.*

```{r, fig.show='hold', out.width = "48%", fig.align='center', fig.cap="**Fig.13.** Best fit on PCO 1 of models up to 5 regions with a minimum of 2 vertebrae per region (*left*) or 4 vertebrae per region (*right*)."}
# Fitting up to 5 regions for alligator data with 2 vertebrae minimum per segment:
reg.all.2 <- calcregions(pco_alligator, scores = 1, minvert = 2, noregions = 5,
                         cont = FALSE, verbose = FALSE)
plotsegreg(pco_alligator, scores = 1,
           modelsupport = modelsupport(modelselect(reg.all.2)),
           criterion = "bic")

# Fitting up to 5 regions for alligator data with 4 vertebrae minimum per segment:
reg.all.4 <- calcregions(pco_alligator, scores = 1, minvert = 4, noregions = 5,
                         cont = FALSE, verbose = FALSE)
plotsegreg(pco_alligator, scores = 1,
           modelsupport = modelsupport(modelselect(reg.all.4)),
           criterion = "bic")
```

When setting the minimum number of vertebrae per region to 2, the best fit model (using BIC) is the 5 regions model with RSS = 0.001 and BIC weight = 0.99.

When forcing the minimum number of vertebrae per region to 4, the best fit model (using BIC) is the 4 regions model with RSS = 0.002 and BIC weight = 0.61. The 5 regions model is only the 3^rd^ best model (after the 3 regions model) with RSS = 0.002 and BIC weight = 0.02.


### Forcing or preventing specific breakpoint positions {#ForceBP}

The `omitbp` and `includebp` arguments of `calcregions` allows one to prevent or force breakpoint(s) at specific position(s), respectively.

```{r, fig.show='hold', out.width = "48%", fig.align='center', fig.cap="**Fig.14.** Best fit on PCO 1 and 2 of models up to 5 regions preventing any breakpoint in the thoraco-lumbar region (vertebrae 8 to 25) *(top)*  and forcing breakpoints at anatomical region boundaries (vertebrae 17, 25, 35) *(bottom)*."}
# Fit models for dolphin preventing any BP in the thoracic and lumbar regions:
reg.dol.noBP <- calcregions(dolphin_pco, scores = PCOs, noregions = 5,
                            omitbp = 8:25, verbose = FALSE)
plotsegreg(dolphin_pco, scores = PCOs,
           modelsupport = modelsupport(modelselect(reg.dol.noBP)),
           criterion = "bic")

# Fit models for dolphin forcing BPs at anatomical region boundaries:
reg.dol.forceBP <- calcregions(dolphin_pco, scores = PCOs, noregions = 5,
                               includebp = c(17, 25, 35), verbose = FALSE)
plotsegreg(dolphin_pco, scores=PCOs,
           modelsupport = modelsupport(modelselect(reg.dol.forceBP)),
           criterion = "bic")
```


### Exhaustive _vs._ non-exhaustive search {#Exhaus}

*MorphoRegions* has been implemented to fit numerous models efficiently enough to run in a few minutes on most computers. However, in the case of quite complex models (numerous elements, regions to fit, and/or PCO scores), computational time can quickly increase. To speed up the process, it is possible to fit models in a non-exhaustive way by setting `exhaus = FALSE` in `calcregions()`.

When a non-exhaustive search is used, all possible models for 1 and 2 regions models are fitted, but for 3 and more regions, the algorithm will only fit a subset of all the possible models. For a given number of regions, $n$, the optimal positions of breakpoints of the models with $n-1$ regions are used as a starting point to define the subset of models to fit.

```{r}
reg.dol.Nex <- calcregions(dolphin_pco, scores = PCOs, noregions = 5,
                           exhaus = FALSE, verbose = FALSE)
summary(reg.dol.Nex)
```

In a non-exhaustive search, the `summary()` function used on `calcregions()` returns, for each number of regions, the total number of possible models, the number of models actually fitted, and the number of models saved to output in order to limit memory usage.

If the variability of breakpoint positions is calculated on non-exhaustive search output, the proportion of best models used to calculate the variability corresponds to the given percentage of all possible models. It's therefore possible that in some cases the number of models saved by the non-exhaustive search is lower than the requested percentage. In such cases, the variability will be calculated on all saved models and a warning message is returned.

```{r}
calcBPvar(reg.dol.Nex, noregions = 5, pct = 0.1, criterion = "bic")
```

Because average and variability of breakpoint position is calculated using weighted means and SD, respectively, results usually remain similar to those obtained using full top 10% (models fitted with an exhaustive search) unless there are numerous models which fit the data well.

```{r}
calcBPvar(regionresults, noregions = 5, pct = 0.1, criterion = "bic")
``` 


### Parallel computing {#Para}

For specimens with a high number of elements and/or numerous regions and for which computing non-exhaustive searches are still time consuming, the `cl` argument in `calcregions()` allows one to implement parallel computing. See the `cl` argument to `pbapply::pblapply()` for more details on how it can be specified. Below, we request 6 cores to be used (supplying a number is not available on Windows).

```{r, eval=F}
# Exhaustive search with parallel computing using 6 cores:
calcregions(dolphin_pco, scores = 2, noregions = 10, exhaus = TRUE,
            cl = 6, verbose = FALSE)
```

Note that parallel computing will only be significantly faster on heavy computing specimens.

## Intrapsecific variability {#IntraSp}

To account for variability within a species or group, it is possible to **run the regionalization analysis on multiple specimens in a single analysis**.
The `porpoise` dataset contains data of 3 different specimens of harbor porpoise.


```{r, eval=T}
data("porpoise")
```
```{r, echo=F}
x <- rbind("Porpoise 1" = dim(porpoise1),
           "Porpoise 2" = dim(porpoise2),
           "Porpoise 3" = dim(porpoise3))
colnames(x) <- c("Elements", "Variables")
knitr::kable(x)
```

The total number of elements (vertebrae) can differ among specimens, but the number of variables and their order must be the same. The highest number of elements is used to generate the vertebral map. In the porpoise dataset, the first specimen has 58 post-cervical vertebrae, the second has 56, and the third one has 59, but they all have 17 variables. In the case of differing number of elements, homologous elements (vertebrae) must be identified by the same number in the positional information column of the data.
 

```{r, fig.show='hold', fig.align="center", out.width="65%", fig.cap="**Fig.15.** PCO plots of the three specimens of harbor porpoise."}
# Process measurements data
porpoise_data <- process_measurements(list(porpoise1, porpoise2, porpoise3), pos = "Vertebra")

# Compute PCOs
porpoise_pco <- svdPCO(porpoise_data, "gower")

# Plot PCOs
plot(porpoise_pco, 1:2)
plot(porpoise_pco, pco_x = 1, pco_y = 2)
```

Once data of each specimen have been supplied to the `process_measurements` function, **the process to run the analysis is similar to running the analysis on a single specimen at a time**.

```{r, warning=FALSE}
# Calculate number of PCOs to use in subsequent analyses
PCOs <- PCOselect(porpoise_pco, "variance", cutoff = .05)

# Performance for all models across all breakpoint combos
regionresults <- calcregions(porpoise_pco, scores = PCOs,
                             noregions = 3, verbose = FALSE)

# Adding more models to existing results
regionresults <- addregions(regionresults, noregions = 4:5,
                            exhaus = FALSE, verbose = FALSE)

# Select best model
models <- modelselect(regionresults)
supp <- modelsupport(models)
supp

# Calculate variability of optimal breakpoints
bpvar <- calcBPvar(regionresults, noregions = 5, pct = .05,
                   criterion = "bic")
bpvar

# Calculate model performance (R2) of best model
modelperf(porpoise_pco, scores = PCOs, modelsupport = supp,
          criterion = "bic", model = 1)

```

```{r, out.width="60%", fig.align='center', fig.cap="**Fig.16.** Best fit model (based on BIC) on PCO 1 and 2 calculated on the three harbor porpoise specimens."}
# Plot PCOs of best fit model
plotsegreg(porpoise_pco, scores = 1:2, modelsupport = supp,
            criterion = "bic", model = 1)
```

```{r, fig.width=7, fig.height=0.7, fig.align='center', fig.cap="**Fig.17.** Vertebral map of best fit model (including variance in breakpoint position) based on the three harbor porpoise specimens."}
# Plot vertebral map of best fit model
plotvertmap(porpoise_pco, name = "Porpoise", 
            type = "percent", centraL = "Lc",
            reg.lim = c(20, 34, 54), dropNA = TRUE,
            bpvar = bpvar, sd.col = "grey20")
```

## Other useful options

### Fit a specific model

Instead of fitting all possible models for a given range of region number, `calcmodel()` allows one to fit a single model with user-defined breakpoint positions. 

```{r, fig.width=5.5, fig.height=5.5, out.width="60%", fig.cap="**Fig.18.** Best fit segmented linear regression model on PCO 1-3 when forcing breakpoint position at vertebrae 19, 24, 27, 32, 35, 38, and 41."}
#Fit a single breakpoints model
regionsmodel <- calcmodel(dolphin_pco, scores = 1:3, cont = TRUE,
                          bps = c(19, 24, 27, 32, 35, 38, 41))
regionsmodel

#Evaluate performance (R2) on that model
modelperf(regionsmodel)

#Plot model results:
plotsegreg(regionsmodel, scores = 1:3)

```


### Subsampling elements

For specimens with numerous elements (more than 100), computational time can become high. It can then be useful to run the regionalization analysis on a subset of elements If the analyses are run in a comparative framework and there is substantial variation of the number of elements across specimens, it is also advised to subsample elements to a constant number of elements for all specimens.

`subsample()` allows one to subsample elements either by providing the number of elements to retain or the proportion of total elements to retain. 
```{r, fig.width=7, fig.height=0.7, fig.show='hold', fig.align='center', fig.cap='**Fig.19.** Vertebral map of dolphin dataset with 33 subsampled vetrebrae *(top)* and 70% of total vertebrae subsampled *(bottom)*. Vertebrae dropped are in grey and vertebrae retained are in blue shade*.'}
# Retain 33 vertebrae
dolphin_pco_sub.33 <- subsample(dolphin_pco, sample = 33)

# Retain 70% of initial number of vertebrae
dolphin_pco_sub.7 <- subsample(dolphin_pco, sample = .7)

plotvertmap(dolphin_pco_sub.33, name = "Dolphin 33")
plotvertmap(dolphin_pco_sub.7, name = "Dolphin .7")
```

By default, elements to drop will be selected to be equidistant along the structure (rounded to integers). A random subsampling of elements along the structure can be applied by supplying `type = "random"` to `subsample()`.

### Simulating data

`simregions()` allows one to simulate datasets that satisfy some user-defined constraints such as number of elements, regions, PCO axes, minimum number of elements per region, variance around the linear fits, and difference in slope between successive linear segments. 

```{r, out.width='65%', fig.cap="**Fig.20.** Plot on PCO 1 and 2 of the **simulated dataset** with 40 vertebrae and 5 regions (4 breakpoints)."}
# Simulate data 
set.seed(2024)
sim <- simregions(nvert = 40, nregions = 5, nvar = 4, r2 = .92,
                  minvert = 3, cont = TRUE)

# Plot:
plot(sim, scores = 1:2)
```


The regionalization analysis can then be run on the simulated dataset.

```{r, out.width='65%', fig.show='hold', fig.cap="**Fig.21.** Plot on PCO 1 and 2 of the **recovered best fit** on regionalization analysis ran on the simulated dataset."}
simresults <- calcregions(sim, scores = 1:4, noregions = 6, verbose = FALSE)

simmodels <- modelselect(simresults)
simmodels

simsupport <- modelsupport(simmodels)
simsupport

modelperf(sim, modelsupport = simsupport, criterion = "aic", model = 1)

#Plot best model fit
plotsegreg(sim, scores = 1:2, modelsupport = simsupport,
            criterion = "aic", model = 1)

```


The simulated dataset had 5 predefined regions with breakpoints at vertebrae 16, 20, 29, and 34. When running the regionalization analysis on the simulated dataset, the analysis correctly recovered the 5 regions model as the best fit model using AICc, however the 5 regions model is second best using BIC. 

The AICc best fit model recovered breakpoints at vertebrae 15, 21, 28, 34. The recovered position of the first three breakpoints differ sightly from the simulated positions, and the recovered position of the fourth breakpoint matches that of the simulated data.

```{r}
# Simulated data breakpoints:
sim$BPs

# Recovered best fit breakpoints:
simsupport$Model_support[1,2:5]

# Variability of recovered best fit breakpoints:
calcBPvar(simresults, noregions = 5, pct = 0.1, criterion = "aic")
```

## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcregions.R
\name{calcregions}
\alias{calcregions}
\alias{addregions}
\alias{summary.regions_results}
\alias{ncombos}
\title{Fit segmented regression models for all combinations of breakpoints}
\usage{
calcregions(
  pco,
  scores,
  noregions,
  minvert = 3,
  cont = TRUE,
  exhaus = TRUE,
  includebp = NULL,
  omitbp = NULL,
  ncombos_file_trigger = 1e+07,
  temp_file_dir = tempdir(TRUE),
  cl = NULL,
  verbose = interactive()
)

addregions(
  regions_results,
  noregions,
  exhaus = TRUE,
  ncombos_file_trigger = 1e+07,
  temp_file_dir = tempdir(TRUE),
  cl = NULL,
  verbose = interactive()
)

\method{summary}{regions_results}(object, ...)

ncombos(pco, noregions, minvert = 3, includebp = NULL, omitbp = NULL)
}
\arguments{
\item{pco}{a \code{regions_pco} object; the output of a call to \code{\link[=svdPCO]{svdPCO()}}.}

\item{scores}{\code{numeric}; the indices of the PCO scores to use as the outcomes in fitting the models (e.g., \code{1:4} to use the first four scores). Can also be the output of a call to \code{\link[=PCOselect]{PCOselect()}}.}

\item{noregions}{\code{numeric}; for \code{calcregions()}, the maximum number of regions for which models are fit (e.g, 4 to request models with 1 to 4 regions); for \code{addregions()}, a vector containing the numbers of regions to add (e.g., 5:6 to request models with 5 and 6 regions); for \code{ncombos()}, a vector containing the numbers of regions to check.}

\item{minvert}{\code{numeric}; the minimum number of vertebrae allowed in each region. Default is 3.}

\item{cont}{\code{logical}; whether to fit models that are continuous (\code{TRUE}) or discontinuous (\code{FALSE}) at the breakpoints. Default is \code{TRUE}.}

\item{exhaus}{\code{logical}; whether to fit all possible models (\code{TRUE}) or use heuristics to reduce the number of models fit (\code{FALSE}). Default is \code{TRUE}. See Details. Setting to \code{FALSE} can reduce the size of the resulting object.}

\item{includebp}{an optional vector of vertebrae that must be included in any tested set of breakpoints, e.g., if it is known that two regions are divided at that vertebra. \code{includebp} does not have to obey the \code{minvert} rules, but a warning will be thrown if it doesn't.}

\item{omitbp}{an optional vector of vertebrae to be omitted from the list of possible breakpoints, e.g., if it is known that two adjacent vertebrae belong to the same region.}

\item{ncombos_file_trigger}{\code{numeric}; when the number of eligible combinations of breakpoints exceeds this number, the problem will be split into smaller problems, with the results of each stored in its own temporary file in the directory supplied to \code{temp_file_dir} before being re-read into memory. The primary purpose of this is to preserve memory when \code{exhaus = FALSE} by delegating storage of the results to disk instead of RAM.}

\item{temp_file_dir}{string; the directory where the temporary files will be saved (and then deleted) when the number of breakpoint combinations exceeds \code{ncombos_file_trigger}. Default is the directory produced by \code{\link[=tempdir]{tempdir()}}, but it is much safer to provide your own directory, which must already exist on your machine. See Details.}

\item{cl}{a cluster object created by \code{\link[parallel:makeCluster]{parallel::makeCluster()}}, an integer to indicate number of child-processes (integer values are ignored on Windows) for parallel evaluations, or \code{"future"} to use a future backend. \code{NULL} (the default) refers to sequential evaluation (no parallelization). See \code{\link[pbapply:pbapply]{pbapply::pbapply()}} for details.}

\item{verbose}{\code{logical}; whether to print information about the fitting process, including a progress bar. Default is \code{TRUE} when running interactively and \code{FALSE} otherwise.}

\item{regions_results, object}{a \code{regions_results} object; the output of a call to \code{calcregions()} or \code{addregions()}.}

\item{\dots}{ignored.}
}
\value{
A \code{regions_results} object with the following components:
\itemize{
\item \code{results} - the results of the fitting process for each combination of breakpoints
\item \code{stats} - statistics summarizing the fitting process. Use \code{summary()} to view this information in a clean format.
}

\code{ncombos()} returns a numeric vector with the number of breakpoint combinations for each number of regions (which are stored as the names).
}
\description{
\code{calcregions()} enumerates all possible combinations of breakpoints to fit multivariate segmented regression models. \code{addregions()} adds models with additional numbers of regions to the resulting output object. \code{ncombos()} computes an upper bound on the number of breakpoint combinations that will be tested.
}
\details{
\code{calcregions()} enumerates all possible combinations of breakpoints that satisfy the constraint imposed by \code{minvert} (i.e., that breakpoints need to be at least \code{minvert} vertebrae apart) and fits the segmented regression models implied by each combination. These are multivariate regression models with the PCO scores specified by \code{scores} as the outcomes. When \code{cont = TRUE}, these regression models are continuous; i.e., the regression lines for each region connect at the breakpoints. Otherwise, the models are discontinuous so that each region has its own intercept and slope. The models are fit using \code{\link[=.lm.fit]{.lm.fit()}}, which efficiently implements ordinary least squares regression.

When \code{exhaus = FALSE}, heuristics are used to reduce the number of models to fit, which can be useful for keeping the size of the resulting object down by avoiding fitting models corresponding to breakpoint combinations that yield a poor fit to the data. Only breakpoint combinations that correspond to the breakpoints of the best fitting model with a smaller number of regions +/- 3 vertebrae are used, and only models that have an RSS smaller than half a standard deviation more the smallest RSS are kept.

\code{addregions()} should be used on an existing \code{regions_results} object to add models with more regions. Internally, it works just the same as \code{calcregions()}.

\code{ncomobs()} computes an upper bound on the number of possible breakpoint combinations. When \code{exhaus = FALSE} or \code{includebp} is specified, the actual number of combinations will be smaller than that produced by \code{ncombos()}.

When the number of possible combinations of breakpoints for a given number of regions (as computed by \code{ncombos()}) is larger than \code{ncombos_file_trigger}, the problem will be split into smaller problems, with the results of each stored in temporary files that are deleted when the function completes. These temporary files will be stored in the directory supplied to \code{temp_file_dir}. By default, this is the temporary directory produced by \code{\link[=tempdir]{tempdir()}}. However, this directory can be deleted by R at any time without warning, which will cause the function to crash, so it is a good idea to supply your own directory that will be preserved. You can use \code{ncombos()} to check to see if the number of breakpoint combinations exceeds \code{ncombos_file_trigger}.
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Fit segmented regression models for 1 to 5 regions
# using PCOs 1 to 4 and a continuous model with a
# non-exhaustive search
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 5,
                             minvert = 3,
                             cont = TRUE,
                             exhaus = FALSE)

regionresults

# View model fitting summary
summary(regionresults)

# Add additional regions to existing results,
# exhaustive search this time
regionresults <- addregions(regionresults,
                            noregions = 6:7,
                            exhaus = TRUE)

regionresults

summary(regionresults)

# Fit segmented regression models for 1 to 5 regions
# using PCOs 1 to 4 and a discontinuous model with a
# exhaustive search, excluding breakpoints at vertebrae
# 10 and 15
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 5,
                             minvert = 3,
                             cont = FALSE,
                             omitbp = c(10, 15))

regionresults

summary(regionresults)

# Compute the number of breakpoint combinations for given
# specification using `ncombos()`; if any number exceeds
# the value supplied to `ncombos_file_trigger`, results
# will temporary be stored in files before being read in and
# deleted.
ncombos(alligator_PCO,
        noregions = 1:8,
        minvert = 3)
}
\seealso{
\code{\link[=calcmodel]{calcmodel()}} to fit a segmented regression model for a single set of breakpoints; \code{\link[=modelselect]{modelselect()}} to select the best model for each number of regions based on RSS; \code{\link[=modelsupport]{modelsupport()}} to compute statistics the describe the support of the best models; \code{\link[=calcBPvar]{calcBPvar()}} to compute the variability in the optimal breakpoints.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_input.R
\name{process_gmPC}
\alias{process_gmPC}
\title{Process vertebral measurements and PC scores of 2D or 3D geometric morphometric data}
\usage{
process_gmPC(data, pos = "names", pcscores, eigenvals, specimens = NULL)
}
\arguments{
\item{data}{a 3D array (\eqn{p \times k \times n}) of Procrustes aligned landmarks where \eqn{p} is the number of of landmark points per vertebra, \eqn{k} is the number of landmark dimensions (2D or 3D), and \eqn{n} is the number of vertebrae sampled for the specimen.}

\item{pos}{either character string \code{"names"} in which case the function will extract vertebral position info form the names of the array (using \code{dimnames(data)[[3]]}), or a vector containing numeric values of vertebral position. Default is \code{"names"}.}

\item{pcscores}{a matrix or data frame containing PC scores of each vertebra.}

\item{eigenvals}{a numeric vector containing the eigenvalues of each PC axis.}

\item{specimens}{if \code{data} contains multiple specimens, a vector of specimen names of length equal to the number of vertebrae in \code{data}.}
}
\value{
A \code{regions_pco} object, which contains user-provided eigenvectors in the \code{scores} component and eigenvalues in the \code{eigen.val} component. The original dataset, including landmark coordinates and positional information, is stored in the \code{data} attribute.
}
\description{
\code{process_gmPC()} initializes the analysis workflow by processing vertebral measurements and already computed PC scores from 2D or 3D geometric morphometric datasets into an object usable by \pkg{MorphoRegions}. Such processing includes identifying the vertebra indices, measurements, and PC scores.
}
\details{
Unlike \code{process_measurements()}, \code{process_gmPC()} does not fill in missing values and these should have been removed or replaced by numeric values before running \code{process_gmPC()}.
}
\examples{
# Load coordinates of aligned landmarks, PC scores, and
# eigenvalues; vertebra index as names of array elements
# of aligned landmarks
data("seal_gmm")

# Process PC scores and morphological data:
seal_pca <- process_gmPC(data = seal_gmm$coord_gpa,
                         pcscores = seal_gmm$scores,
                         eigenvals = seal_gmm$eigenvals)

# Process multiple datasets; specimen names in 'specimens'
# vector, vertebrae index in 'vertebrae' vector
data("wolves_gmm")

# Process PC scores and geometric morphometric dataset:
wolves_pca <- process_gmPC(data = wolves_gmm$coord_gpa,
                           pos = wolves_gmm$vertebrae,
                           pcscores = wolves_gmm$scores,
                           eigenvals = wolves_gmm$eigenvals,
                           specimens = wolves_gmm$specimens)
}
\seealso{
\itemize{
\item \code{\link[=process_PC]{process_PC()}} for processing PC scores from traditional morphometric datasets
\item \code{\link[=svdPCO]{svdPCO()}} for computing principal coordinate axes from processed vertebra data.
\item \code{\link[=plot.regions_pco]{plot.regions_pco()}} for plotting PCO axes
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{alligator}
\alias{alligator}
\title{Measurements from the vertebral column of an alligator}
\format{
A matrix with 22 vertebrae and 19 measurements. Column 1, vertebra, is the positional
information.
}
\usage{
alligator
}
\description{
Linear and angular measurements from Alligator mississipiensis MCZ 81457
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelselect.R
\name{modelselect}
\alias{modelselect}
\title{Select the best models}
\usage{
modelselect(results, scores = NULL)
}
\arguments{
\item{results}{a \code{regions_results} object; the output of a call to \code{calcregions()} or \code{addregions()}.}

\item{scores}{\code{numeric}; a vector corresponding to the indices of the PCOs the \eqn{R^2} of which will be used to determine the best model for each number of regions. If \code{NULL}, the default, all PCOs used included in the fitting will be used.}
}
\value{
A \code{regions_modelselect} object, which contains information about the best models for each number of regions extracted from \code{results}.
}
\description{
\code{modelselect()} narrows down the search for the best model by identifying the best model for each number of regions as determined by its residual sums of squares (RSS).
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Fit segmented regression models for 1 to 7 regions
# using PCOs 1 to 4 and a continuous model with a
# non-exhaustive search
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 7,
                             minvert = 3,
                             cont = TRUE,
                             exhaus = FALSE,
                             verbose = FALSE)

regionresults

# For each number of regions, identify best
# model based on minimizing RSS
bestresults <- modelselect(regionresults)
bestresults

# Evaluate support for each model and rank models
supp <- modelsupport(bestresults)
supp

# 5 regions best based on AICc; 6 regions based on BIC
}
\seealso{
\code{\link[=modelsupport]{modelsupport()}} for computing statistics that describe the support of each model using information criteria; \code{\link[=modelperf]{modelperf()}} for computing fit statistics for selected models.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{musm}
\alias{musm}
\title{Measurements from the vertebral column of a mouse}
\format{
A matrix with 23 vertebrae and 19 measurements. Column 1, vertebra, is the positional
information.
}
\usage{
musm
}
\description{
Linear and angular measurements from Mus musculus MCZ 59560
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{seal_gmm}
\alias{seal_gmm}
\title{Geometric morphometric data and PC scores of the vertebral column a seal}
\format{
A list containing: a 3D array of Procrustes aligned 3D landmarks (\code{coord_gpa}) of 25 presacral vertebrae with 40 landmarks per vertebra, a matrix of PC scores from a PCA performed on Procrustes aligned landmarks (\code{scores}),
and a numeric vector with the eigenvalues from the PCA (\code{eigenvals}).
}
\usage{
data("seal_gmm")
}
\description{
Procrustes aligned 3d landmarks (\code{coord_gpa}) and PC scores (\code{scores}) of the backbone of Phoca vitulina (ZMUC 160).
}
\references{
Esteban, J. M., Martín-Serra, A., Pérez-Ramos, A., Mulot, B., Jones, K. E., & Figueirido, B.  (2023). The impact of the land-to-sea transition on evolutionary integration and modularity of the pinniped backbone. Communications Biology, 6, 1141.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcmodel.R
\name{calcmodel}
\alias{calcmodel}
\title{Calculate results of a single segmented regression model}
\usage{
calcmodel(x, scores, bps, cont = TRUE)
}
\arguments{
\item{x}{a \code{regions_pco} object; the output of a call to \code{svdPCO()}.}

\item{scores}{\code{numeric}; the indices of the PCO scores to use as the outcomes in fitting the model (e.g., \code{1:4} to use the first four scores).}

\item{bps}{\code{numeric}; the indices of the breakpoints to use in fitting the model. To request a model with no breakpoints, set to \code{NA}.}

\item{cont}{\code{logical}; whether to fit a model that is continuous (\code{TRUE}) or discontinuous (\code{FALSE}) at the breakpoints. Default is \code{TRUE}. Ignored when \code{bps} is \code{NA}.}
}
\value{
A \code{regions_results_single} object, which contains the results of the model (breakpoints and RSS of each PCO and overall) and model support statistics.
}
\description{
\code{calcmodel()} fits a multivariate segmented regression model using the supplied PCOs and breakpoints.
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Calculate a single segmented regression model
# using first 2 PCOs and a discontinuous model
regionsmodel <- calcmodel(alligator_PCO,
                          scores = 1:3,
                          bps = c(8, 16, 21),
                          cont = FALSE)

regionsmodel

#Evaluate performance (R2) on that model
modelperf(regionsmodel)

#Plot model results:
plotsegreg(regionsmodel, scores = 1:3)
}
\seealso{
\code{\link[=calcregions]{calcregions()}} and \code{\link[=addregions]{addregions()}} for computing all possible models instead of just a single one; \code{\link[=plotsegreg]{plotsegreg()}}, for which the \code{plot} method is an alias, for plotting the fitted regression lines; \code{\link[=modelsupport]{modelsupport()}} for interpreting the model support statistics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svdPCO.R
\name{svdPCO}
\alias{svdPCO}
\title{Calculate PCO (principal co-ordinates analysis) based on SVD}
\usage{
svdPCO(x, metric = "gower", scale = TRUE)
}
\arguments{
\item{x}{a \code{regions_data} object; the output of a call to \code{\link[=process_measurements]{process_measurements()}}.}

\item{metric}{string; the distance matrix calculation metric. Allowable options include those support by \code{\link[cluster:daisy]{cluster::daisy()}}, which are \code{"euclidean"}, \code{"manhattan"}, or \code{"gower"}. Default is \code{"gower"}. Abbreviations allowed.}

\item{scale}{\code{logical}; whether to scale the variables prior to including them in the PCO estimation. Default is \code{TRUE}, which is especially advisable when using the bootstrap to select the number of PCOs to use in downstream analyses. Passed to the \code{stand} argument of \code{cluster::daisy()}. Ignored if \code{metric = "gower"}.}
}
\value{
A \code{regions_pco} object, which contains eigenvectors in the \code{scores} component and eigenvalues in the \code{eigen.val} component. The original dataset is stored in the \code{data} attribute.
}
\description{
Calculates distance matrix from raw data, then conducts a PCO ordination using a
single value decomposition (SVD). This differs from other PCO functions which use \code{\link[stats:cmdscale]{stats::cmdscale()}} and rely on a
spectral decomposition.
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data,
                        metric = "gower")

alligator_PCO

# Plot PCOs against vertebra index
plot(alligator_PCO, pco_y = 1:2)

# Plot PCOs against each other
plot(alligator_PCO, pco_y = 1, pco_x = 2)
}
\seealso{
\itemize{
\item \code{\link[=plot.regions_pco]{plot.regions_pco()}} for plotting PCO axes
\item \code{\link[cluster:daisy]{cluster::daisy()}}, which is used to compute the distance matrix used in the calculation
\item \code{\link[stats:cmdscale]{stats::cmdscale()}} for a spectral decomposition-based implementation
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelperf.R
\name{modelperf}
\alias{modelperf}
\alias{modelperf.regions_pco}
\alias{modelperf.regions_sim}
\alias{modelperf.regions_results_single}
\title{Assess model performance}
\usage{
modelperf(x, ...)

\method{modelperf}{regions_pco}(
  x,
  scores,
  modelsupport = NULL,
  criterion = "aic",
  model = 1,
  bps = NULL,
  cont = TRUE,
  ...
)

\method{modelperf}{regions_sim}(
  x,
  scores = NULL,
  modelsupport = NULL,
  criterion = "aic",
  model = 1,
  bps = NULL,
  cont = TRUE,
  ...
)

\method{modelperf}{regions_results_single}(x, scores = NULL, ...)
}
\arguments{
\item{x}{a \code{regions_pco} object, the output of a call to \code{\link[=svdPCO]{svdPCO()}}, or a \code{regions_results_single} object, the output of a call to \code{\link[=calcmodel]{calcmodel()}}.}

\item{\dots}{ignored.}

\item{scores}{\code{numeric}; the indices of the PCO scores for which fit statistics are to be computed.}

\item{modelsupport}{a \code{regions_modelsupport} object, the output of a call to \code{\link[=modelsupport]{modelsupport()}}. When \code{x} is a \code{regions_pco} object, either \code{modelsupport}, \code{criterion}, and \code{model} must be supplied or \code{bps} and \code{cont} must be supplied. See Details.}

\item{criterion}{string; the criterion to use to select the best model for which breakpoints are to be displayed when \code{modelsupport} is specified. Ignored otherwise. Allowable options include \code{"aic"} to use the AICc and \code{"bic"} to use the BIC. Abbreviations allowed. Default is \code{"aic"}. When \code{x} is a \code{regions_pco} object, either \code{modelsupport}, \code{criterion}, and \code{model} must be supplied or \code{bps} and \code{cont} must be supplied. See Details.}

\item{model}{\code{numeric}; for which model among the best as determined by \code{criterion} should fit statistics be computed. 1 is the best model, 2 the second best, etc. Default is 1. When \code{x} is a \code{regions_pco} object, either \code{modelsupport}, \code{criterion}, and \code{model} must be supplied or \code{bps} and \code{cont} must be supplied. See Details.}

\item{bps}{\code{numeric}; a vector of breakpoints for which model fit should be computed. When \code{x} is a \code{regions_pco} object, either \code{modelsupport}, \code{criterion}, and \code{model} must be supplied or \code{bps} and \code{cont} must be supplied. See Details.}

\item{cont}{\code{logical}; whether to fit a model that is continuous (\code{TRUE}) or discontinuous (\code{FALSE}) at the breakpoints supplied to \code{bps}. Default is \code{TRUE}. When \code{x} is a \code{regions_pco} object, either \code{modelsupport}, \code{criterion}, and \code{model} must be supplied or \code{bps} and \code{cont} must be supplied. See Details.}
}
\value{
A \code{regions_perf} object containing the breakpoints of the specified model, the univariate \eqn{R^2} and adjusted \eqn{R^2} statistics for each PCO score, and the multivariate \eqn{R^2} and adjusted \eqn{R^2} statistics.
}
\description{
\code{modelperf()} computes model performance statistics in the form of \eqn{R^2} measures for a given combination of breakpoints.
}
\details{
\code{modelperf()} operates on a single model identified by breakpoints and whether the model is continuous or not. When \code{x} is a \code{regions_pco} object, the model is selected either as the best model in the supplied \code{modelsupport} object (where "best" is determined by the arguments to \code{criterion} and \code{model}) or as specified by the user using the arguments to \code{bps} and \code{cont}. When \code{x} is a \code{regions_results_single} object, the breakpoints and model form are determined based on the supplied object.
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Evaluate model performance (R2) given supplied
# breakpoints for a continuous model
modelperf(alligator_PCO, scores = 1:3,
          bps = c(7, 15, 20), cont = TRUE)

plotsegreg(alligator_PCO, scores = 1:3,
           bps = c(7, 15, 20), cont = TRUE)
## See also `?calcmodel` for use with a single model

# Fit segmented regression models for 1 to 7 regions
# using PCOs 1 to 4 and a continuous model with a
# non-exhaustive search
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 7,
                             minvert = 3,
                             cont = TRUE,
                             exhaus = FALSE,
                             verbose = FALSE)

regionresults

# For each number of regions, identify best
# model based on minimizing RSS
bestresults <- modelselect(regionresults)

# Evaluate support for each model and rank
supp <- modelsupport(bestresults)

# Evaluate model performance (R2) for best model
# as chosen by BIC
modelperf(alligator_PCO, scores = 1:4,
          modelsupport = supp,
          criterion = "bic", model = 1)

# Plot that model for the first PCO score
plotsegreg(alligator_PCO, scores = 1:4,
           modelsupport = supp,
           criterion = "bic", model = 1)

## See `?simregions` for use with simulated data
}
\seealso{
\code{\link[=modelsupport]{modelsupport()}} for assessing model support using information criteria; \code{\link[=calcmodel]{calcmodel()}} for fitting a single segmented regression model; \code{\link[=plotsegreg]{plotsegreg()}} for plotting the results of a single segmented regression model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.regions_pco.R
\name{plot.regions_pco}
\alias{plot.regions_pco}
\title{Plot PC axes}
\usage{
\method{plot}{regions_pco}(x, pc_y = 1, pc_x = NULL, ...)
}
\arguments{
\item{x}{a \code{regions_pco} object; the output of a call to \code{\link[=svdPCO]{svdPCO()}}.}

\item{pc_y, pc_x}{number; PC score indices for the y- and x-axes, respectively. \code{pc_x} can be \code{NULL}.}

\item{...}{arguments passed to \code{\link[ggplot2:geom_text]{ggplot2::geom_text()}} when \code{pc_x} is not \code{NULL}. If \code{scores} is supplied as an argument, it will replace \code{pc_y} if unspecified.}
}
\value{
A \code{ggplot} object.
}
\description{
\code{plot()} visualizes the relationship between a PCO axis and the vertebra or between pairs of PCO axes.
}
\details{
When \code{pc_x} is \code{NULL} (the default), \code{plot()} will display a scatterplot of the PCO axis identified by \code{pc_y} and vertebra position using \code{\link[ggplot2:geom_point]{ggplot2::geom_point()}}. This plot is similar to that generated by \code{\link[=plotsegreg]{plotsegreg()}}. Otherwise, \code{plot()} uses \code{\link[ggplot2:geom_text]{ggplot2::geom_text()}} to identify vertebrae positions in the space corresponding to the requested PCOs.
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data,
                        metric = "gower")

alligator_PCO

# Plot PCOs against vertebra index
plot(alligator_PCO, pco_y = 1:2)

# Plot PCOs against each other
plot(alligator_PCO, pco_y = 1, pco_x = 2)
}
\seealso{
\code{\link[=svdPCO]{svdPCO()}} for generating the PCO scores. \code{\link[=plot.regions_sim]{plot.regions_sim()}} for plotting PCO scores against vertebra position for simulated PCOs. \code{\link[=plotsegreg]{plotsegreg()}} for plotting PCO scores against vertebra position after selecting breakpoints for a segmented regression.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelsupport.R
\name{modelsupport}
\alias{modelsupport}
\title{Evaluate model support}
\usage{
modelsupport(models)
}
\arguments{
\item{models}{a \code{regions_modelselect} object; the output of a call to \code{\link[=modelselect]{modelselect()}}.}
}
\value{
A \code{regions_modelsupport} object, which contains the best model for each number of regions as determined by the AICc and BIC. The computed statistics are \code{AICc}/\code{BIC}--the value of the information criterion (IC) for each model, \code{deltaAIC}/\code{deltaBIC}--the difference between the IC for the corresponding model and that of the model with the lowest IC value, \code{model_lik}--the likelihood ratio of the model against the model with the lowest IC value, and \code{Ak_weight}/\code{BIC_weight}--the Akaike weights for each model used to compute the region score. The region score is a weighted average of the numbers of regions, weighted by the Akaike weights to represent the variability around the optimal number of regions.
}
\description{
\code{modelsupport()} computes measures of the relative support of each of the best models identified by \code{\link[=modelselect]{modelselect()}} to facilitate selecting the optimal number and position of regions. These measures are in the form of information criteria (AICc and BIC).
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Fit segmented regression models for 1 to 7 regions
# using PCOs 1 to 4 and a continuous model with a
# non-exhaustive search
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 7,
                             minvert = 3,
                             cont = TRUE,
                             exhaus = FALSE,
                             verbose = FALSE)

regionresults

# For each number of regions, identify best
# model based on minimizing RSS
bestresults <- modelselect(regionresults)
bestresults

# Evaluate support for each model and rank models
supp <- modelsupport(bestresults)
supp

# 5 regions best based on AICc; 6 regions based on BIC
}
\seealso{
\code{\link[=modelselect]{modelselect()}}, \code{\link[=calcregions]{calcregions()}}, \code{\link[=calcBPvar]{calcBPvar()}}, \code{\link[=modelperf]{modelperf()}}, \code{\link[=plotsegreg]{plotsegreg()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simregions.R
\name{simregions}
\alias{simregions}
\alias{plot.regions_sim}
\title{Simulate regions data}
\usage{
simregions(
  nvert,
  nregions,
  nvar = 1,
  r2 = 0.95,
  minvert = 3,
  cont = TRUE,
  sl.dif = 0
)

\method{plot}{regions_sim}(x, scores = 1, lines = TRUE, ...)
}
\arguments{
\item{nvert}{\code{numeric}; the number of vertebrae for which to simulate data.}

\item{nregions}{\code{numeric}; the desired number of regions in the simulated data.}

\item{nvar}{\code{numeric}; the number of PCO axes to simulate. Default is 1.}

\item{r2}{\code{numeric}; a vector containing the \eqn{R^2} of the true segmented regression model for each simulated PCO. If a single value is supplied, all PCOs will receive that value. Otherwise, one value should be supplied for each simulated PCO.}

\item{minvert}{\code{numeric}; the minimum number of vertebrae allowed in each region. Default is 3.}

\item{cont}{\code{logical}; whether to use models that are continuous (\code{TRUE}) or discontinuous (\code{FALSE}) at the breakpoints to generate the data. Default is \code{TRUE}.}

\item{sl.dif}{\code{numeric}; the minimum required difference in slopes between adjacent regions, expressed as a proportion of the maximal difference between allowable slopes. Must be between 0 and 1. See Details.}

\item{x}{a \code{regions_sim} object.}

\item{scores}{\code{numeric}; for which simulated PCO scores the simulated values should be plotted.}

\item{lines}{\code{logical}; whether to display the simulated regression lines on the plot. Default is \code{TRUE}.}

\item{\dots}{ignored.}
}
\value{
\code{simregions()} returns a \code{regions_sim} object, which contains the vertebra indices in the \code{Xvar} entry, the PCO scores in the \code{Yvar} entry, the simulated breakpoints in the \code{BPs} entry, the simulated model coefficients in the \code{coefs} entry, and the simulated error standard deviation in the \code{ersd} entry. The attribute \code{"design"} contains the design matrix, which when multiplied by the coefficients and added to a random normal variate with standard deviation equal to the error standard deviation yields the observed PCO scores.

\code{plot()} returns a \code{ggplot} object that can be manipulated using \code{ggplot2} syntax. The plot is similar to that produced by \code{\link[=plot.regions_pco]{plot.regions_pco()}} and to that produced by \code{\link[=plotsegreg]{plotsegreg()}} except that the displayed lines (if requested) are the true rather than fitted regression lines.
}
\description{
\code{simregions()} simulates vertebrae and PCOs that satisfy certain constraints.
}
\details{
\code{simregions()} generates PCO scores for each requested vertebra such that certain conditions are met. The slopes for each region are drawn from a uniform distribution with limits of -.5 and .5. If a set of slopes contains two adjacent slopes that have a difference less than \code{sl.dif}, it is rejected and a new one is drawn. The scaling of the PCOs is determined by the slopes and the requested \eqn{R^2}. The PCOs will not necessarily be in order from most variable to least variable as they are in a traditional PCO analysis.

Intercepts (the intercept of the first region when \code{cont = TRUE} and the intercept of all regions when \code{cont = FALSE}) are drawn from a uniform distribution with limits of \eqn{-n/4} and \eqn{n/4}, where \eqn{n} is the number of breakpoints, one less than \code{nregions}. Intercepts other than the first when \code{cont = TRUE} are determined by the slopes.

The \code{cont}, \code{r2}, and \code{sl.dif} arguments control how easy it is for fitted segmented regression models to capture the true structure. When \code{cont = TRUE}, it can be harder to determine exactly where regions begin and end, especially if \code{sl.dif} is 0. When \code{r2} is high, there is little variation around the true line, so the fitted lines will be more precise and region boundaries clearer. When \code{sl.dif} is high, slopes of adjacent regions are different from each other, so it is easier to detect region boundaries. Setting \code{sl.dif} to between .5 and 1 ensures that the slopes in adjacent regions have different signs.
}
\examples{
# Simulate 40 vertebra, 4 regions (3 breakpoints), 3 PCOs,
# true model R2 of .9, continuous
set.seed(11)
sim <- simregions(nvert = 30, nregions = 4, nvar = 3, r2 = .95,
                  minvert = 3, cont = TRUE)

sim

# Plot the true data-generating lines and breakpoints
plot(sim, scores = 1:3)

# Run segmented regression models on simulated data,
# up to 6 regions
simresults <- calcregions(sim, scores = 1:3, noregions = 6,
                          minvert = 3, cont = TRUE,
                          verbose = FALSE)

summary(simresults)

# Select best model for each number of regions
(simmodels <- modelselect(simresults))

# Evaluate support for each model and rank models
(simsupp <- modelsupport(simmodels))
# AICc supports 3-4 regions

# Evaluate model performance of best model
modelperf(sim, modelsupport = simsupp,
          criterion = "aic", model = 1)
# Second best model (3 regions) does quite well, too
modelperf(sim, modelsupport = simsupp,
          criterion = "aic", model = 2)

#Plot best model fit
plotsegreg(sim, scores = 1:3,
           modelsupport = simsupp,
           criterion = "aic", model = 1)

# Calculate variability of estimate breakpoints for
# 3-region model; high uncertainty for breakpoints
# 1 and 2. Note weighted value for breakpoint 2
# differs from that of best model
bpvar <- calcBPvar(simresults, noregions = 4,
                   pct = .05, criterion = "aic")
bpvar

# Plot estimated vertebral map with variability
plotvertmap(sim, modelsupport = simsupp, model = 1,
            criterion = "aic", text = TRUE)

# True map; pretty close
plotvertmap(sim, bps = c(3, 7, 24),
            text = TRUE)
}
\seealso{
\code{\link[=calcregions]{calcregions()}} for fitting segmented regression models to the simulated data; \code{\link[=calcmodel]{calcmodel()}} for fitting a single segmented regression model to the simulated data; \code{\link[=plotsegreg]{plotsegreg()}} for plotting estimated regression lines.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcBPvar.R
\name{calcBPvar}
\alias{calcBPvar}
\title{Calculate variability of breakpoints}
\usage{
calcBPvar(regions_results, noregions, pct = 0.05, criterion = "aic")
}
\arguments{
\item{regions_results}{a \code{regions_results} object; the output of a call to \code{\link[=calcregions]{calcregions()}} or \code{\link[=addregions]{addregions()}}.}

\item{noregions}{the number of regions for which the weighted mean and standard deviation are to be computed.}

\item{pct}{the proportion of best model to keep from the original total number of possible models}

\item{criterion}{string; the criterion used to compute the weights. Allowable options include \code{"aic"} and \code{"bic"}. Abbreviations allowed.}
}
\value{
A \code{regions_BPvar} object, which has two components:
\itemize{
\item \code{WeightedBP} is a matrix containing the weighted mean and standard deviation of each breakpoint
\item \code{BestModels} is a data frame containing the models used to compute the weighted breakpoint statistics and the weights each one is given.
}
}
\description{
\code{calcBPvar()} computes an estimate of the variability of the breakpoints for a given number of regions. This involves computing the weighted mean and standard deviation of each breakpoint using Akaike weights.
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Fit segmented regression models for 1 to 7 regions
# using PCOs 1 to 4 and a continuous model with a
# exhaustive search
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 7,
                             minvert = 3,
                             cont = TRUE,
                             exhaus = TRUE,
                             verbose = FALSE)

# Compute Akaike-weighted location and SD of optimal
# breakpoints using top 10\% of models with 4 regions
calcBPvar(regionresults, noregions = 4,
          pct = .1, criterion = "aic")
}
\seealso{
\code{\link[=calcregions]{calcregions()}} for fitting segmented regression models to all combinations of breakpoints.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_input.R
\name{process_PC}
\alias{process_PC}
\title{Process vertebral measurements and PC scores of traditional morphometric data}
\usage{
process_PC(data, pos = 1L, pcscores, eigenvals, posPC)
}
\arguments{
\item{data}{a data frame containing a column of vertebra indices and measurements for each vertebra, or a named list thereof for multiple specimens with the names of the list corresponding to the unique identifiers of each specimen.}

\item{pos}{the name or index of the variable in \code{data} containing the vertebra indices. Default is to use the first column.}

\item{pcscores}{a matrix or data frame containing PC scores of each vertebra. Note that for multiple specimens, an ordination method should have been performed on the concatenated data across all specimens so that \code{pcscores} is a single data frame or matrix containing scores for all specimens.}

\item{eigenvals}{a numeric vector containing the eigenvalues of each PC axis.}

\item{posPC}{a vector containing the positional information of vertebrae in \code{pcscores}, or a named list thereof for multiple specimens with the names of the list corresponding to the unique identifiers of each specimen. If not provided, will assume the row order of \code{pcscores} matches the vertebra order in \code{pos}.}
}
\value{
A \code{regions_pco} object, which contains user-provided eigenvectors in the \code{scores} component and eigenvalues in the \code{eigen.val} component. The original dataset, including positional information, is stored in the \code{data} attribute.
}
\description{
\code{process_PC()} initializes the analysis workflow by processing a dataset of vertebral measurements and already computed PC scores into an object usable by \pkg{MorphoRegions}. Such processing includes identifying the vertebra indices, measurements, and PC scores.
}
\details{
Unlike \code{process_measurements()}, \code{process_PC()} does not fill in missing values and these should have been removed or replaced by numeric values before running \code{process_PC()}.
}
\examples{
# Load dataset; vertebral index in first column
# ('Vertebra' column)
data("dolphin")

# Compute PC scores with prcomp:
PCA <- prcomp(dolphin[-1], scale = TRUE)

# Extract PC scores and eigenvalues:
PCA_scores <- as.data.frame(PCA$x)
PCA_eigenval <- PCA$sdev^2


# Process PC scores and morphological data:
pco_dolphin <- process_PC(data = dolphin,
                          pos = "Vertebra",
                          pcscores = PCA_scores,
                          eigenvals = PCA_eigenval,
                          posPC = dolphin[[1]])
}
\seealso{
\itemize{
\item \code{\link[=process_gmPC]{process_gmPC()}} for processing PC scores from 2D or 3D geometric morphometric datasets
\item \code{\link[=svdPCO]{svdPCO()}} for computing principal coordinate axes from processed vertebra data.
\item \code{\link[=plot.regions_pco]{plot.regions_pco()}} for plotting PCO axes
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{wolves_gmm}
\alias{wolves_gmm}
\title{Geometric morphometric data and PC scores of the vertebral column two wolves}
\format{
A list containing: a 3D array of Procrustes aligned 3D landmarks (\code{coord_gpa}) of 10 subsampled vertebrae per specimen, and 30 landmarks per vertebra, a matrix of PC scores from a PCA performed on Procrustes aligned landmarks (\code{scores}),
a numeric vector with the eigenvalues from the PCA (\code{eigenvals}), a numeric vector with the position information of vertebrae (\code{vertebrae}), and a vector with the names of the specimens (\code{specimens}).
}
\usage{
data("wolves_gmm")
}
\description{
Procrustes aligned 3d landmarks (\code{coord_gpa}) and PC scores (\code{scores}) from  10 subsampled vertebrae along the backbone of two specimens of Canis lupus (MU 071 and MU 073).
}
\references{
Schwab, J., Figueirido, B., & Jones, K. E. (2026). Ecological inference from isolated vertebrae: Evaluating functional signal across the carnivoran spine. Journal of Morphology, 287(1), e70109.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsample.R
\name{subsample}
\alias{subsample}
\title{Subsample a dataset}
\usage{
subsample(pco, sample = NULL, type = "seq")
}
\arguments{
\item{pco}{a \code{regions_pco} object; the output of a call to \code{\link[=svdPCO]{svdPCO()}}.}

\item{sample}{\code{numeric}; either the number or proportion of vertebrae to remain the sampled dataset. If \code{NULL}, the original dataset is returned.}

\item{type}{string; the type of subsampling to do, either \code{"seq"} for sampling in sequence or \code{"random"} for random sampling. Default is \code{"seq"}. Abbreviations allowed.}
}
\value{
A \code{regions_pco} object, a subset of the original supplied to \code{pco}. The original dataset is stored as an attribute, which itself contains the subsampling indices.
}
\description{
\code{subsample()} creates a smaller version of the original dataset by sampling its rows. Because PCOs should be computed on the full dataset and most other functions take in \code{regions_pco} objects, \code{subsample()} requires a \code{regions_pco} object as its input.
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Plot vertebrae before subsampling
plotvertmap(alligator_PCO, dropNA = FALSE,
            text = TRUE)

# Subsample data after estimating PCOs; subsample down
# to 15 vertebrae
alligator_PCO_sample <- subsample(alligator_PCO,
                                  sample = 15)

# Plot vertebrae after subsampling; gray vertebrae
# have been dropped
plotvertmap(alligator_PCO_sample, dropNA = FALSE,
            text = TRUE)
}
\seealso{
\code{\link[=svdPCO]{svdPCO()}}, \code{\link[=process_measurements]{process_measurements()}}, \code{\link[=plotvertmap]{plotvertmap()}} to visualize the vertebral map after subsampling.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PCOselect.R
\name{PCOselect}
\alias{PCOselect}
\alias{plot.regions_pco_select}
\alias{summary.regions_pco_select}
\title{Select PCO scores}
\usage{
PCOselect(
  pco,
  method = "manual",
  scores = NULL,
  cutoff = 0.05,
  nreps = 500,
  results = NULL,
  criterion = "aic",
  verbose = interactive()
)

\method{plot}{regions_pco_select}(x, ...)

\method{summary}{regions_pco_select}(object, ...)
}
\arguments{
\item{pco}{a \code{regions_pco} object; the output of a call to \code{\link[=svdPCO]{svdPCO()}}.}

\item{method}{string; the method used to select the number of PCOs. Allowable options include \code{"manual"}, \code{"boot"}, \code{"variance"}, and \code{"max"}. Default is \code{"manual"}. Abbreviations allowed. See Details.}

\item{scores}{when \code{method = "manual"}, the number of PCO scores to use.}

\item{cutoff}{when \code{method = "variance"}, the cutoff for the variance explained by each PCO score.}

\item{nreps}{when \code{method = "boot"}, the number of bootstrap replications to use.}

\item{results}{when \code{method = "max"}, a \code{regions_results} object, the output of a call to \code{\link[=calcregions]{calcregions()}}.}

\item{criterion}{when \code{method = "max"}, which criterion should be used to select the number of scores. Allowable options include \code{"aic"} and \code{"bic"}. Abbreviations allowed.}

\item{verbose}{when \code{method = "boot"}, whether to display a progress bar. Default is \code{TRUE} when running interactively and \code{FALSE} otherwise.}

\item{x}{for \code{plot.regions_pco_select()}, a \code{regions_pco_select} object, the output of a call to \code{PCOselect()} with \code{method = "boot"} or \code{"max"}.}

\item{\dots}{ignored.}

\item{object}{a \code{regions_pco_select} object, the output of a call to \code{PCOselect()} with \code{method = "max"}.}
}
\value{
For \code{PCOselect()}, a \code{regions_pco_select} object, which is a numeric vector containing the indices of the chosen PCs, with attributes containing information about the PC scores chosen by the specified method. When \code{method = "boot"}, the bootstrap results are stored in the \code{"boot"} attribute. When \code{method = "max"}, the \code{regions_results} object passed to \code{regions} and other information about the quality of fit for each number of PCs are stored in the \code{"pcomax"} attribute.

The \code{plot()} methods each return a \code{ggplot} object that can manipulated using \pkg{ggplot2} syntax. The \code{summary()} method returns a data.frame of results.
}
\description{
\code{PCOselect()} provides several methods to select the number of principal coordinates (PCOs) analysis scores to be used in subsequent analyses.
}
\details{
Each method provides an alternate way to select the number of scores. These are described below.
\subsection{\code{method = "manual"}:}{

This simply returns the number supplied to \code{scores} after running some checks to ensure it is valid.
}

\subsection{\code{method = "boot"}}{

Bootstrapping works by comparing the eigenvalue distributions of PCOs to those with randomized data in order to extract PCO axes with significant signal, which are defined as those with eigenvalues greater than those from randomized data. The returned PCO cutoff is the largest PCO axis whose eigenvalues fall below the mean eigenvalue for that axis from the randomized data. Data are randomly sampled by row. Bootstrapping is sensitive to unequal variances of columns, so \code{scale = TRUE} should be set in the call to \code{\link[=svdPCO]{svdPCO()}}, which is the default; the data are scaled in the same way prior to bootstrapping. The \code{plot()} method displays the eigenvalues of the true PCOs and boxplots summarizing the distribution of the bootstrapped eigenvalues for each PCO. The \code{boot} method is unavailable for user-supplied PC scores (\code{regions_pco} object obtained from \code{process_PC} or \code{process_gmPC} functions).
}

\subsection{\code{method = "variance"}}{

This method works by computing the ratio of each eigenvalue to the sum of the eigenvalues (i.e., to compute the proportion of variance explained by each PC score) and select the number of scores with ratios greater than the cutoff value supplied to \code{cutoff}.
}

\subsection{\code{method = "max"}}{

This method works by selecting the smallest number of PCs that gives a region score within .001 of the maximum possible region score for the segmented models fit in the object supplied to \code{results}. Which criterion is maximized (AIC or BIC) is determined by the value supplied to \code{criterion}. The \code{summary()} method displays the region score (estimated number of regions) for each PC (\code{RSind}) and for PCs cumulatively (\code{RScum}) selected using the AICc or BIC as well as the cumulative proportion of variance explained by the PCs. The \code{plot()} method displays this information graphically, with the left y-axis displaying the region score for the PCs individually (pale blue triangles) and cumulatively (orange circles) using each of the two criteria, and the right y-axis displaying the cumulative percentage of variance explained by the PCs.
}
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Select which PCOs to use
## Manually (first 4 PCOs)
(PCOs <- PCOselect(alligator_PCO, "manual", scores = 4))

## Using variance cutoff: PCOs that explain 5\% or more
## of total PCO variance
(PCOs <- PCOselect(alligator_PCO, "variance", cutoff = .05))

## Using bootstrapping with 50 reps (more reps should
## be used in practice; default is fine)
(PCOs <- PCOselect(alligator_PCO, "boot", nreps = 50))

plot(PCOs) #plot true eigenvalues against bootstrapped

## Using PCOs that optimize region score:
regionresults <- calcregions(alligator_PCO, scores = 1:21, noregions = 7,
                             minvert = 3, cont = TRUE, exhaus = TRUE,
                             verbose = FALSE)

(PCOs <- PCOselect(alligator_PCO, "max",
                   results = regionresults,
                   criterion = "bic"))

plot(PCOs)

summary(PCOs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MorphoRegions-package.R
\docType{package}
\name{MorphoRegions-package}
\alias{MorphoRegions}
\alias{MorphoRegions-package}
\title{MorphoRegions: Analysis of Regionalization Patterns in Serially Homologous Structures}
\description{
Computes the optimal number of regions (or subdivisions) and their position in serial structures without a priori assumptions and to visualize the results. After reducing data dimensionality with the built-in function for data ordination, regions are fitted as segmented linear regressions along the serial structure. Every region boundary position and increasing number of regions are iteratively fitted and the best model (number of regions and boundary positions) is selected with an information criterion. This package expands on the previous 'regions' package (Jones et al. (2018) \doi{10.1126/science.aar3126}) with improved computation and more fitting and plotting options.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://aagillet.github.io/MorphoRegions/}
  \item \url{https://github.com/aagillet/MorphoRegions}
  \item Report bugs at \url{https://github.com/aagillet/MorphoRegions/issues}
}

}
\author{
\strong{Maintainer}: Amandine Gillet \email{gillet.aman@gmail.com} (\href{https://orcid.org/0000-0001-9763-6061}{ORCID})

Authors:
\itemize{
  \item Amandine Gillet \email{gillet.aman@gmail.com} (\href{https://orcid.org/0000-0001-9763-6061}{ORCID})
  \item Katrina Jones \email{katrina.jones@bristol.ac.uk} (\href{https://orcid.org/0000-0003-1088-0497}{ORCID})
  \item Noah Greifer \email{ngreifer@iq.harvard.edu} (\href{https://orcid.org/0000-0003-3067-7154}{ORCID})
  \item Stephanie Pierce \email{spierce@oeb.harvard.edu} (\href{https://orcid.org/0000-0003-0717-1841}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotvertmap.R
\name{plotvertmap}
\alias{plotvertmap}
\title{Plot a vertebra map}
\usage{
plotvertmap(
  x,
  type = "count",
  bps = NULL,
  modelsupport = NULL,
  criterion = "aic",
  model = 1,
  bpvar = NULL,
  bp.sd = NULL,
  sd.col = "black",
  dropNA = FALSE,
  text = FALSE,
  name = NULL,
  centraL = NULL,
  reg.lim = NULL,
  lim.col = "black",
  block.cols = NULL,
  block.lim = NULL
)
}
\arguments{
\item{x}{a \code{regions_data}, \code{regions_pco}, or \code{regions_sim} object; the output of a call to \code{\link[=process_measurements]{process_measurements()}}, \code{\link[=svdPCO]{svdPCO()}}, or \code{\link[=simregions]{simregions()}}, respectively.}

\item{type}{string; the labeling of the x-axis of the plot. Either \code{"count"} to identify the vertebra index (or absolute position when \code{centraL} is supplied) or \code{"percent"} to identify the percent vertebra count (or percent total length when \code{centraL} is supplied). Abbreviations allowed. Default is \code{"count"}.}

\item{bps}{an optional vector containing the region breakpoints. One of \code{bps}, \code{modelsupport}, or \code{bpvar} should be specified to display regions breakpoints. See Details.}

\item{modelsupport}{an optional \code{regions_modelsupport} object; the output of a call to \code{\link[=modelsupport]{modelsupport()}}. One of \code{bps}, \code{modelsupport}, or \code{bpvar} should be specified to display regions breakpoints. See Details.}

\item{criterion}{string; the criterion to use to select the best model for which breakpoints are to be displayed when \code{modelsupport} is specified. Ignored otherwise. Allowable options include \code{"aic"} to use the AICc and \code{"bic"} to use the BIC. Abbreviations allowed. Default is \code{"aic"}.}

\item{model}{\code{numeric}; from which model among the best as determined by \code{criterion} should breakpoints be selected when \code{modelsupport} is supplied. Ignored otherwise. 1 is the best model, 2 the second best, etc. Default is 1.}

\item{bpvar}{an optional \code{regions_BPvar} object; the output of a call to \code{\link[=calcBPvar]{calcBPvar()}}. One of \code{bps}, \code{modelsupport}, or \code{bpvar} should be specified to display regions breakpoints. See Details.}

\item{bp.sd}{an optional vector of the standard deviations of the breakpoints (e.g., as calculated by \code{\link[=calcBPvar]{calcBPvar()}}). When \code{bpvar} is supplied, the weighted standard deviations are used.}

\item{sd.col}{when \code{bp.sd} is specified, the color of the mark on plot indicating the standard deviations. Default is black.}

\item{dropNA}{\code{logical}; when some vertebrae are missing, e.g., due to subsampling or starting the analysis at a vertebra beyond the first, whether to remove the missing vertebrae from the plot (\code{TRUE}) or retain them and label them as missing (i.e., lacking a region) (\code{FALSE}). Default is \code{FALSE} to retain them.}

\item{text}{\code{logical}; whether to print the vertebra index on each vertebra. Default is \code{FALSE}.}

\item{name}{an optional string containing a label used on the left side of the plot.}

\item{centraL}{an optional numeric vector containing centrum length for each vertebra, which is used to change the size of the plotted vertebrae, or a string containing the name of the variable in the original dataset containing centrum length. Should be of length equal to the number of included vertebrae (i.e., the length of the original dataset). Any vertebrae with centrum length of 0 will be omitted.}

\item{reg.lim}{a vector of breakpoints indicating other region limits, e.g., anatomic regions.}

\item{lim.col}{when \code{reg.lim} is specified, the color of the lines separating the regions. Default is black.}

\item{block.cols}{when breakpoints are specified (i.e., using \code{bps}, \code{modelsupport}, or \code{bpvar}) and \code{block.lim} is not specified, a vector of color names or hex codes, one for each region. If not specified, \code{\link[RColorBrewer:brewer.pal]{RColorBrewer::brewer.pal()}} with \code{name = "paired"} will be used to generate colors. When \code{block.lim} is specified, a named list of vectors of color names or hex codes. See Details.}

\item{block.lim}{a vector of breakpoints indicating the limits of traditional regions, which will be colored using \code{block.cols}. See Details.}
}
\value{
A \code{ggplot} object that can be manipulated using \code{ggplot2} syntax.
}
\description{
\code{plotvertmap()} plots a map of the supplied vertebrae, optionally adding colors, marks, and text to identify existing and estimated features of the vertebrae.
}
\details{
\code{plotvertmap()} uses \code{\link[ggplot2:geom_rect]{ggplot2::geom_rect()}} to create the plot. The plots are best viewed with a short height and a long width.
\subsection{Specifying breakpoints:}{

There are three ways to specify regions in \code{plotvertmap()}. First is to supply the vector of breakpoints directly to \code{bps}. Second is to supply a \code{regions_modelsupport} object to \code{modelsupport}. When supplied, the \code{criterion} and \code{model} arguments can be used to select which of the sets of breakpoints in the object is to be used. \code{model} selects which breakpoint model is to be used (1 for first best, 2 for second best, etc.), and \code{criterion} determines which criterion (AICc or BIC) is used to rank the models. Third is to supply \code{ regions_BPvar} object to \code{bpvar}. The weighted average breakpoints will be used after rounding (e.g., a weighted average breakpoint of 3.3 will place vertebrae 1, 2, and 3 in a region, and a weighted average breakpoint of 3.9 will place vertebrae 1, 2, 3, and 4 in a region).
}

\subsection{Using \code{block.cols}:}{

When \code{block.lim} is specified, \code{block.cols} must be specified as a list of vectors of colors, with an entry for each "block". Blocks are predefined regions separate from those specified using the above arguments, e.g., traditional regions. For each region, the most common block is found and assigned to that region. A color of that block as supplied in \code{block.cols} is used to color that region. So, each block needs as many colors as there are regions assigned to it. For example, if regions 1 and 2 are both assigned to block 1 (i.e., because block 1 is the most common block in those regions), the entry in \code{block.cols} for that block must have (at least) 2 colors. If an incorrect number of colors per block is supplied, an error will be thrown identifying which blocks are lacking colors. See Examples.
}
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Plot vertebral map with specified breakpoints
plotvertmap(alligator_PCO,
            type = "percent",
            name = "Alligator",
            bps = c(8, 15, 19),
            text = TRUE)

# Fit segmented regression models for 1 to 7 regions
# using PCOs 1 to 4 and a continuous model with a
# exhaustive search
regionresults <- calcregions(alligator_PCO,
                             scores = 1:4,
                             noregions = 7,
                             minvert = 3,
                             cont = TRUE,
                             exhaus = TRUE,
                             verbose = FALSE)

# For each number of regions, identify best
# model based on minimizing RSS
bestresults <- modelselect(regionresults)

# Evaluate support for each model and rank models
supp <- modelsupport(bestresults)

# Plot vertebral map with breakpoints corresponding to
# best segmented regression model as determined by
# AICc
plotvertmap(alligator_PCO,
            type = "percent",
            name = "Alligator",
            modelsupport = supp,
            model = 1,
            criterion = "aic",
            text = TRUE)

# Plot vertebral map with breakpoints corresponding to
# best segmented regression model as determined by
# AICc, using centrum length to size vertebrae
plotvertmap(alligator_PCO,
            name = "Alligator",
            modelsupport = supp,
            model = 1,
            criterion = "aic",
            centraL = "CL",
            text = TRUE)

# Compute Akaike-weighted location and SD of optimal
# breakpoints using top 10\% of models with 4 regions
bpvar <- calcBPvar(regionresults, noregions = 5,
                   pct = .1, criterion = "aic")

#Using weighted BPs and SDs from calcBPvar()
plotvertmap(alligator_PCO, name = "Dolphin",
            bpvar = bpvar,
            text = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{porpoise}
\alias{porpoise}
\alias{porpoise1}
\alias{porpoise2}
\alias{porpoise3}
\title{Measurements from the vertebral column of three porpoises}
\format{
Each is a data frame with 58, 56, and 59 vertebrae (respectively) and 16 measurements. Column 1, Vertebra, contains the positional information.
}
\usage{
data("porpoise")
}
\description{
Linear and angular measurements from Phocoena phocoena NRM 815072 (\code{porpoise1}), NRM 835011 (\code{porpoise2}), and NRM 855083 (\code{porpoise3}).
}
\references{
Gillet, A., Frederich, B., Pierce, S. E., & Parmentier, E. (2022). Iterative habitat transitions are associated with morphological convergence of the backbone in delphinoids. Journal of Mammalian Evolution, 29(4), 931-946.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_input.R
\name{process_measurements}
\alias{process_measurements}
\title{Process vertebral measurements}
\usage{
process_measurements(data, pos = 1L, measurements, fillNA = TRUE)
}
\arguments{
\item{data}{a data frame containing a column of vertebra indices and measurements for each vertebra, or a list thereof for multiple specimens.}

\item{pos}{the name or index of the variable in \code{data} containing the vertebra indices. Default is to use the first column.}

\item{measurements}{the names or indices of the variables in \code{data} containing the relevant vertebra measurements. If unspecified, will use all variables other than that specified in \code{pos}.}

\item{fillNA}{\code{logical}; whether to fill in missing values using a simple linear imputation. Default is \code{TRUE}. See Details.}
}
\value{
A \code{regions_data} object, which is a list of data frames (one for each specimen) with attributes containing metadata.
}
\description{
\code{process_measurements()} initializes the analysis workflow by processing a dataset of vertebra measurements into an object usable by \pkg{MorphoRegions}. Such processing includes identifying the vertebra indices and the measurements and filling in missing values.
}
\details{
Any rows with missing values for all measurements will be removed. When missing values in non-removed rows are present and \code{fillNA} is set to \code{TRUE}, \code{process_measurements()} fills them in if the sequence of missing values is no greater than 2 in length. For numeric variables, it uses a linear interpolation, and for categorical variables, it fills in the missing values with the surrounding non-missing values if they are identical and leaves them missing otherwise. Otherwise, missing values are left as they are.

When a list of data frames is supplied to \code{data}, only the variables named in \code{measurements} that are common across datasets will be stored as measurement variables.
}
\examples{
# Process dataset; vertebra index in "Vertebra" column
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Process multiple datasets; vertebra index in first column
data("porpoise")

porpoise_data <- process_measurements(list(porpoise1,
                                           porpoise2,
                                           porpoise3),
                                      pos = 1)
}
\seealso{
\code{\link[=svdPCO]{svdPCO()}} for computing principal coordinate axes from processed vertebra data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{dolphin}
\alias{dolphin}
\title{Measurements from the vertebral column of a dolphin}
\format{
A matrix with 40 vertebrae and 16 measurements. Column 1, vertebra, is the positional
information.
}
\usage{
dolphin
}
\description{
Linear and angular measurements from Platanista gangetica SMNS 45653
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PCOload.R
\name{PCOload}
\alias{PCOload}
\alias{plot.regions_pco_load}
\title{Calculate PCO loadings}
\usage{
PCOload(x, scores)

\method{plot}{regions_pco_load}(x, ...)
}
\arguments{
\item{x}{for \code{PCOload()}, a \code{regions_pco} object; the output of a call to \code{\link[=svdPCO]{svdPCO()}}. For \code{plot()}, a \code{regions_pco_load} object.}

\item{scores}{a numeric vector containing the indices of the desired scores.}

\item{\dots}{ignored.}
}
\value{
\code{PCOload()} returns a \code{regions_pco_load} object, which is a matrix with a column for each PCO score requested and a row for each variable in the original dataset; values indicate the correlation between each variable and each PCO score. \code{plot()} returns a \code{ggplot} object, which can be manipulated using \pkg{ggplot2} syntax, that displays the loadings visually.
}
\description{
\code{PCOload()} computes the loadings for each principal coordinates (PCOs) analysis score, which are the correlations between the features used to compute the PCOs and the PCOs.
}
\details{
The loadings for a constructed variable, \code{vert.size}, are also computed and displayed. This is computed as the mean of the features for each vertebra. This function is not suitable for geometric morphometric data.
}
\examples{
data("alligator")

alligator_data <- process_measurements(alligator,
                                       pos = "Vertebra")

# Compute PCOs
alligator_PCO <- svdPCO(alligator_data)

# Compute PCO loadings
loadings <- PCOload(alligator_PCO, scores = 1:4)
loadings

# Plot loadings
plot(loadings)
}
\seealso{
\code{\link[=svdPCO]{svdPCO()}} for computing the PCOs; \code{\link[=plot.regions_pco]{plot.regions_pco()}} for visualizing the correlations between PCO scores.
}
